@@ -6,7 +6,7 @@
  * Descr.: mathematical predicate management - C part                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -69,7 +69,7 @@
 void
 Pl_Fd_Set_Full_Ac_Flag_1(WamWord full_ac_word)
 {
-  pl_full_ac = Pl_Rd_Integer(full_ac_word);
+  pl_full_ac = Pl_Rd_C_Int(full_ac_word);
 }
 
 
