@@ -6,7 +6,7 @@
  * Descr.: FD variable values management - C part                          *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -78,7 +78,7 @@
  * Type Definitions                *
  *---------------------------------*/
 
-typedef Bool (*CmpFct) ();
+typedef Bool (*CmpFct) (WamWord *last_fdv_adr, WamWord *new_fdv_adr);
 
 
 
@@ -308,12 +308,12 @@ Select_Value(WamWord *fdv_adr, int value_method)
 
     case METHOD_BISECT:
     case METHOD_MIDDLE:
-      n = Nb_Elem(fdv_adr) / 2;			   /* here nb_elem > 1 => n >= 1 */
+      n = (int) Nb_Elem(fdv_adr) / 2;		   /* here nb_elem > 1 => n >= 1 */
       return Pl_Range_Ith_Elem(Range(fdv_adr), n); /* Ith is in 1..nb_elem */
-      
+
     case METHOD_RANDOM_V:
-      n = Nb_Elem(fdv_adr);
-      n = Pl_M_Random_Integer(n);		       /* random returns in 0..nb_elem-1 */
+      n = (int) Nb_Elem(fdv_adr);
+      n = (int) Pl_M_Random_Integer(n);		       /* random returns in 0..nb_elem-1 */
       return Pl_Range_Ith_Elem(Range(fdv_adr), n + 1); /* Ith is in 1..nb_elem */
     }
 
@@ -335,7 +335,7 @@ Pl_Indomain_2(WamWord x_word, WamWord method_word)
   int value_method;
   int value;
 
-  value_method = Pl_Rd_Integer(method_word);
+  value_method = Pl_Rd_C_Int(method_word);
 
   Fd_Deref_Check_Fd_Var(x_word, word, tag_mask);
 
@@ -388,8 +388,8 @@ Pl_Indomain_Alt_0(void)
 
   fdv_adr = (WamWord *) (A(0) & ~1);
   extra_cstr = A(0) & 1;
-  value_method = A(1);
-  value = A(2);
+  value_method = (int) A(1);
+  value = (int) A(2);
 
   if (value_method == METHOD_LIMITS_MIN)
     value_method = METHOD_LIMITS_MAX;
@@ -540,7 +540,7 @@ Pl_Fd_Sel_Array_Pick_Var_4(WamWord sel_array_word, WamWord method_word,
 {
   WamWord **array;
   WamWord **p, **end;
-  CmpFct cmp_meth;
+  CmpFct cmp_meth = NULL;	/* init for the compiler */
   PlLong n;
   int i;
   WamWord *fdv_adr;
@@ -561,7 +561,7 @@ Pl_Fd_Sel_Array_Pick_Var_4(WamWord sel_array_word, WamWord method_word,
   array++;
   end = array + n;
 
-  reorder = Pl_Rd_Integer_Check(reorder_word);
+  reorder = (Bool) Pl_Rd_Integer_Check(reorder_word);
 
   switch (Pl_Rd_Integer_Check(method_word))
     {
@@ -588,7 +588,7 @@ Pl_Fd_Sel_Array_Pick_Var_4(WamWord sel_array_word, WamWord method_word,
     case METHOD_RANDOM:
       for (;;)
 	{
-	  i = Pl_M_Random_Integer(n);
+	  i = (int) Pl_M_Random_Integer(n);
 	  end--;
 	  n--;
 	  fdv_adr = array[i];
@@ -640,7 +640,7 @@ Pl_Fd_Sel_Array_Pick_Var_4(WamWord sel_array_word, WamWord method_word,
   if (n > 50 && nb_ground >= n / 2)
     {
       n = n - nb_ground;
-      Trail_MV(array - 1, n + 1);
+      Trail_MV(array - 1, (int) (n + 1));
       array[-1] = (WamWord *) n;
       for (p = q = array; n; p++)
 	{
@@ -683,8 +683,8 @@ Cmp_First_Fail(WamWord *last_fdv_adr, WamWord *new_fdv_adr)
 static Bool
 Cmp_Most_Constrained(WamWord *last_fdv_adr, WamWord *new_fdv_adr)
 {
-  int l_nb = Nb_Elem(last_fdv_adr);
-  int n_nb = Nb_Elem(new_fdv_adr);
+  int l_nb = (int) Nb_Elem(last_fdv_adr);
+  int n_nb = (int) Nb_Elem(new_fdv_adr);
 
   return n_nb < l_nb ||
     (n_nb == l_nb && Nb_Cstr(new_fdv_adr) > Nb_Cstr(last_fdv_adr));
