@@ -6,7 +6,7 @@
  * Descr.: dynamic predicate management - C part                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -63,9 +63,9 @@ static DynCInf *last_clause;
  * Function Prototypes             *
  *---------------------------------*/
 
-static Bool Clause_Alt(DynCInf *clause, WamWord *w);
+static PlLong Clause_Alt(DynCInf *clause, WamWord *w, Bool is_last);
 
-static Bool Retract_Alt(DynCInf *clause, WamWord *w);
+static PlLong Retract_Alt(DynCInf *clause, WamWord *w, Bool is_last);
 
 
 
@@ -78,12 +78,11 @@ void
 Pl_Assert_5(WamWord head_word, WamWord body_word,
 	    WamWord asserta_word, WamWord check_perm_word, WamWord pl_file_word)
 {
-  Bool asserta = Pl_Rd_Integer(asserta_word);
-  Bool check_perm = Pl_Rd_Integer(check_perm_word);
+  Bool asserta = Pl_Rd_C_Int(asserta_word);
+  Bool check_perm = Pl_Rd_C_Int(check_perm_word);
   int pl_file = Pl_Rd_Atom(pl_file_word);
 
-  last_clause = Pl_Add_Dynamic_Clause(head_word, body_word, asserta,
-				      check_perm, pl_file);
+  last_clause = Pl_Add_Dynamic_Clause(head_word, body_word, asserta, check_perm, pl_file);
 }
 
 
@@ -117,7 +116,7 @@ Pl_Clause_3(WamWord head_word, WamWord body_word, WamWord for_what_word)
       tag_mask != TAG_LST_MASK && tag_mask != TAG_STC_MASK)
     Pl_Err_Type(pl_type_callable, body_word);
 
-  for_what = Pl_Rd_Integer_Check(for_what_word);
+  for_what = Pl_Rd_C_Int_Check(for_what_word);
 
 #ifdef DEBUG
   DBGPRINTF("clause/2: arity: %d", arity);
@@ -142,7 +141,7 @@ Pl_Clause_3(WamWord head_word, WamWord body_word, WamWord for_what_word)
 			pl_permission_type_private_procedure, word);
     }
 
-  dyn = (DynPInf *) (pred->dyn);
+  dyn = pred->dyn;
   if (dyn == NULL)		/* no dynamic info */
     return FALSE;
 
@@ -152,9 +151,8 @@ Pl_Clause_3(WamWord head_word, WamWord body_word, WamWord for_what_word)
   w[0] = head_word;
   w[1] = body_word;
 
-  clause = Pl_Scan_Dynamic_Pred(-1, 0, (DynPInf *) (pred->dyn), word,
-				(ScanFct) Clause_Alt, DYN_ALT_FCT_FOR_TEST, 2,
-				w);
+  clause = Pl_Scan_Dynamic_Pred(-1, 0, pred->dyn, word,
+				Clause_Alt, DYN_ALT_FCT_FOR_TEST, 2, w);
   if (clause == NULL)
     return FALSE;
 
@@ -170,8 +168,8 @@ Pl_Clause_3(WamWord head_word, WamWord body_word, WamWord for_what_word)
  * CLAUSE_ALT                                                              *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static Bool
-Clause_Alt(DynCInf *clause, WamWord *w)
+static PlLong  /* a Bool in fact but respect type ScanFct see dynam_supp.h */
+Clause_Alt(DynCInf *clause, WamWord *w, Bool is_last)
 {
   WamWord head_word1, body_word1;
 
@@ -229,7 +227,7 @@ Pl_Retract_2(WamWord head_word, WamWord body_word)
 			pl_permission_type_static_procedure, word);
     }
 
-  dyn = (DynPInf *) (pred->dyn);
+  dyn = pred->dyn;
   if (dyn == NULL)		/* no dynamic info */
     return FALSE;
 
@@ -239,9 +237,8 @@ Pl_Retract_2(WamWord head_word, WamWord body_word)
   w[0] = head_word;
   w[1] = body_word;
 
-  clause = Pl_Scan_Dynamic_Pred(-1, 0, (DynPInf *) (pred->dyn), word,
-				(ScanFct) Retract_Alt, DYN_ALT_FCT_FOR_TEST, 2,
-				w);
+  clause = Pl_Scan_Dynamic_Pred(-1, 0, pred->dyn, word,
+				Retract_Alt, DYN_ALT_FCT_FOR_TEST, 2, w);
   if (clause == NULL)
     return FALSE;
 
@@ -261,8 +258,8 @@ Pl_Retract_2(WamWord head_word, WamWord body_word)
  * RETRACT_ALT                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-static Bool
-Retract_Alt(DynCInf *clause, WamWord *w)
+static PlLong  /* a Bool in fact but respect type ScanFct see dynam_supp.h */
+Retract_Alt(DynCInf *clause, WamWord *w, Bool is_last)
 {
   WamWord head_word1, body_word1;
 
@@ -277,6 +274,7 @@ Retract_Alt(DynCInf *clause, WamWord *w)
 
 
 
+#if 0 /* really seems unused, wait and consider destroying it */
 /*-------------------------------------------------------------------------*
  * PL_RETRACT_LAST_FOUND_0                                                 *
  *                                                                         *
@@ -291,17 +289,17 @@ Pl_Retract_Last_Found_0(void)
 
 
 /*-------------------------------------------------------------------------*
- * SETARG_Of_LAST_FOUND_2                                                  *
+ * SETARG_IN_LAST_FOUND_2                                                  *
  *                                                                         *
  * update in place the ArgNo th argument of last_clause. NewValue must be  *
  * a 1-tagged word data (atom, integer).                                   *
  *-------------------------------------------------------------------------*/
 void
-Pl_Setarg_Of_Last_Found_2(WamWord arg_no_word, WamWord new_value_word)
+Pl_Setarg_In_Last_Found_2(WamWord arg_no_word, WamWord new_value_word)
 {
   WamWord word, tag_mask;
   WamWord *adr;
-  int arg_no;
+  PlLong arg_no;
 
   arg_no = Pl_Rd_Integer(arg_no_word) - 1;
 
@@ -312,6 +310,7 @@ Pl_Setarg_Of_Last_Found_2(WamWord arg_no_word, WamWord new_value_word)
 
   Arg(adr, arg_no) = word;
 }
+#endif
 
 
 
@@ -404,7 +403,7 @@ Pl_Remove_Predicate_2(WamWord name_word, WamWord arity_word)
   int func, arity;
 
   func = Pl_Rd_Atom_Check(name_word);
-  arity = Pl_Rd_Integer_Check(arity_word);
+  arity = Pl_Rd_C_Int_Check(arity_word);
 
   Pl_Update_Dynamic_Pred(func, arity, 2, -1);
 }
