@@ -6,7 +6,7 @@
  * Descr.: parameter header file                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -38,7 +38,7 @@
 
 #define MAX_VAR_NAME_LENGTH        1024
 
-#define MAX_VAR_IN_TERM            32768
+#define MAX_VAR_IN_TERM            (1024 * 1024)
 
 #define MAX_SYS_VARS               256
 
