@@ -6,7 +6,7 @@
  * Descr.: byte-code support                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -38,6 +38,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #define OBJ_INIT Byte_Code_Initializer
 
@@ -100,6 +101,7 @@ typedef enum
   PUT_NIL,
   PUT_LIST,
   PUT_STRUCTURE,
+//  PUT_META_TERM,
 
   MATH_LOAD_X_VALUE,
   MATH_LOAD_Y_VALUE,
@@ -168,9 +170,9 @@ typedef union
   double d;
 #if WORD_SIZE == 64
   int *p;
-  PlLong l;
+  PlLong l; // why not uint64_t l;
 #endif
-  unsigned u[2];
+  uint32_t u[2];
 }
 C64To32;
 
@@ -224,14 +226,13 @@ static int BC_Arg_Func_Arity(WamWord arg_word, int *arity);
 
 WamCont Pl_BC_Emulate_Pred(int func, DynPInf *dyn);
 
-static WamCont BC_Emulate_Pred_Alt(DynCInf *clause, WamWord *w);
+static WamCont BC_Emulate_Pred_Alt(DynCInf *clause, WamWord *w, Bool is_last);
 
 static WamCont BC_Emulate_Clause(DynCInf *clause);
 
 static WamCont BC_Emulate_Byte_Code(BCWord *bc);
 
-static void Prep_Debug_Call(int func, int arity, int caller_func,
-			    int caller_arity);
+static void Prep_Debug_Call(int func, int arity, int caller_func, int caller_arity);
 
 
 
@@ -299,6 +300,7 @@ Byte_Code_Initializer(void)
   Op_In_Tbl("put_nil", PUT_NIL);
   Op_In_Tbl("put_list", PUT_LIST);
   Op_In_Tbl("put_structure", PUT_STRUCTURE);
+//  Op_In_Tbl("put_meta_term", PUT_META_TERM);
   Op_In_Tbl("math_load_value", MATH_LOAD_X_VALUE);
 
   Op_In_Tbl("unify_variable", UNIFY_X_VARIABLE);
@@ -323,10 +325,9 @@ Byte_Code_Initializer(void)
   Op_In_Tbl("cut", CUT_X);
   Op_In_Tbl("soft_cut", SOFT_CUT_X);
 
-  nb_op = p - op_tbl;
+  nb_op = (int) (p - op_tbl);
 
-  qsort(op_tbl, nb_op, sizeof(op_tbl[0]),
-	(int (*)(const void *, const void *)) Compar_Inst_Code_Op);
+  qsort(op_tbl, nb_op, sizeof(op_tbl[0]), (int (*)(const void *, const void *)) Compar_Inst_Code_Op);
 
 
   bc_nb_block = 1;
@@ -404,9 +405,9 @@ Pl_BC_Start_Pred_8(WamWord func_word, WamWord arity_word,
   PredInf *pred;
 
   func = Pl_Rd_Atom_Check(func_word);
-  arity = Pl_Rd_Integer_Check(arity_word);
+  arity = Pl_Rd_C_Int_Check(arity_word);
   pl_file = Pl_Rd_Atom_Check(pl_file_word);
-  pl_line = Pl_Rd_Integer_Check(pl_line_word);
+  pl_line = Pl_Rd_C_Int_Check(pl_line_word);
 
   if (Pl_Rd_Atom_Check(sta_dyn_word) == atom_dynamic)
     prop = MASK_PRED_DYNAMIC | MASK_PRED_PUBLIC;
@@ -476,18 +477,23 @@ Pl_BC_Start_Emit_0(void)
 void
 Pl_BC_Stop_Emit_0(void)
 {
+  int pl_byte_code_len;		/* could be declared in bc_supp.h if needed elsewhere */
   int i;
+  
+  pl_byte_code_len = (int) (bc_sp - bc);
 
-  pl_byte_len = bc_sp - bc;
+  pl_byte_code = (unsigned *) Malloc(pl_byte_code_len * sizeof(BCWord));
 
-#ifdef DEBUG
-  DBGPRINTF("byte-code size:%d\n", pl_byte_len);
+#if 0
+  memcpy(pl_byte_code, bc, pl_byte_code_len * sizeof(BCWord));
+#else
+  for (i = 0; i < pl_byte_code_len; i++)
+    pl_byte_code[i] = bc[i].word;
 #endif
 
-  pl_byte_code = (unsigned *) Malloc(pl_byte_len * sizeof(BCWord));
-
-  for (i = 0; i < pl_byte_len; i++)
-    pl_byte_code[i] = bc[i].word;
+#ifdef DEBUG
+  DBGPRINTF("byte-code size:%d\n", pl_byte_code_len);
+#endif
 }
 
 
@@ -529,7 +535,7 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
   int op;
   int size_bc;
   BCWord w;			/* code-op word */
-  unsigned w1, w2, w3;		/* additional words */
+  unsigned w1, w2, w3 = 0;	/* additional words */ /* init for the compiler */
   PlLong l;
   int nb_word;
   C64To32 cv;
@@ -540,12 +546,11 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
 
   op = Find_Inst_Code_Op(func);
 
-  size_bc = bc_sp - bc;
+  size_bc = (int) (bc_sp - bc);
   if (size_bc + 3 >= bc_nb_block * BC_BLOCK_SIZE)
     {
       bc_nb_block++;
-      bc = (BCWord *) Realloc((char *) bc,
-			      bc_nb_block * BC_BLOCK_SIZE * sizeof(BCWord));
+      bc = (BCWord *) Realloc((char *) bc, bc_nb_block * BC_BLOCK_SIZE * sizeof(BCWord));
       bc_sp = bc + size_bc;
     }
 
@@ -561,7 +566,7 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
     case PUT_Y_UNSAFE_VALUE - 1:
     case MATH_LOAD_X_VALUE:
       BC1_XY(w) = BC_Arg_X_Or_Y(*arg_adr++, &op);
-      BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
+      BC1_X0(w) = Pl_Rd_C_Int(*arg_adr);
       break;
 
     case GET_ATOM:
@@ -574,14 +579,14 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
 	  op++;
 	  nb_word = 2;
 	}
-      BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
+      BC1_X0(w) = Pl_Rd_C_Int(*arg_adr);
       break;
 
     case GET_INTEGER:
     case PUT_INTEGER:
       l = Pl_Rd_Integer(*arg_adr++);
       if (Fit_In_16bits(l))
-	BC1_Atom(w) = l;
+	BC1_Atom(w) = (int) l;
       else
 	{
 	  op++;
@@ -595,14 +600,14 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
 	  nb_word = 3;
 #endif
 	}
-      BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
+      BC1_X0(w) = Pl_Rd_C_Int(*arg_adr);
       break;
 
     case GET_FLOAT:
     case PUT_FLOAT:
       nb_word = 3;
       cv.d = Pl_Rd_Float(*arg_adr++);
-      BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
+      BC1_X0(w) = Pl_Rd_C_Int(*arg_adr);
       w1 = cv.u[0];
       w2 = cv.u[1];
       break;
@@ -611,7 +616,7 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
     case GET_LIST:
     case PUT_NIL:
     case PUT_LIST:
-      BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
+      BC1_X0(w) = Pl_Rd_C_Int(*arg_adr);
       break;
 
     case GET_STRUCTURE:
@@ -619,11 +624,18 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
       nb_word = 2;
       w1 = BC_Arg_Func_Arity(*arg_adr++, &arity);
       BC1_Arity(w) = arity;
-      BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
+      BC1_X0(w) = Pl_Rd_C_Int(*arg_adr);
       break;
-
+/*
+    case PUT_META_TERM:
+      nb_word = 2;
+      w1 = Pl_Rd_Atom(*arg_adr++);
+      BC1_X0(w) = Pl_Rd_Integer(*arg_adr++);
+      BC1_XY(w) = Pl_Rd_Integer(*arg_adr);
+      break;
+*/
     case PUT_VOID:
-      BC1_X0(w) = Pl_Rd_Integer(*arg_adr);
+      BC1_X0(w) = Pl_Rd_C_Int(*arg_adr);
       break;
 
 
@@ -650,7 +662,7 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
     case UNIFY_INTEGER:
       l = Pl_Rd_Integer(*arg_adr++);
       if (Fit_In_24bits(l))
-	BC2_Int(w) = l;
+	BC2_Int(w) = (int) l;
       else
 	{
 	  op++;
@@ -674,7 +686,7 @@ Pl_BC_Emit_Inst_1(WamWord inst_word)
 
     case UNIFY_VOID:
     case ALLOCATE:
-      BC2_Int(w) = Pl_Rd_Integer(*arg_adr);
+      BC2_Int(w) = Pl_Rd_C_Int(*arg_adr);
       break;
 
 
@@ -772,6 +784,24 @@ Pl_BC_Emit_Inst_Execute_Native(int func, int arity, PlLong *codep)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_EMIT_BC_EXECUTE_WRAPPER                                              *
+ *                                                                         *
+ * Called by pl2wam for each clause of a dynamic or multifile predicate    *
+ * Each clause has been compiled to native code (aux pred).                *
+ * We here create a call to this clause.                                   *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Emit_BC_Execute_Wrapper(int func, int arity, PlLong *codep)
+{
+  Pl_BC_Start_Emit_0();
+  Pl_BC_Emit_Inst_Execute_Native(func, arity, codep);
+  Pl_BC_Stop_Emit_0();
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * BC_ARG_X_OR_Y                                                           *
  *                                                                         *
@@ -788,7 +818,7 @@ BC_Arg_X_Or_Y(WamWord arg_word, int *op)
   if (Functor(adr) != ATOM_CHAR('x'))
     (*op)++;			/* +1 for op when Y is involved */
 
-  return Pl_Rd_Integer(Arg(adr, 0));
+  return Pl_Rd_C_Int(Arg(adr, 0));
 }
 
 
@@ -808,7 +838,7 @@ BC_Arg_Func_Arity(WamWord arg_word, int *arity)
   stc_adr = UnTag_STC(word);
 
   DEREF(Arg(stc_adr, 1), word, tag_mask);	/* arity */
-  *arity = UnTag_INT(word);
+  *arity = (int) UnTag_INT(word);
 
   DEREF(Arg(stc_adr, 0), word, tag_mask);	/* functor */
   return UnTag_ATM(word);
@@ -828,7 +858,7 @@ BC_Arg_Func_Arity(WamWord arg_word, int *arity)
  *-------------------------------------------------------------------------*/
 WamCont
 Pl_BC_Call_Terminal_Pred_3(WamWord pred_word, WamWord call_info_word,
-			WamWord first_call_word)
+			   WamWord first_call_word)
 {
   int func, arity;
   WamWord *arg_adr;
@@ -839,8 +869,7 @@ Pl_BC_Call_Terminal_Pred_3(WamWord pred_word, WamWord call_info_word,
 
   debug_call = (call_info_word & (1 << TAG_SIZE_LOW)) != 0;
 
-  if (pl_debug_call_code != NULL && debug_call &&
-      (first_call_word & (1 << TAG_SIZE_LOW)))
+  if (pl_debug_call_code != NULL && debug_call && (first_call_word & (1 << TAG_SIZE_LOW)))
     {
       A(0) = pred_word;
       A(1) = call_info_word;
@@ -864,7 +893,7 @@ Pl_BC_Call_Terminal_Pred_3(WamWord pred_word, WamWord call_info_word,
   if (pred->prop & MASK_PRED_NATIVE_CODE)	/* native code */
     return (WamCont) (pred->codep);
 
-  return Pl_BC_Emulate_Pred(func, (DynPInf *) (pred->dyn));
+  return Pl_BC_Emulate_Pred(func, pred->dyn);
 }
 
 
@@ -881,29 +910,26 @@ Pl_BC_Emulate_Pred(int func, DynPInf *dyn)
   WamCont codep;
   int arity;
 
-start:
-  if (dyn == NULL)
-    goto fail;
+  while (dyn)
+    {
+      arity = dyn->arity;
+      A(arity) = Pl_Get_Current_Choice();	/* init cut register */
+      A(arity + 1) = debug_call;
 
-  arity = dyn->arity;
-  A(arity) = Pl_Get_Current_Choice();	/* init cut register */
-  A(arity + 1) = debug_call;
+      clause = Pl_Scan_Dynamic_Pred(func, arity, dyn, A(0),
+				    (ScanFct) BC_Emulate_Pred_Alt,
+				    DYN_ALT_FCT_FOR_JUMP, arity + 2, &A(0));
+      if (clause == NULL)
+	break;			/* fail */
 
-  clause = Pl_Scan_Dynamic_Pred(func, arity, dyn, A(0),
-				(PlLong (*)()) BC_Emulate_Pred_Alt,
-				DYN_ALT_FCT_FOR_JUMP, arity + 2, &A(0));
-  if (clause == NULL)
-    goto fail;
+      codep = BC_Emulate_Clause(clause);
+      if (codep)
+	return (codep);
 
-  codep = BC_Emulate_Clause(clause);
-  if (codep)
-    return (codep);
-
-  func = glob_func;
-  dyn = glob_dyn;
-  goto start;
-
-fail:
+      func = glob_func;
+      dyn = glob_dyn;
+    }
+  /* fail */
   return ALTB(B);
 }
 
@@ -915,7 +941,7 @@ fail:
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static WamCont
-BC_Emulate_Pred_Alt(DynCInf *clause, WamWord *w)
+BC_Emulate_Pred_Alt(DynCInf *clause, WamWord *w, Bool is_last)
 {
   DynPInf *dyn;
   int arity;
@@ -928,9 +954,9 @@ BC_Emulate_Pred_Alt(DynCInf *clause, WamWord *w)
 
   do
     *adr++ = *w++;
-  while (--arity >= 0);		/* >=0 to also restore cut register */
+  while (--arity >= 0);		/* >= 0 to also restore cut register */
 
-  debug_call = *w;
+  debug_call = (Bool) *w;
 
   codep = BC_Emulate_Clause(clause);
   return (codep) ? codep : Pl_BC_Emulate_Pred(glob_func, glob_dyn);
@@ -954,7 +980,14 @@ BC_Emulate_Clause(DynCInf *clause)
 
   bc = (BCWord *) clause->byte_code;
 
-  if (bc)			/* emulated code */
+  /* Issue #12: infinite loop when debugger is active on Call ports for dynamic/multifile compiled code.
+   * Each clause of such a predicate gives rise to both an assert (suitable for interpretation) and
+   * a native code which is invoked by BC with a EXECUTE_NATIVE instruction (CALL_NATIVE exists but
+   * not yet used). See in .wam call_c to Pl_Emit_BC_Execute_Wrapper and Pl_BC_Emit_Inst_Execute_Native.
+   * To fix the issue: either do not use BC if debug is active (use interpreted code - see below)
+   * or do not call the debugger inside EXECUTE_NATIVE (and CALL_NATIVE for consistency ?)
+   */
+  if (bc && !debug_call)	/* emulated code (see above for test !debug_call) */
     return BC_Emulate_Byte_Code(bc);
 				/* interpreted code */
   Pl_Copy_Clause_To_Heap(clause, &head_word, &body_word);
@@ -963,15 +996,12 @@ BC_Emulate_Clause(DynCInf *clause)
 
   for (i = 0; i < arity; i++)	/* head unification */
     if (!Pl_Unify(A(i), *arg_adr++))
-      goto fail;
+      return ALTB(B);		/* fail */
 
   A(2) = A(arity);		/* before since pb with cut if arity <= 1 */
   A(0) = body_word;
   A(1) = Tag_INT(Call_Info(func, arity, debug_call));
   return (CodePtr) Prolog_Predicate(CALL_INTERNAL_WITH_CUT, 3);
-
-fail:
-  return ALTB(B);
 }
 
 
@@ -1185,7 +1215,15 @@ bc_loop:
       bc++;
       X(x0) = Pl_Put_Structure(func, arity);
       goto bc_loop;
-
+/*
+    case PUT_META_TERM:
+      x0 = BC1_X0(w);
+      x = BC1_XY(w);
+      module = bc->word;
+      bc++;
+      X(x) = Pl_Put_Meta_Term(module, X(x0));
+      goto bc_loop;
+*/
     case MATH_LOAD_X_VALUE:
       x0 = BC1_X0(w);
       x = BC1_XY(w);
@@ -1288,9 +1326,11 @@ bc_loop:
 
     case ALLOCATE:
       Pl_Allocate(BC2_Int(w));
+      Y(E, 0) = BCI;
       goto bc_loop;
 
     case DEALLOCATE:
+      BCI = Y(E, 0);
       Pl_Deallocate();
       goto bc_loop;
 
@@ -1326,7 +1366,7 @@ bc_loop:
       bc++;			/* useless since CP already set */
 #endif
       glob_func = func;
-      glob_dyn = (DynPInf *) (pred->dyn);
+      glob_dyn = pred->dyn;
       return NULL;		/* to then call BC_Emulate_Pred */
 
     case CALL_NATIVE:
@@ -1366,6 +1406,7 @@ bc_loop:
       bc++;
       codep = (WamCont) (cv.p);
 #endif
+      /* Issue #12: alternative fix could be to never call the debugger here */
       if (pl_debug_call_code != NULL && debug_call)
 	{
 	  Prep_Debug_Call(func, arity, 0, 0);
@@ -1378,7 +1419,7 @@ bc_loop:
 
     case FAIL:
       if (pl_debug_call_code != NULL && debug_call)
-	{			/* invoke the debugger that will then call fail/0 */
+	{			/* invoke the debugger which will then call fail/0 */
 	  Prep_Debug_Call(atom_fail, 0, 0, 0);
 	  return pl_debug_call_code;
 	}
