@@ -6,7 +6,7 @@
  * Descr.: byte-Code support - header file                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -49,16 +49,6 @@
  * Global Variables                *
  *---------------------------------*/
 
-#ifdef BC_SUPP_FILE
-
-int pl_byte_len;
-
-#else
-
-extern int pl_byte_len;
-
-#endif
-
 				/* defined as this to avoid to force the   */
 				/* inclusion of bc_supp.o if not needed    */
 				/* (dynam_supp.c uses pl_byte_code)        */
