@@ -6,7 +6,7 @@
  * Descr.: C interface support                                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -82,7 +82,7 @@
   DEREF(start_word, word, tag_mask);                                        \
   if (tag_mask != TAG_REF_MASK && tag_mask != TAG_INT_MASK)                 \
     Pl_Err_Type(pl_type_integer, word);                                     \
-  if (tag_mask == TAG_INT_MASK && UnTag_INT(word)<0)                        \
+  if (tag_mask == TAG_INT_MASK && UnTag_INT(word) < 0)                      \
     Pl_Err_Domain(pl_domain_not_less_than_zero, word)
 
 
@@ -280,6 +280,81 @@ Pl_Rd_Positive(WamWord start_word)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PLLONG_TO_C_INT                                                         *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static
+int PlLong_To_C_Int(PlLong x, WamWord start_word) {
+
+#if SIZEOF_LONG > SIZEOF_INT
+
+#define C_INT_MIN (-(1L << (8 * SIZEOF_INT - 1)))
+#define C_INT_MAX ( (1L << (8 * SIZEOF_INT - 1)) - 1)
+
+  if (x < C_INT_MIN || x > C_INT_MAX)
+    Pl_Err_Domain(pl_representation_integer_32bits, start_word);
+
+#endif
+
+  return (int) x;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_RD_C_INT_CHECK                                                       *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Pl_Rd_C_Int_Check(WamWord start_word)
+{
+  return PlLong_To_C_Int(Pl_Rd_Integer_Check(start_word), start_word);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_RD_C_INT                                                             *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Pl_Rd_C_Int(WamWord start_word)
+{
+  return (int) Pl_Rd_Integer(start_word);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_RD_C_INT_POSITIVE_CHECK                                              *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Pl_Rd_C_Int_Positive_Check(WamWord start_word)
+{
+  return PlLong_To_C_Int(Pl_Rd_Positive_Check(start_word), start_word);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_RD_C_INT_POSITIVE                                                    *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Pl_Rd_C_Int_Positive(WamWord start_word)
+{
+  return (int) Pl_Rd_Positive(start_word);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_RD_FLOAT_CHECK                                                       *
  *                                                                         *
@@ -540,13 +615,13 @@ Pl_Rd_In_Char(WamWord start_word)
 int
 Pl_Rd_Code_Check(WamWord start_word)
 {
-  int c;
+  PlLong c;
 
   c = Pl_Rd_Integer_Check(start_word);
   if (!Is_Valid_Code(c))
     Pl_Err_Representation(pl_representation_character_code);
 
-  return c;
+  return (int) c;
 }
 
 
@@ -559,7 +634,7 @@ Pl_Rd_Code_Check(WamWord start_word)
 int
 Pl_Rd_Code(WamWord start_word)
 {
-  return Pl_Rd_Integer(start_word);
+  return (int) Pl_Rd_Integer(start_word);
 }
 
 
@@ -572,13 +647,13 @@ Pl_Rd_Code(WamWord start_word)
 int
 Pl_Rd_In_Code_Check(WamWord start_word)
 {
-  int c;
+  PlLong c;
 
   c = Pl_Rd_Integer_Check(start_word);
   if (c != -1 && !Is_Valid_Code(c))
     Pl_Err_Representation(pl_representation_in_character_code);
 
-  return c;
+  return (int) c;
 }
 
 
@@ -591,7 +666,7 @@ Pl_Rd_In_Code_Check(WamWord start_word)
 int
 Pl_Rd_In_Code(WamWord start_word)
 {
-  return Pl_Rd_Integer(start_word);
+  return (int) Pl_Rd_Integer(start_word);
 }
 
 
@@ -615,7 +690,7 @@ Pl_Rd_Byte_Check(WamWord start_word)
   if (tag_mask != TAG_INT_MASK || !Is_Valid_Byte(c))
     Pl_Err_Type(pl_type_byte, word);
 
-  return c;
+  return (int) c;
 }
 
 
@@ -628,7 +703,7 @@ Pl_Rd_Byte_Check(WamWord start_word)
 int
 Pl_Rd_Byte(WamWord start_word)
 {
-  return Pl_Rd_Integer(start_word);
+  return (int) Pl_Rd_Integer(start_word);
 }
 
 
@@ -652,7 +727,7 @@ Pl_Rd_In_Byte_Check(WamWord start_word)
   if (tag_mask != TAG_INT_MASK || (c != -1 && !Is_Valid_Byte(c)))
     Pl_Err_Type(pl_type_in_byte, word);
 
-  return c;
+  return (int) c;
 }
 
 
@@ -665,7 +740,7 @@ Pl_Rd_In_Byte_Check(WamWord start_word)
 int
 Pl_Rd_In_Byte(WamWord start_word)
 {
-  return Pl_Rd_Integer(start_word);
+  return (int) Pl_Rd_Integer(start_word);
 }
 
 
@@ -762,6 +837,38 @@ Pl_Rd_Codes(WamWord start_word)
 
 
 
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_RD_CHARS_OR_CODES_CHECK                                              *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+char *
+Pl_Rd_Chars_Or_Codes_Check(WamWord start_word)
+{
+  Pl_Rd_Chars_Or_Codes_Str_Check(start_word, pl_glob_buff);
+  return pl_glob_buff;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_RD_CHARS_OR_CODES                                                    *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+char *
+Pl_Rd_Chars_Or_Codes(WamWord start_word)
+{
+  Pl_Rd_Chars_Or_Codes_Str(start_word, pl_glob_buff);
+  return pl_glob_buff;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_RD_CHARS_STR_CHECK                                                   *
  *                                                                         *
@@ -912,6 +1019,89 @@ Pl_Rd_Codes_Str(WamWord start_word, char *str)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_RD_CHARS_OR_CODES_STR_CHECK                                          *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Pl_Rd_Chars_Or_Codes_Str_Check(WamWord start_word, char *str)
+{
+  WamWord word, tag_mask;
+  WamWord save_start_word;
+  WamWord *lst_adr;
+  int n = 0;
+  Bool is_chars = TRUE;		/* only known after first element read */
+
+  save_start_word = start_word;
+
+  for (;;)
+    {
+      DEREF(start_word, word, tag_mask);
+
+      if (tag_mask == TAG_REF_MASK)
+	Pl_Err_Instantiation();
+
+      if (word == NIL_WORD)
+	break;
+
+      if (tag_mask != TAG_LST_MASK)
+	Pl_Err_Type(pl_type_list, save_start_word);
+
+      lst_adr = UnTag_LST(word);
+
+      if (n == 0)
+	is_chars = Pl_Builtin_Atom(Car(lst_adr));
+      
+      *str++ = (is_chars) ? Pl_Rd_Char_Check(Car(lst_adr)) : Pl_Rd_Code_Check(Car(lst_adr));
+      n++;
+
+      start_word = Cdr(lst_adr);
+    }
+
+  *str = '\0';
+  return n;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_RD_CHARS_OR_CODES_STR                                                *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+int
+Pl_Rd_Chars_Or_Codes_Str(WamWord start_word, char *str)
+{
+  WamWord word, tag_mask;
+  WamWord *lst_adr;
+  int n = 0;
+  Bool is_chars = TRUE;		/* only known after first element read */
+
+  for (;;)
+    {
+      DEREF(start_word, word, tag_mask);
+
+      if (word == NIL_WORD)
+	break;
+
+      lst_adr = UnTag_LST(word);
+
+      if (n == 0)
+	is_chars = Pl_Builtin_Atom(Car(lst_adr));
+      
+      *str++ = (is_chars) ? Pl_Rd_Char_Check(Car(lst_adr)) : Pl_Rd_Code_Check(Car(lst_adr));
+      n++;
+
+      start_word = Cdr(lst_adr);
+    }
+
+  *str = '\0';
+  return n;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_RD_LIST_CHECK                                                        *
  *                                                                         *
@@ -2285,7 +2475,7 @@ Pl_Mk_Float(double value)
 WamWord
 Pl_Mk_Number(double value)
 {
-  int n;
+  PlLong n;
 
   n = (PlLong) value;
 
@@ -2587,3 +2777,27 @@ Pl_Mk_Variable(void)
 {
   return Pl_Put_X_Variable();
 }
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_CHECK_EMPTY_LIST                                                     *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Check_Empty_List(WamWord start_word, WamWord full_list_word)
+{
+  WamWord word, tag_mask;
+
+  DEREF(start_word, word, tag_mask);
+
+  if (tag_mask == TAG_REF_MASK)
+    Pl_Err_Instantiation();
+
+  if (tag_mask == TAG_LST_MASK)
+    Pl_Err_Domain(pl_domain_number_of_arguments, full_list_word);
+
+  if (word != NIL_WORD)
+    Pl_Err_Type(pl_type_list, full_list_word);
+}
