@@ -6,7 +6,7 @@
  * Descr.: C interface support - header file                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -120,6 +120,10 @@ char *Pl_Rd_Codes_Check(WamWord start_word);
 
 char *Pl_Rd_Codes(WamWord start_word);
 
+char *Pl_Rd_Chars_Or_Codes_Check(WamWord start_word);
+
+char *Pl_Rd_Chars_Or_Codes(WamWord start_word);
+
 int Pl_Rd_Chars_Str_Check(WamWord start_word, char *str);
 
 int Pl_Rd_Chars_Str(WamWord start_word, char *str);
@@ -128,6 +132,10 @@ int Pl_Rd_Codes_Str_Check(WamWord start_word, char *str);
 
 int Pl_Rd_Codes_Str(WamWord start_word, char *str);
 
+int Pl_Rd_Chars_Or_Codes_Str_Check(WamWord start_word, char *str);
+
+int Pl_Rd_Chars_Or_Codes_Str(WamWord start_word, char *str);
+
 WamWord *Pl_Rd_List_Check(WamWord start_word);
 
 WamWord *Pl_Rd_List(WamWord start_word);
@@ -313,3 +321,6 @@ WamWord Pl_Mk_Compound(int func, int arity, WamWord *arg);
 WamWord Pl_Mk_Callable(int func, int arity, WamWord *arg);
 
 WamWord Pl_Mk_Variable(void);
+
+void Pl_Check_Empty_List(WamWord start_word, WamWord full_list_word);
+
