@@ -6,7 +6,7 @@
  * Descr.: file consulting - C part                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -62,46 +62,47 @@
 #if 1
 
 /*-------------------------------------------------------------------------*
- * PL_CONSULT_2                                                            *
+ * PL_CONSULT_3                                                            *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Consult_2(WamWord tmp_file_word, WamWord pl_file_word)
+Pl_Consult_1(WamWord pl2wam_args_word)
 {
-  char *tmp_file = Pl_Rd_String_Check(tmp_file_word);
-  char *pl_file = Pl_Rd_String_Check(pl_file_word);
   StmInf *pstm_o = pl_stm_tbl[pl_stm_top_level_output];
   StmInf *pstm_i = pl_stm_tbl[pl_stm_top_level_input];
   int pid;
-  FILE *f_out, *f_in;
+  FILE *f_in = M_SPAWN_REDIRECT_CREATE;
+  FILE *f_out = M_SPAWN_REDIRECT_CREATE;
   FILE **pf_in;
-  PlLong save;
   unsigned char *p = NULL;
   int status, c;
   int save_use_le_prompt;
-  char *arg[] = { "pl2wam", "-w", "--compile-msg", "--no-redef-error",
-		  "--pl-state", tmp_file, "-o", tmp_file, pl_file,
-		  NULL, NULL, NULL, NULL };  /* 3 warnings + 1 for terminal NULL */
-  int warn_i = sizeof(arg) / sizeof(arg[0]) - 4; /* the 4 NULL */
+  char *arg[2 + Pl_List_Length(pl2wam_args_word)];
+  WamWord word, tag_mask;
+  WamWord *lst_adr;
+  int n = 0;
 
+  arg[n++] = "pl2wam";
 
-#define ADD_WARN(flag, opt_str)  if (!Flag_Value(flag))  arg[warn_i++] = opt_str
+  for (;;)
+    {
+      DEREF(pl2wam_args_word, word, tag_mask);
+      if (word == NIL_WORD)
+        break;
 
-  ADD_WARN(suspicious_warning, "--no-susp-warn");
-  ADD_WARN(singleton_warning, "--no-singl-warn");
-  ADD_WARN(multifile_warning, "--no-mult-warn");
+      lst_adr = UnTag_LST(word);
 
+      arg[n++] = Pl_Rd_String(Car(lst_adr));
+      pl2wam_args_word = Cdr(lst_adr);
+    }
+  arg[n] = NULL;
 
-  save = SYS_VAR_SAY_GETC;
 #ifndef NO_USE_PIPED_STDIN_FOR_CONSULT
-  SYS_VAR_SAY_GETC = 1;
   pf_in = &f_in;
 #else
   f_in = NULL;
   pf_in = NULL;
 #endif
-  Pl_Write_Pl_State_File(tmp_file_word);
-  SYS_VAR_SAY_GETC = save;
 
   Pl_Flush_All_Streams();
   pid = Pl_M_Spawn_Redirect(arg, 0, pf_in, &f_out, &f_out);
@@ -114,8 +115,7 @@ Pl_Consult_2(WamWord tmp_file_word, WamWord pl_file_word)
   if (pid < 0)
     {
     error_pl2wam:
-      Pl_Err_System(Pl_Create_Atom("error trying to execute pl2wam "
-				"(maybe not found)"));
+      Pl_Err_System(Pl_Create_Atom("error trying to execute pl2wam (maybe not found)"));
       return FALSE;
     }
 
@@ -133,7 +133,7 @@ Pl_Consult_2(WamWord tmp_file_word, WamWord pl_file_word)
 	break;
 
 #ifndef NO_USE_PIPED_STDIN_FOR_CONSULT
-      if (c == CHAR_TO_EMIT_WHEN_CHAR)
+      if (c == CHAR_TO_EMIT_ON_PIPED_GETC)
 	{
 	  if (p == NULL)
 	    {
@@ -160,7 +160,6 @@ Pl_Consult_2(WamWord tmp_file_word, WamWord pl_file_word)
       Pl_Stream_Putc(c, pstm_o);
     }
   pl_use_le_prompt = save_use_le_prompt;
-
   if (f_in)
     fclose(f_in);
   fclose(f_out);
