@@ -6,7 +6,7 @@
  * Descr.: debugger                                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -275,6 +275,8 @@ spypoint_condition(Goal, _, _) :-
 
 
 
+:- meta_predicate(spy(:)).	% useful ?
+
 spy(Spec) :-
 	set_bip_name(spy, 1),
 	'$debug_list_of_pred'(Spec, L),
@@ -289,6 +291,8 @@ spy(_).
 
 
 
+:- meta_predicate(nospy(:)).	% useful ?
+
 nospy(Spec) :-
 	set_bip_name(nospy, 1),
 	'$debug_list_of_pred'(Spec, L),
@@ -389,12 +393,13 @@ nospyall.
 	append(L1, L2, L).
 
 '$debug_list_of_pred'(N / A1 - A2, L) :-
-	'$debug_list_of_pred1'(N, A1, A2, L), !.
+	!,
+	'$debug_list_of_pred1'(N, A1, A2, L).
 
 '$debug_list_of_pred'(N, L) :-
 	atom(N), !,
 	current_prolog_flag(max_arity, Max),
-	'$debug_list_of_pred1'(N, 0, Max, L), !.
+	'$debug_list_of_pred1'(N, 0, Max, L).
 
 '$debug_list_of_pred'(PI, L) :-
 	'$get_pred_indic'(PI, N, A),
@@ -440,7 +445,7 @@ nospyall.
 	        A2 = Max,
 	        Z = any
 	    ;   Z = A1 - A2
-	    ),
+	    ), !,
 	    format(debugger_output, 'Warning: spy ~a/~w - no matching predicate~n', [N, Z])
 	;   true
 	).
@@ -498,22 +503,22 @@ nospyall.
 	NewAncLst = [a(Goal, Invoc1, Index1, B)|OldAncLst],
 	setarg(1, DebugInfo, Invoc1),
 	setarg(2, DebugInfo, NewAncLst),
-%format('starting of call:~w',[Goal]), disp_B(''),
+%format('DBG: starting of call:~w',[Goal]), disp_B(''),
 	'$debug_call1'(Goal, CallInfo, Invoc1, Index1, NewAncLst, DebugInfo, Invoc, OldAncLst).
 
 
 
 %disp_B(Msg):-
 %'$get_current_B'(B),
-%format(' ~w B:%d (%#x)\n',[Msg, B, B]).
+%format(' ~w B:~d (~%#x)\n',[Msg, B, B]).
 
 
 '$debug_call1'(Goal, CallInfo, Invoc1, Index1, NewAncLst, DebugInfo, _, OldAncLst) :-
-%format('Goal:~w  Call Info:~w~n',[Goal,CallInfo]),
+%format('DBG: Goal:~w  Call Info:~w~n',[Goal,CallInfo]),
 	'$get_current_B'(B),
 	'$catch_internal'('$debug_call_port'(Goal, CallInfo, Invoc1, Index1, NewAncLst), Ball, '$debug_exception_port'(Goal, Invoc1, Index1, NewAncLst, Ball), 0),
 	'$get_current_B'(B1),
-%format(' after effective call: ~w B(start):%#x B1(end):%#x~n',[Goal,B,B1]),
+%format('DBG: after effective call: ~w B(start):~%#x B1(end):~%#x~n',[Goal,B,B1]),
 %disp_B('before end call'),
 	'$debug_end_call'(Goal, Invoc1, Index1, NewAncLst, DebugInfo, OldAncLst),
 %disp_B('after end call and before test determin'),
@@ -544,9 +549,9 @@ nospyall.
 	'$debug_port'(Goal, Invoc, Index, AncLst, call),
 	g_read('$debug_unify', DebugUnify),
 	(   DebugUnify == '' ->
-%format('now I call ~w~n', [Goal]),
+%format('DBG: now I call ~w~n', [Goal]),
 	    Goal \== fail,	% NB: bc_supp.c calls the debugger for 'fail/0'.
-				% but don(t call 'call_from_debugger since it is a
+				% but don't call 'call_from_debugger' since it is a
 				% control-construct (thus its native codep == NULL)
 	    '$call_from_debugger'(Goal, CallInfo)
 	;   Goal = DebugUnify
@@ -865,7 +870,7 @@ nospyall.
 	;   Indic = (+)
 	),
 	'$debug_port_pretty'(Port, Port1),
-	format(debugger_output, '~N ~a %4d %4d  ~a', [Indic, Invoc, Index, Port1]),
+	format(debugger_output, '~N ~a ~%4d ~%4d  ~a', [Indic, Invoc, Index, Port1]),
 	write_term(debugger_output, Goal, [quoted(true), max_depth(Depth)]).
 
 
