@@ -6,7 +6,7 @@
  * Descr.: debugger - C part                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -139,7 +139,7 @@ static WamWord *Read_Bank_Adr(Bool only_stack, int arg_nb, char **bank_name);
 
 static PlLong Read_An_Integer(int arg_nb);
 
-static void Print_Bank_Name_Offset(char *prefix, char *bank_name, int offset);
+static void Print_Bank_Name_Offset(char *prefix, char *bank_name, PlLong offset);
 
 static void Print_Wam_Word(WamWord *word_adr);
 
@@ -147,7 +147,7 @@ static void Modify_Wam_Word(WamWord *word_adr);
 
 static WamWord *Detect_Stack(WamWord *adr, char **stack_name);
 
-static PredInf *Detect_Pred_From_Code(CodePtr codep);
+static PredInf *Detect_Pred_From_Code(PlLong *codep);
 
 static Bool Help(void);
 
@@ -159,7 +159,7 @@ static Bool Help(void);
 #define DEBUG_CALL                 X1_2464656275675F63616C6C
 
 Prolog_Prototype(INIT_DEBUGGER, 0);
-Prolog_Prototype(DEBUG_CALL, 3);
+Prolog_Prototype(DEBUG_CALL, 2);
 
 
 
@@ -199,7 +199,7 @@ My_System_Directives(void)
 void
 Pl_Set_Debug_Call_Code_0(void)
 {
-  pl_debug_call_code = Prolog_Predicate(DEBUG_CALL, 3);
+  pl_debug_call_code = Prolog_Predicate(DEBUG_CALL, 2);
 
   Flag_Value(debug) = TRUE;
 }
@@ -241,31 +241,43 @@ Pl_Remove_One_Choice_Point_1(WamWord b_word)
 
 
 /*-------------------------------------------------------------------------*
- * PL_CHOICE_POINT_INFO_5                                                  *
+ * PL_CHOICE_POINT_INFO_4                                                  *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Choice_Point_Info_5(WamWord b_word, 
-		       WamWord module_word, WamWord func_word, WamWord arity_word,
-		       WamWord lastb_word)
+Pl_Choice_Point_Info_4(WamWord b_word, WamWord name_word, WamWord arity_word, WamWord lastb_word)
 {
   WamWord word, tag_mask;
   WamWord *b;
+  HashScan scan;
   PredInf *pred;
+  PredInf *last_pred = NULL;	/* init for the compiler */
+  PlULong code, code1;
+  PlULong last_code = 0;
+  int func, arity;
 
 
   DEREF(b_word, word, tag_mask);
   b = From_WamWord_To_B(word);
 
-  pred = Detect_Pred_From_Code(ALTB(b));
+  code = (PlULong) ALTB(b);
 
-  if (pred != NULL)	/* should not occur in practice */
+  for (pred = (PredInf *) Pl_Hash_First(pl_pred_tbl, &scan); pred;
+       pred = (PredInf *) Pl_Hash_Next(&scan))
     {
-      Pl_Get_Atom(pred->mod->module, module_word);
-      Pl_Get_Atom(Functor_Of(pred->f_n), func_word);
-      Pl_Get_Integer(Arity_Of(pred->f_n), arity_word);
+      code1 = (PlULong) (pred->codep);
+      if (code >= code1 && code1 >= last_code)
+	{
+	  last_pred = pred;
+	  last_code = code1;
+	}
     }
 
+  func = Functor_Of(last_pred->f_n);
+  arity = Arity_Of(last_pred->f_n);
+
+  Pl_Get_Atom(func, name_word);
+  Pl_Get_Integer(arity, arity_word);
   Pl_Unify(From_B_To_WamWord(BB(b)), lastb_word);
 }
 
@@ -273,27 +285,25 @@ Pl_Choice_Point_Info_5(WamWord b_word,
 
 
 /*-------------------------------------------------------------------------*
- * PL_SCAN_CHOICE_POINT_INFO_4                                             *
+ * PL_SCAN_CHOICE_POINT_INFO_3                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Scan_Choice_Point_Info_4(WamWord b_word, 
-			    WamWord module_word, WamWord func_word,
-			    WamWord arity_word)
+Pl_Scan_Choice_Point_Info_3(WamWord b_word, WamWord name_word,
+			 WamWord arity_word)
 {
   WamWord word, tag_mask;
   WamWord *b;
-  int module, func, arity;
+  int func, arity;
 
   DEREF(b_word, word, tag_mask);
   b = From_WamWord_To_B(word);
 
-  module = Pl_Scan_Choice_Point_Pred(b, &func, &arity);
-  if (module < 0)
+  func = Pl_Scan_Choice_Point_Pred(b, &arity);
+  if (func < 0)
     return FALSE;
 
-  Pl_Get_Atom(module, module_word);
-  Pl_Get_Atom(func, func_word);
+  Pl_Get_Atom(func, name_word);
   Pl_Get_Integer(arity, arity_word);
 
   return TRUE;
@@ -440,7 +450,7 @@ Find_Function(void)
   if (nb_read_arg == 0)
     return NULL;
 
-  lg = strlen(read_arg[0]);
+  lg = (int) strlen(read_arg[0]);
 
   for (i = 0; i < sizeof(cmd) / sizeof(InfCmd); i++)
     if (strncmp(cmd[i].name, read_arg[0], lg) == 0)
@@ -463,8 +473,8 @@ Write_Data_Modify(void)
 {
   WamWord *adr;
   char *bank_name;
-  int offset;
-  int nb;
+  PlLong offset;
+  PlLong nb;
   int incr = 1;
 
   if ((adr = Read_Bank_Adr(FALSE, 1, &bank_name)) != NULL)
@@ -492,9 +502,8 @@ Write_Data_Modify(void)
 	  Pl_Stream_Printf(pstm_o, ":");
 
 	  if (*read_arg[0] == 'w')
-	    Pl_Write_Term(pstm_o, -1,
-			  MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS, NULL,
-			  adr[offset]);
+	    Pl_Write_Term(pstm_o, 0, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS,
+			  NULL, adr[offset]);
 	  else
 	    {
 	      Print_Wam_Word(adr + offset);
@@ -547,12 +556,12 @@ What(void)
       return FALSE;
     }
 
-  if ((pred = Detect_Pred_From_Code((CodePtr) adr)) != NULL)
+  if ((pred = Detect_Pred_From_Code(adr)) != NULL)
     {
       func = Functor_Of(pred->f_n);
       arity = Arity_Of(pred->f_n);
       Pl_Stream_Printf(pstm_o, "%s/%d", pl_atom_tbl[func].name, arity);
-      if (adr > (PlLong *) (pred->codep))
+      if (adr > pred->codep)
 	Pl_Stream_Printf(pstm_o, "+%d", (char *) adr - (char *) (pred->codep));
       Pl_Stream_Printf(pstm_o, "\n");
       return FALSE;
@@ -574,7 +583,7 @@ static Bool
 Where(void)
 {
   char *bank_name;
-  int offset;
+  PlLong offset;
   WamWord *adr;
 
   if ((adr = Read_Bank_Adr(FALSE, 1, &bank_name)) != NULL)
@@ -603,7 +612,7 @@ Dereference(void)
 {
   char *bank_name;
   char *stack_name;
-  int offset;
+  PlLong offset;
   WamWord word, tag_mask;
   WamWord word1, *d_adr;
   WamWord *adr;
@@ -658,7 +667,7 @@ static Bool
 Environment(void)
 {
   WamWord *adr;
-  int offset;
+  PlLong offset;
   char *stack_name;
   int i;
 
@@ -698,7 +707,7 @@ static Bool
 Backtrack(void)
 {
   WamWord *adr;
-  int offset;
+  PlLong offset;
   char *stack_name;
   int i;
   PredInf *pred;
@@ -711,7 +720,7 @@ Backtrack(void)
       Detect_Stack(B, &stack_name);
       for (adr = B; adr > Local_Stack + 10; adr = BB(adr))
 	{
-	  pred = Detect_Pred_From_Code(ALTB(adr));
+	  pred = Detect_Pred_From_Code((PlLong *) ALTB(adr));
 
 	  func = Functor_Of(pred->f_n);
 	  arity = Arity_Of(pred->f_n);
@@ -720,8 +729,8 @@ Backtrack(void)
 	  if (arity == 0 && strcmp(pl_atom_tbl[func].name, "$clause_alt") == 0)
 	    {
 	      Pl_Stream_Printf(pstm_o, " for ");
-	      Pl_Write_Term(pstm_o, -1, MAX_PREC,
-			    WRITE_NUMBER_VARS | WRITE_NAME_VARS, NULL, AB(adr, 0));
+	      Pl_Write_Term(pstm_o, 0, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS,
+			    NULL, AB(adr, 0));
 	    }
 
 	  Pl_Stream_Printf(pstm_o, "\n");
@@ -746,7 +755,7 @@ Backtrack(void)
       adr += offset;
     }
 
-  pred = Detect_Pred_From_Code(ALTB(adr));
+  pred = Detect_Pred_From_Code((PlLong *) ALTB(adr));
 
   func = Functor_Of(pred->f_n);
   arity = Arity_Of(pred->f_n);
@@ -754,8 +763,8 @@ Backtrack(void)
   if (arity == 0 && strcmp(pl_atom_tbl[func].name, "$clause_alt") == 0)
     {
       Pl_Stream_Printf(pstm_o, " for ");
-      Pl_Write_Term(pstm_o, -1, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS, NULL,
-		    AB(adr, 0));
+      Pl_Write_Term(pstm_o, 0, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS,
+		    NULL, AB(adr, 0));
     }
   Pl_Stream_Printf(pstm_o, "\n");
 
@@ -785,12 +794,11 @@ Read_Bank_Adr(Bool only_stack, int arg_nb, char **bank_name)
 
   if (nb_read_arg < arg_nb + 1)
     {
-      Pl_Stream_Printf(pstm_o, "%s name expected\n", (only_stack) ? "Stack"
-		    : "Bank");
+      Pl_Stream_Printf(pstm_o, "%s name expected\n", (only_stack) ? "Stack" : "Bank");
       return NULL;
     }
 
-  lg = strlen(read_arg[arg_nb]);
+  lg = (int) strlen(read_arg[arg_nb]);
 
   if (!only_stack)
     {
@@ -828,8 +836,7 @@ Read_Bank_Adr(Bool only_stack, int arg_nb, char **bank_name)
 	return pl_stk_tbl[i].stack;
       }
 
-  Pl_Stream_Printf(pstm_o, "Incorrect %s name\n",
-		(only_stack) ? "stack" : "bank");
+  Pl_Stream_Printf(pstm_o, "Incorrect %s name\n", (only_stack) ? "stack" : "bank");
 
   return NULL;
 }
@@ -862,16 +869,16 @@ Read_An_Integer(int arg_nb)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-Print_Bank_Name_Offset(char *prefix, char *bank_name, int offset)
+Print_Bank_Name_Offset(char *prefix, char *bank_name, PlLong offset)
 {
   char str[80];
-  int lg = strlen(prefix);
+  int lg = (int) strlen(prefix);
 
   if (lg)
     Pl_Stream_Printf(pstm_o, "%s", prefix);
 
-  sprintf(str, "%s[%d]", bank_name, offset);
-  lg += strlen(str);
+  sprintf(str, "%s[%" PL_FMT_d "]", bank_name, offset);
+  lg += (int) strlen(str);
 
   if (lg > BANK_NAME_OFFSET_LENGTH)
     lg = BANK_NAME_OFFSET_LENGTH;
@@ -899,7 +906,7 @@ Print_Wam_Word(WamWord *word_adr)
   int i;
 
   Pl_Stream_Printf(pstm_o, "%#*" PL_FMT_x "  %*" PL_FMT_d "  ", HEXADECIMAL_LENGTH, (PlLong) word,
-		DECIMAL_LENGTH, (PlLong) word);
+		   DECIMAL_LENGTH, (PlLong) word);
 
   if ((adr = Detect_Stack((WamWord *) word, &stack_name)) != NULL)
     Print_Bank_Name_Offset("", stack_name, (WamWord *) word - adr);
@@ -919,21 +926,20 @@ Print_Wam_Word(WamWord *word_adr)
       case LONG_INT:
 	value = (WamWord) UnTag_Long_Int(word);
 	Pl_Stream_Printf(pstm_o, "%s,%*" PL_FMT_d, pl_tag_tbl[i].name,
-		      VALUE_PART_LENGTH, (PlLong) value);
+			 VALUE_PART_LENGTH, (PlLong) value);
 	break;
 
       case SHORT_UNS:
 	value = (WamWord) UnTag_Short_Uns(word);
-	if (tag == ATM && value >= 0 && value < pl_max_atom &&
-	    pl_atom_tbl[value].name != NULL)
-	  Pl_Stream_Printf(pstm_o, "ATM,%*s (%" PL_FMT_d,
-			VALUE_PART_LENGTH, pl_atom_tbl[value].name,
-			(PlLong) value);
+	if (tag == ATM && Is_Valid_Atom(value) && pl_atom_tbl[value].name != NULL)
+	  Pl_Stream_Printf(pstm_o, "ATM,%*s (%" PL_FMT_d ")",
+			   VALUE_PART_LENGTH, pl_atom_tbl[value].name,
+			   (PlLong) value);
 	else if (tag == ATM)
 	  tag = -1;
 	else
 	  Pl_Stream_Printf(pstm_o, "%s,%*" PL_FMT_u, pl_tag_tbl[i].name,
-			VALUE_PART_LENGTH, (PlLong) value);
+			   VALUE_PART_LENGTH, (PlULong) value);
 	break;
 
       case ADDRESS:
@@ -941,8 +947,7 @@ Print_Wam_Word(WamWord *word_adr)
 	if ((adr = Detect_Stack((WamWord *) value, &stack_name)) != NULL)
 	  {
 	    Pl_Stream_Printf(pstm_o, "%s,", pl_tag_tbl[i].name);
-	    Print_Bank_Name_Offset("", stack_name,
-				   (WamWord *) value - adr);
+	    Print_Bank_Name_Offset("", stack_name, (WamWord *) value - adr);
 	  }
 	else
 	  tag = -1;
@@ -963,7 +968,7 @@ Print_Wam_Word(WamWord *word_adr)
 
       if (tag == TFC)
 	Pl_Stream_Printf(pstm_o, "%s,%#*" PL_FMT_x, trail_tag_name[tag],
-		      VALUE_PART_LENGTH, (PlLong) value);
+			 VALUE_PART_LENGTH, (PlLong) value);
       else
 	if (tag < NB_OF_TRAIL_TAGS &&
 	    (adr = Detect_Stack((WamWord *) value, &stack_name)) != NULL &&
@@ -981,7 +986,7 @@ Print_Wam_Word(WamWord *word_adr)
   functor = Functor_Of(word);
   arity = Arity_Of(word);
 
-  if (functor >= 0 && functor < pl_max_atom && pl_atom_tbl[functor].name != NULL
+  if (Is_Valid_Atom(functor) && pl_atom_tbl[functor].name != NULL
       && arity >= 0 && arity <= MAX_ARITY)
     Pl_Stream_Printf(pstm_o, "%12s/%-3d", pl_atom_tbl[functor].name, arity);
 }
@@ -999,19 +1004,18 @@ Modify_Wam_Word(WamWord *word_adr)
   WamWord word;
   char *bank_name;
   WamWord *adr;
-  int offset;
+  PlLong offset;
   char str[80];
   char *comma;
   char *slash;
   char *p;
-  int i, j;
+  PlLong i, j;
 
   for (;;)
     {
       Pl_Stream_Printf(pstm_o, "\n");
 
-      if (Pl_Stream_Gets_Prompt("New value: ", pstm_o,
-			     str, sizeof(str), pstm_i) == NULL ||
+      if (Pl_Stream_Gets_Prompt("New value: ", pstm_o, str, sizeof(str), pstm_i) == NULL ||
 	  *str == '\0' || *str == '\n')
 	break;
 
@@ -1114,7 +1118,7 @@ Modify_Wam_Word(WamWord *word_adr)
       word = Str_To_PlLong(read_arg[0], &p, 0);
       if (*p != '\0')
 	word = (PlLong) Pl_Create_Allocate_Atom(read_arg[0]);
-      else if (word < 0 || word >= pl_max_atom)
+      else if (!Is_Valid_Atom(word))
 	goto err;
 
       *word_adr = Functor_Arity(word, i);
@@ -1157,26 +1161,25 @@ Detect_Stack(WamWord *adr, char **stack_name)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static PredInf *
-Detect_Pred_From_Code(CodePtr codep)
+Detect_Pred_From_Code(PlLong *codep)
 {
-  ModuleInf *mod;
-  HashScan scan_mod, scan;
+  HashScan scan;
   PredInf *pred;
-  CodePtr cur_codep, last_codep = 0;
   PredInf *last_pred = NULL;
+  PlLong dist = 0, d;		/* init for the compiler */
 
- for(mod = (ModuleInf *) Pl_Hash_First(pl_module_tbl, &scan_mod); mod;
-     mod = (ModuleInf *) Pl_Hash_Next(&scan_mod))
+  for (pred = (PredInf *) Pl_Hash_First(pl_pred_tbl, &scan); pred;
+       pred = (PredInf *) Pl_Hash_Next(&scan))
     {
-      for (pred = (PredInf *) Pl_Hash_First(mod->pred_tbl, &scan); pred;
-	   pred = (PredInf *) Pl_Hash_Next(&scan))
+      d = codep - pred->codep;
+
+      if ((pred->prop & MASK_PRED_DYNAMIC) || d < 0)
+	continue;
+
+      if (last_pred == NULL || d < dist)
 	{
-	  cur_codep = (pred->codep);
-	  if (last_codep <= cur_codep && cur_codep <= codep)
-	    {			/* cur_codep between last_codep and codep */
-	      last_pred = pred;
-	      last_codep = cur_codep;
-	    }
+	  last_pred = pred;
+	  dist = d;
 	}
     }
 
@@ -1203,7 +1206,7 @@ Help(void)
   L("   write     A [N] write   N (or 1) Prolog terms starting at A");
   L("   data      A [N] display N (or 1) words starting at A");
   L("   modify    A [N] display and modify N (or 1) words starting at A");
-  L("   where     A     display the real address corresponding to SA");
+  L("   where     A     display the real address corresponding to A");
   L("   what      RA    display what corresponds to the real address RA");
   L("   deref     A     display the dereferenced word starting at A");
   L("   envir     [SA]  display an environment located at SA (or current)");
