@@ -6,7 +6,7 @@
  * Descr.: DEC-10 I/O compatibility library                                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -52,10 +52,9 @@
 
 '$find_existing_stream'(File, Stream, Mode) :-
 	clause('$dec10_stream'(File, Stream, Mode), _), !,
-	(   current_stream(Stream),                 % test if it still exists
+	(   current_stream(Stream),                     % test if it still exists
 	    stream_property(Stream, file_name(File)),   % (with correct name)
-	    stream_property(Stream, Mode)                % (and correct mode)
-	                                  ->
+	    stream_property(Stream, Mode) ->            % (and correct mode)
 	    true
 	;   retract('$dec10_stream'(File, Stream, Mode)),       % no, retract
 	    fail
@@ -200,7 +199,7 @@ skip(X) :-
 
 tab(Exp) :-
 	set_bip_name(tab, 1),
-	'$arith_eval'(Exp, N),
+	'$arith_evaluate'(N, Exp), % N is Exp
 	for(_, 1, N),
 	put_char(' '),
 	fail.
