@@ -6,7 +6,7 @@
  * Descr.: Prolog errors support                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -69,8 +69,8 @@ static int c_bip_arity;
 
 
 static char *last_err_file = NULL;
-static int last_err_line;
-static int last_err_col;
+static PlLong last_err_line;
+static PlLong last_err_col;
 static char *last_err_msg;
 
 
@@ -161,6 +161,7 @@ Error_Supp_Initializer(void)
   pl_domain_flag_value = Pl_Create_Atom("flag_value");
   pl_domain_io_mode = Pl_Create_Atom("io_mode");
   pl_domain_non_empty_list = Pl_Create_Atom("non_empty_list");
+  pl_domain_number_of_arguments = Pl_Create_Atom("number_of_arguments");
   pl_domain_not_less_than_zero = Pl_Create_Atom("not_less_than_zero");
   pl_domain_operator_priority = Pl_Create_Atom("operator_priority");
   pl_domain_operator_specifier = Pl_Create_Atom("operator_specifier");
@@ -172,6 +173,7 @@ Error_Supp_Initializer(void)
   pl_domain_stream_or_alias = Pl_Create_Atom("stream_or_alias");
   pl_domain_stream_position = Pl_Create_Atom("stream_position");
   pl_domain_stream_property = Pl_Create_Atom("stream_property");
+  pl_domain_file_stream = Pl_Create_Atom("file_stream");
   pl_domain_write_option = Pl_Create_Atom("write_option");
   pl_domain_order = Pl_Create_Atom("order");
   pl_domain_term_stream_or_alias = Pl_Create_Atom("term_stream_or_alias");
@@ -181,7 +183,7 @@ Error_Supp_Initializer(void)
 				/* for g_vars */
   pl_domain_stream_seek_method = Pl_Create_Atom("stream_seek_method"); /* for seek/4 */
   pl_domain_format_control_sequence = Pl_Create_Atom("format_control_sequence");
-				/* for format/2-3 */
+  pl_domain_radix = Pl_Create_Atom("radix"); /* for format/2-3 */
   pl_domain_os_path = Pl_Create_Atom("os_path");/* for absolute_file_name/2 */
   pl_domain_os_file_permission = Pl_Create_Atom("os_file_permission");
 				/* for file_permission/2 */
@@ -227,18 +229,20 @@ Error_Supp_Initializer(void)
   pl_representation_max_arity = Pl_Create_Atom("max_arity");
   pl_representation_max_integer = Pl_Create_Atom("max_integer");
   pl_representation_min_integer = Pl_Create_Atom("min_integer");
+  pl_representation_integer_32bits = Pl_Create_Atom("integer_32bits");
   pl_representation_too_many_variables = Pl_Create_Atom("too_many_variables");
 				/* for Pl_Copy_Term(),... */
 
-  pl_evluation_float_overflow = Pl_Create_Atom("float_overflow");
-  pl_evluation_int_overflow = Pl_Create_Atom("int_overflow");
-  pl_evluation_undefined = Pl_Create_Atom("undefined");
-  pl_evluation_underflow = Pl_Create_Atom("underflow");
-  pl_evluation_zero_divisor = Pl_Create_Atom("zero_divisor");
+  pl_evaluation_float_overflow = Pl_Create_Atom("float_overflow");
+  pl_evaluation_int_overflow = Pl_Create_Atom("int_overflow");
+  pl_evaluation_undefined = Pl_Create_Atom("undefined");
+  pl_evaluation_underflow = Pl_Create_Atom("underflow");
+  pl_evaluation_zero_divisor = Pl_Create_Atom("zero_divisor");
 
 
   pl_resource_print_object_not_linked = Pl_Create_Atom("print_object_not_linked");
 				/* for print and format */
+  pl_resource_finite_memory = Pl_Create_Atom("finite_memory"); /* e.g. for length(L, L) */
   if (pl_fd_init_solver)		/* FD solver linked */
     {
       pl_resource_too_big_fd_constraint = Pl_Create_Atom("too_big_fd_constraint");
@@ -262,7 +266,7 @@ Pl_Set_Bip_Name_2(WamWord func_word, WamWord arity_word)
   arity = Pl_Rd_Integer_Check(arity_word);
   if (arity > MAX_ARITY)
     arity = -1;
-  cur_bip_arity = arity;
+  cur_bip_arity = (int) arity;
 
   Pl_Unset_C_Bip_Name();
 
@@ -395,11 +399,12 @@ Pl_Context_Error_1(WamWord err_word)
 static char *
 Context_Error_String(void)
 {
-  static char buff[256];
+  static char buff[1024];
 
-  sprintf(buff, "%s", pl_atom_tbl[cur_bip_func].name);
-  if (cur_bip_arity >= 0)
-    sprintf(buff + strlen(buff), "/%d", cur_bip_arity);
+  if (cur_bip_arity < 0)
+    return pl_atom_tbl[cur_bip_func].name;
+
+  sprintf(buff + strlen(buff), "%s/%d", pl_atom_tbl[cur_bip_func].name, cur_bip_arity);
 
   return buff;
 }
@@ -412,7 +417,7 @@ Context_Error_String(void)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Set_Last_Syntax_Error(char *file_name, int err_line, int err_col,
+Pl_Set_Last_Syntax_Error(char *file_name, PlLong err_line, PlLong err_col,
 			 char *err_msg)
 {
   last_err_file = file_name;
@@ -457,9 +462,9 @@ Pl_Syntax_Error(int flag_value)
   char str[512];
 
   if (last_err_file == NULL || *last_err_file == '\0')
-    sprintf(str, "(char:%d) %s", last_err_col, last_err_msg);
+    sprintf(str, "(char:%" PL_FMT_d ") %s", last_err_col, last_err_msg);
   else
-    sprintf(str, "%s:%d (char:%d) %s", last_err_file,
+    sprintf(str, "%s:%" PL_FMT_d " (char:%" PL_FMT_d ") %s", last_err_file,
 	    last_err_line, last_err_col, last_err_msg);
 
   if (flag_value == PF_ERR_ERROR)
@@ -468,8 +473,8 @@ Pl_Syntax_Error(int flag_value)
   Update_Cur_From_C_Bip();
   if (flag_value == PF_ERR_WARNING)
     Pl_Stream_Printf(pl_stm_tbl[pl_stm_top_level_output],
-		  "warning: syntax error: %s (from %s)\n",
-		  str, Context_Error_String());
+		     "warning: syntax error: %s (from %s)\n",
+		     str, Context_Error_String());
 }
 
 
@@ -507,9 +512,9 @@ Pl_Unknown_Pred_Error(int func, int arity)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Os_Error(int ret_val)
+Pl_Os_Error(int err_no)
 {
-  char *err_str = Pl_M_Sys_Err_String(ret_val);
+  char *err_str = Pl_M_Sys_Err_String(err_no);
 
   if (Flag_Value(os_error) == PF_ERR_ERROR)
     Pl_Err_System(Pl_Create_Allocate_Atom(err_str));
