@@ -6,7 +6,7 @@
  * Descr.: Prolog errors support - header file                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -34,6 +34,7 @@
  * the GNU Lesser General Public License along with this program.  If      *
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
+#include <errno.h>
 
 
 /*---------------------------------*
@@ -77,6 +78,7 @@ int pl_domain_close_option;
 int pl_domain_flag_value;
 int pl_domain_io_mode;
 int pl_domain_non_empty_list;
+int pl_domain_number_of_arguments;
 int pl_domain_not_less_than_zero;
 int pl_domain_operator_priority;
 int pl_domain_operator_specifier;
@@ -88,6 +90,7 @@ int pl_domain_stream_option;
 int pl_domain_stream_or_alias;
 int pl_domain_stream_position;
 int pl_domain_stream_property;
+int pl_domain_file_stream;
 int pl_domain_write_option;
 int pl_domain_order;
 int pl_domain_term_stream_or_alias;		/* for term_streams */
@@ -95,6 +98,7 @@ int pl_domain_g_array_index;			/* for g_vars */
 int pl_domain_g_argument_selector;		/* for g_vars */
 int pl_domain_stream_seek_method;		/* for seek/4 */
 int pl_domain_format_control_sequence;		/* for format/2-3 */
+int pl_domain_radix;		        	/* for format/2-3 */
 int pl_domain_os_path;				/* for absolute_file_name/2 */
 int pl_domain_os_file_permission;		/* for file_permission/2 */
 int pl_domain_selectable_item;			/* for select_read/3 */
@@ -136,16 +140,18 @@ int pl_representation_in_character_code;
 int pl_representation_max_arity;
 int pl_representation_max_integer;
 int pl_representation_min_integer;
+int pl_representation_integer_32bits;
 int pl_representation_too_many_variables;	/* for Pl_Copy_Term(),... */
 
-int pl_evluation_float_overflow;
-int pl_evluation_int_overflow;
-int pl_evluation_undefined;
-int pl_evluation_underflow;
-int pl_evluation_zero_divisor;
+int pl_evaluation_float_overflow;
+int pl_evaluation_int_overflow;
+int pl_evaluation_undefined;
+int pl_evaluation_underflow;
+int pl_evaluation_zero_divisor;
 
 
 int pl_resource_print_object_not_linked; 	/* for print and format */
+int pl_resource_finite_memory;			/* e.g. for length(L, L) */
 int pl_resource_too_big_fd_constraint; 		/* for FD */
 
 
@@ -179,6 +185,7 @@ extern int pl_domain_close_option;
 extern int pl_domain_flag_value;
 extern int pl_domain_io_mode;
 extern int pl_domain_non_empty_list;
+extern int pl_domain_number_of_arguments;
 extern int pl_domain_not_less_than_zero;
 extern int pl_domain_operator_priority;
 extern int pl_domain_operator_specifier;
@@ -190,6 +197,7 @@ extern int pl_domain_stream_option;
 extern int pl_domain_stream_or_alias;
 extern int pl_domain_stream_position;
 extern int pl_domain_stream_property;
+extern int pl_domain_file_stream;
 extern int pl_domain_write_option;
 extern int pl_domain_order;
 extern int pl_domain_term_stream_or_alias;	/* for term_streams */
@@ -197,6 +205,7 @@ extern int pl_domain_g_array_index;		/* for g_vars */
 extern int pl_domain_g_argument_selector;	/* for g_vars */
 extern int pl_domain_stream_seek_method;	/* for seek/4 */
 extern int pl_domain_format_control_sequence;	/* for format/2-3 */
+extern int pl_domain_radix;			/* for format/2-3 */
 extern int pl_domain_os_path;			/* for absolute_file_name/2 */
 extern int pl_domain_os_file_permission; 	/* for file_permission/2 */
 extern int pl_domain_selectable_item; 		/* for select_read/3 */
@@ -240,17 +249,18 @@ extern int pl_representation_in_character_code;
 extern int pl_representation_max_arity;
 extern int pl_representation_max_integer;
 extern int pl_representation_min_integer;
+extern int pl_representation_integer_32bits;
 extern int pl_representation_too_many_variables;/* for Pl_Copy_Term(),... */
 
 
-extern int pl_evluation_float_overflow;
-extern int pl_evluation_int_overflow;
-extern int pl_evluation_undefined;
-extern int pl_evluation_underflow;
-extern int pl_evluation_zero_divisor;
+extern int pl_evaluation_float_overflow;
+extern int pl_evaluation_int_overflow;
+extern int pl_evaluation_undefined;
+extern int pl_evaluation_underflow;
+extern int pl_evaluation_zero_divisor;
 
 
-extern int resource_too_many_open_streams; 	/* for streams */
+extern int pl_resource_finite_memory;		/* e.g. for length(L, L) */
 extern int pl_resource_print_object_not_linked; /* for print and format */
 extern int pl_resource_too_big_fd_constraint; 	/* for FD */
 
@@ -273,14 +283,14 @@ void Pl_Unset_C_Bip_Name(void);
 
 int Pl_Get_Current_Bip(int *arity);
 
-void Pl_Set_Last_Syntax_Error(char *file_name, int err_line, int err_col,
-			   char *err_msg);
+void Pl_Set_Last_Syntax_Error(char *file_name, PlLong err_line, PlLong err_col,
+			      char *err_msg);
 
 void Pl_Syntax_Error(int flag_value);
 
 void Pl_Unknown_Pred_Error(int func, int arity);
 
-void Pl_Os_Error(int ret_val);
+void Pl_Os_Error(int err_no);
 
 void Pl_Err_Instantiation(void);
 
@@ -308,7 +318,7 @@ void Pl_Err_System(int pl_atom_error);
   do {						\
     if ((tst) == NULL)				\
       {						\
-	Pl_Os_Error(-1);			\
+	Pl_Os_Error(errno);			\
 	return FALSE;				\
       }						\
   } while(0)
@@ -319,7 +329,7 @@ void Pl_Err_System(int pl_atom_error);
     int _tst = (tst);				\
     if (_tst < 0)				\
       {						\
-	Pl_Os_Error(_tst);			\
+	Pl_Os_Error(errno);			\
 	return FALSE;				\
       }						\
   } while(0)
