@@ -6,7 +6,7 @@
  * Descr.: file name management - C part                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -188,15 +188,19 @@ Pl_Prolog_File_Name_2(WamWord path1_word, WamWord path2_word)
     goto same;
 
   strcpy(pl_glob_buff, path1);
-  len = strlen(path1);
+  len = (int) strlen(path1);
 
   q = suffixes;
   do
     {
       p = q + 1;
-      if (*p == '\0')		/* no more suffixes: set default one */
+      if (*p == '\0')		/* no more suffixes */
 	{
-	  p = PROLOG_FILE_SUFFIX;
+#if 0 // activate it if we want to open an existing file without suffix (e.g. [foo] with no foo.pl but foo)
+	  pl_glob_buff[len] = '\0';
+	  if (access(pl_glob_buff, F_OK)) /* keep the file without suffix if it exists */
+#endif	    
+	    p = PROLOG_FILE_SUFFIX; /* else add the default suffix */
 	  break;
 	}
 
