@@ -6,7 +6,7 @@
  * Descr.: Prolog flag and system variable management                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -110,20 +110,6 @@ current_prolog_flag(Flag, Value) :-
 
 
 
-write_pl_state_file(File) :-
-	set_bip_name(write_pl_state_file, 1),
-	'$call_c_test'('Pl_Write_Pl_State_File'(File)).
-
-
-
-
-read_pl_state_file(File) :-
-	set_bip_name(read_pl_state_file, 1),
-	'$call_c_test'('Pl_Read_Pl_State_File'(File)).
-
-
-
-
 argument_counter(N) :-
 	set_bip_name(argument_counter, 1),
 	'$call_c_test'('Pl_Argument_Counter_1'(N)).
