@@ -6,7 +6,7 @@
  * Descr.: Prolog flag and system variable support                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -38,10 +38,10 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 
 #include "engine_pl.h"
-#include "gprolog_cst.h"
 #include "bips_pl.h"
 
 
@@ -55,7 +55,6 @@
  * Type Definitions                *
  *---------------------------------*/
 
-
 /*---------------------------------*
  * Global Variables                *
  *---------------------------------*/
@@ -63,23 +62,6 @@
 static FlagInf flag_tbl[NB_OF_FLAGS];
 static int nb_flag;
 
-static int atom_on;
-static int atom_off;
-
-/* pl_atom_error is already defined in stream_supp.[ch] */
-static int atom_warning;
-static int atom_fail;
-
-static int atom_chars;
-static int atom_codes;
-static int atom_atom;
-static int atom_chars_no_escape;
-static int atom_codes_no_escape;
-static int atom_atom_no_escape;
-
-static int atom_toward_zero;
-static int atom_down;
-
 
 
 
@@ -99,10 +81,16 @@ Prolog_Prototype(CURRENT_PROLOG_FLAG_ALT, 0);
  *                                                                         *
  * Since 1.4.4 Prolog flags are handled in a more general way in order to  *
  * simplify the addition of new flags.                                     *
+ * Since 1.6.0 it has been further generalized with the introduction of    *
+ * flags whose value is defined by a set (table) of possible atoms.        *
+ * The associated value is the index in the table.                         *
+ * This simplified many types now reduced as a table of atoms: false/true, *
+ * on/off, error/warning/fail, chars/codes/atom/...                        *
+ *                                                                         *
  * A flag has the following attributes (see flag_supp.h) :                 *
- *   - a name (string -> atom),                                            *
- *   - is modifiable or not (ro),                                          *
- *   - a type (several predefined types + 'any' for customization)         *
+ *   - a name (string -> atom)                                             *
+ *   - is modifiable or not (ro/rw)                                        *
+ *   - a type (3 predefined types + 'any' for customization)               *
  *   - a value (a PlLong) used for predefined types and available for 'any'*
  *   - a function get: used by current_prolog_flag                         *
  *   - a function chk: used by set_prolog_flag (even if not modifiable)    *
@@ -112,51 +100,14 @@ Prolog_Prototype(CURRENT_PROLOG_FLAG_ALT, 0);
  * compatible with the flag even if the flag is read-only. If the function *
  * fails a domain_error is raised (if it succeeds and the flag is read-only*
  * the a permission_error will be raised).                                 *
+ * This function is not used by current_prolog_flag (as requiered in ISO). *
  *                                                                         *
- * For some flags it is not clear what to test. E.g. for version_data we   *
+ * For some flags it is not clear what to check, e.g. for version_data we  *
  * currently check the value is a struct gprolog/4 (args are not checked). *
  * This function can be omitted (in that case no domain_error is checked). *
- *                                                                         *
- * NB: this function is not used by current_prolog_flag (see ISO).         *
  *-------------------------------------------------------------------------*/
 
 
-/*-------------------------------------------------------------------------*
- * INIT_FLAG_SUPP                                                          *
- *                                                                         *
- * no declared as other initializers, since we must be sure it has been    *
- * initialized before others (in particular before flag_c.c initializer).  *
- *-------------------------------------------------------------------------*/
-static void
-Init_Flag_Supp(void)
-{
-  static Bool initialized = FALSE;
-
-  if (initialized)
-    return;
-
-  initialized = TRUE;
-
-  atom_toward_zero = Pl_Create_Atom("toward_zero");
-  atom_down = Pl_Create_Atom("down");
-
-  atom_on = Pl_Create_Atom("on");
-  atom_off = Pl_Create_Atom("off");
-
-  atom_warning = Pl_Create_Atom("warning");
-  atom_fail = Pl_Create_Atom("fail");
-
-  atom_chars = Pl_Create_Atom("chars");
-  atom_codes = Pl_Create_Atom("codes");
-  atom_atom = Pl_Create_Atom("atom");
-  atom_chars_no_escape = Pl_Create_Atom("chars_no_escape");
-  atom_codes_no_escape = Pl_Create_Atom("codes_no_escape");
-  atom_atom_no_escape = Pl_Create_Atom("atom_no_escape");
-}
-
-
-
-
 /*-------------------------------------------------------------------------*
  * FCT_GET_xxx FCT_CHK_xxx FCT_SET_xxx                                     *
  *                                                                         *
@@ -206,197 +157,38 @@ Fct_Set_Atom(FlagInf *flag, WamWord value_word)
 
 
 static WamWord
-Fct_Get_Round(FlagInf *flag)
+Fct_Get_Atom_Tbl(FlagInf *flag)
 {
-  return flag->value == PF_ROUND_ZERO ? Tag_ATM(atom_toward_zero) : Tag_ATM(atom_down);
+  return Tag_ATM(flag->tbl_atom[flag->value]);
 }
 
 static Bool
-Fct_Chk_Round(FlagInf *flag, WamWord tag_mask, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-
-  return tag_mask == TAG_ATM_MASK && 
-    (atom == atom_toward_zero || atom == atom_down);
-}
-
-static Bool
-Fct_Set_Round(FlagInf *flag, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-  flag->value = (atom == atom_toward_zero) ? PF_ROUND_ZERO : PF_ROUND_DOWN;
-  return TRUE;
-}
-
-
-
-
-static WamWord
-Fct_Get_Bool(FlagInf *flag)
-{
-  return flag->value ? Tag_ATM(pl_atom_true) : Tag_ATM(pl_atom_false);
-}
-
-static Bool
-Fct_Chk_Bool(FlagInf *flag, WamWord tag_mask, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-
-  return tag_mask == TAG_ATM_MASK &&
-    (atom == pl_atom_true || atom == pl_atom_false);
-}
-
-static Bool
-Fct_Set_Bool(FlagInf *flag, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-  flag->value = (atom == pl_atom_true);
-  return TRUE;
-}
-
-
-
-
-static WamWord
-Fct_Get_On_Off(FlagInf *flag)
-{
-  return flag->value ? Tag_ATM(atom_on) : Tag_ATM(atom_off);
-}
-
-static Bool
-Fct_Chk_On_Off(FlagInf *flag, WamWord tag_mask, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-
-  return tag_mask == TAG_ATM_MASK &&
-    (atom == atom_on || atom == atom_off);
-}
-
-static Bool
-Fct_Set_On_Off(FlagInf *flag, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-  flag->value = (atom == atom_on);
-  return TRUE;
-}
-
-
-
-
-static WamWord
-Fct_Get_Err(FlagInf *flag)
+Fct_Chk_Atom_Tbl(FlagInf *flag, WamWord tag_mask, WamWord value_word)
 {
   int atom;
+  int i;
 
-  switch (flag->value)
-    {
-    case PF_ERR_ERROR:
-      atom = pl_atom_error;
-      break;
+  if (tag_mask != TAG_ATM_MASK)
+    return FALSE;
 
-    case PF_ERR_WARNING:
-      atom = atom_warning;
-      break;
+  atom = UnTag_ATM(value_word);
+  for(i = 0; flag->tbl_atom[i] >= 0 && flag->tbl_atom[i] != atom; i++)
+    ;
 
-    case PF_ERR_FAIL:
-      atom = atom_fail;
-      break;
-    }
-
-  return Tag_ATM(atom);
+  return (flag->tbl_atom[i] == atom);
 }
 
 static Bool
-Fct_Chk_Err(FlagInf *flag, WamWord tag_mask, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-
-  return tag_mask == TAG_ATM_MASK && 
-    (atom == pl_atom_error || atom == atom_warning || atom == atom_fail);
-}
-
-static Bool
-Fct_Set_Err(FlagInf *flag, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-
-  if (atom == pl_atom_error)
-    flag->value = PF_ERR_ERROR;
-  else if (atom == atom_warning)
-    flag->value = PF_ERR_WARNING;
-  else
-    flag->value = PF_ERR_FAIL;
-
-  return TRUE;
-}
-
-
-
-
-static WamWord
-Fct_Get_Quotes(FlagInf *flag)
+Fct_Set_Atom_Tbl(FlagInf *flag, WamWord value_word)
 {
   int atom;
+  int i;
 
-  switch (flag->value)
-    {
-    case PF_QUOT_AS_CODES:
-      atom = atom_codes;
-      break;
-
-    case PF_QUOT_AS_CODES | PF_QUOT_NO_ESCAPE_MASK:
-      atom = atom_codes_no_escape;
-      break;
-
-    case PF_QUOT_AS_CHARS:
-      atom = atom_chars;
-      break;
-
-    case PF_QUOT_AS_CHARS | PF_QUOT_NO_ESCAPE_MASK:
-      atom = atom_chars_no_escape;
-      break;
-
-    case PF_QUOT_AS_ATOM:
-      atom = atom_atom;
-      break;
-
-    case PF_QUOT_AS_ATOM | PF_QUOT_NO_ESCAPE_MASK:
-      atom = atom_atom_no_escape;
-      break;
-    }
-
-  return Tag_ATM(atom);
-}
-
-static Bool
-Fct_Chk_Quotes(FlagInf *flag, WamWord tag_mask, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-
-  return tag_mask == TAG_ATM_MASK && 
-    (atom == atom_codes || atom == atom_codes_no_escape ||
-     atom == atom_chars || atom == atom_chars_no_escape ||
-     atom == atom_atom  || atom == atom_atom_no_escape);
-}
-
-static Bool
-Fct_Set_Quotes(FlagInf *flag, WamWord value_word)
-{
-  int atom = UnTag_ATM(value_word);
-
-  if (atom == atom_codes)
-    flag->value = PF_QUOT_AS_CODES;
-  else if (atom == atom_codes_no_escape)
-    flag->value = PF_QUOT_AS_CODES | PF_QUOT_NO_ESCAPE_MASK;
-  else if (atom == atom_chars)
-    flag->value = PF_QUOT_AS_CHARS;
-  else if (atom == atom_chars_no_escape)
-    flag->value = PF_QUOT_AS_CHARS | PF_QUOT_NO_ESCAPE_MASK;
-  else if (atom == atom_atom)
-    flag->value = PF_QUOT_AS_ATOM;
-  else
-    flag->value = PF_QUOT_AS_ATOM | PF_QUOT_NO_ESCAPE_MASK;
+  atom = UnTag_ATM(value_word);
+  for(i = 0; flag->tbl_atom[i] != atom; i++)
+    ;
 
+  flag->value = i;
   return TRUE;
 }
 
@@ -409,13 +201,11 @@ Fct_Set_Quotes(FlagInf *flag, WamWord value_word)
  *-------------------------------------------------------------------------*/
 FlagInf *
 Pl_New_Prolog_Flag(char *name, Bool modifiable, FlagType type, PlLong value,
-		   FlagFctGet fct_get, FlagFctChk fct_chk, FlagFctSet fct_set)
+		   FlagFctGet fct_get, FlagFctChk fct_chk, FlagFctSet fct_set, ...)
 {
   int atom_name;
   FlagInf *flag;
-
-  Init_Flag_Supp();
-
+  
   atom_name = Pl_Create_Atom(name);
 
   if (nb_flag == NB_OF_FLAGS)
@@ -428,7 +218,7 @@ Pl_New_Prolog_Flag(char *name, Bool modifiable, FlagType type, PlLong value,
   flag->modifiable = modifiable;
   flag->type = type;
   flag->value = value;
-  
+
   flag->fct_get = fct_get;
   flag->fct_chk = fct_chk;
   flag->fct_set = fct_set;
@@ -437,32 +227,16 @@ Pl_New_Prolog_Flag(char *name, Bool modifiable, FlagType type, PlLong value,
     {
       switch(type)
 	{
-	case PF_TYPE_INTEGER: 
-	  flag->fct_get = Fct_Get_Integer; 
+	case PF_TYPE_INTEGER:
+	  flag->fct_get = Fct_Get_Integer;
 	  break;
 
-	case PF_TYPE_ATOM: 
-	  flag->fct_get = Fct_Get_Atom; 
+	case PF_TYPE_ATOM:
+	  flag->fct_get = Fct_Get_Atom;
 	  break;
 
-	case PF_TYPE_BOOL: 
-	  flag->fct_get = Fct_Get_Bool; 
-	  break;
-
-	case PF_TYPE_ON_OFF: 
-	  flag->fct_get = Fct_Get_On_Off;
-	  break;
-
-	case PF_TYPE_ERR: 
-	  flag->fct_get = Fct_Get_Err;
-	  break;
-
-	case PF_TYPE_QUOTES: 
-	  flag->fct_get = Fct_Get_Quotes;
-	  break;
-
-	case PF_TYPE_ROUND: 
-	  flag->fct_get = Fct_Get_Round;
+	case PF_TYPE_ATOM_TBL:
+	  flag->fct_get = Fct_Get_Atom_Tbl;
 	  break;
 
 	case PF_TYPE_ANY:
@@ -474,35 +248,19 @@ Pl_New_Prolog_Flag(char *name, Bool modifiable, FlagType type, PlLong value,
     {
       switch(type)
 	{
-	case PF_TYPE_INTEGER: 
-	  flag->fct_chk = Fct_Chk_Integer; 
+	case PF_TYPE_INTEGER:
+	  flag->fct_chk = Fct_Chk_Integer;
 	  break;
 
-	case PF_TYPE_ATOM: 
-	  flag->fct_chk = Fct_Chk_Atom; 
+	case PF_TYPE_ATOM:
+	  flag->fct_chk = Fct_Chk_Atom;
 	  break;
 
-	case PF_TYPE_BOOL: 
-	  flag->fct_chk = Fct_Chk_Bool; 
+	case PF_TYPE_ATOM_TBL:
+	  flag->fct_chk = Fct_Chk_Atom_Tbl;
 	  break;
 
-	case PF_TYPE_ON_OFF: 
-	  flag->fct_chk = Fct_Chk_On_Off;
-	  break;
-
-	case PF_TYPE_ERR: 
-	  flag->fct_chk = Fct_Chk_Err;
-	  break;
-
-	case PF_TYPE_QUOTES: 
-	  flag->fct_chk = Fct_Chk_Quotes;
-	  break;
-
-	case PF_TYPE_ROUND: 
-	  flag->fct_chk = Fct_Chk_Round;
-	  break;
-
-	case PF_TYPE_ANY: 
+	case PF_TYPE_ANY:
 	  break;	/* should not occur (but acceptable) */
 	}
     }
@@ -519,31 +277,34 @@ Pl_New_Prolog_Flag(char *name, Bool modifiable, FlagType type, PlLong value,
 	  flag->fct_set = Fct_Set_Atom;
 	  break;
 
-	case PF_TYPE_BOOL:
-	  flag->fct_set = Fct_Set_Bool;
+	case PF_TYPE_ATOM_TBL:
+	  flag->fct_set = Fct_Set_Atom_Tbl;
 	  break;
 
-	case PF_TYPE_ON_OFF:
-	  flag->fct_set = Fct_Set_On_Off;
-	  break;
-
-	case PF_TYPE_ERR: 
-	  flag->fct_set = Fct_Set_Err;
-	  break;
-
-	case PF_TYPE_QUOTES: 
-	  flag->fct_set = Fct_Set_Quotes;
-	  break;
-
-	case PF_TYPE_ROUND:
-	  flag->fct_set = Fct_Set_Round;
-	  break;
-
-	case PF_TYPE_ANY: 
+	case PF_TYPE_ANY:
 	  break;	/* should not occur */
 	}
     }
 
+  if (flag->type == PF_TYPE_ATOM_TBL)
+    {
+      va_list ap, ap2;
+      int nb_elem, i;
+      
+      va_start(ap, fct_set);
+      va_copy(ap2, ap);		/* for 2nd traversal or variadic args */
+      nb_elem = 0;
+      do
+	nb_elem++;
+      while(va_arg(ap, int) >= 0); /* count the -1 sentinel */
+      va_end(ap);
+      
+      flag->tbl_atom = Calloc(nb_elem, sizeof(int));
+      for(i = 0; i < nb_elem; i++)
+	flag->tbl_atom[i] = va_arg(ap2, int);
+      va_end(ap2);
+    }
+
   return flag;
 }
 
@@ -592,7 +353,7 @@ Pl_Set_Prolog_Flag_2(WamWord flag_word, WamWord value_word)
   if (tag_mask == TAG_REF_MASK)
     Pl_Err_Instantiation();
   value_word = word; 		/* dereferenced */
-  
+
 
   if (flag->fct_chk != NULL && !(*flag->fct_chk)(flag, tag_mask, value_word))
     {
@@ -602,7 +363,7 @@ Pl_Set_Prolog_Flag_2(WamWord flag_word, WamWord value_word)
 
       Pl_Err_Domain(pl_domain_flag_value, word);
     }
-    
+  
   if (!flag->modifiable)
     {
       Pl_Err_Permission(pl_permission_operation_modify, pl_permission_type_flag, flag_word);
@@ -671,7 +432,7 @@ Pl_Current_Prolog_Flag_Alt_0(void)
 
   flag_word = AB(B, 0);
   value_word = AB(B, 1);
-  i = AB(B, 2);
+  i = (int) AB(B, 2);
   flag = flag_tbl + i;
 
   if (i + 1 == nb_flag)
