@@ -6,7 +6,7 @@
  * Descr.: foreign interface support                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -175,7 +175,7 @@ Pl_Foreign_Jump_Ret(CodePtr codep)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 PlFIOArg *
-Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word, PlLong (*rd_fct) (),
+Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word, PlLong (*rd_fct) (WamWord word),
 		     int fio_arg_index)
 {
   WamWord word, tag_mask;
@@ -196,7 +196,7 @@ Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word, PlLong (*rd_fct) (),
 	    fa->value.s = Strdup(fa->value.s);
 	}
       else
-	fa->value.d = (*(double (*)()) rd_fct) (word);
+	fa->value.d = (*(double (*)(WamWord)) rd_fct) (word);
     }
 
   return fa;
@@ -210,7 +210,7 @@ Pl_Foreign_Rd_IO_Arg(int arg_long, WamWord start_word, PlLong (*rd_fct) (),
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Foreign_Un_IO_Arg(int arg_long, Bool (*un_fct) (), PlFIOArg *fa,
+Pl_Foreign_Un_IO_Arg(int arg_long, Bool (*un_fct) (PlLong val, WamWord word), PlFIOArg *fa,
 		     WamWord start_word)
 {
   if (!fa->unify)
@@ -219,7 +219,7 @@ Pl_Foreign_Un_IO_Arg(int arg_long, Bool (*un_fct) (), PlFIOArg *fa,
   if (arg_long)
     return (*un_fct) (fa->value.l, start_word);
 
-  return (*un_fct) (fa->value.d, start_word);
+  return (*(Bool (*)(double, WamWord)) un_fct) (fa->value.d, start_word);
 }
 
 
@@ -233,7 +233,7 @@ void
 Pl_Emit_Syntax_Error(char *file_name, int err_line, int err_col, char *err_msg)
 {
   Pl_Set_Last_Syntax_Error(file_name, err_line, err_col, err_msg);
-  Pl_Syntax_Error(Flag_Value(syntax_error));
+  Pl_Syntax_Error((int) Flag_Value(syntax_error));
 }
 
 
@@ -484,7 +484,7 @@ Pl_Type_Of_Term(WamWord start_word)
 
   DEREF(start_word, word, tag_mask);
 
-  return Tag_From_Tag_Mask(tag_mask);
+  return (int) Tag_From_Tag_Mask(tag_mask);
 }
 
 
