@@ -6,7 +6,7 @@
  * Descr.: formatted output management                                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -41,6 +41,8 @@
 '$use_format'.
 
 
+%:- meta_predicate(format(+, :)).
+
 format(Format, Args) :-
 	set_bip_name(format, 2),
 	'$call_c'('Pl_Format_2'(Format, Args)).
@@ -48,6 +50,8 @@ format(Format, Args) :-
 
 
 
+%:- meta_predicate(format(+, +, :)).
+
 format(SorA, Format, Args) :-
 	set_bip_name(format, 3),
 	'$call_c'('Pl_Format_3'(SorA, Format, Args)).
