@@ -6,7 +6,7 @@
  * Descr.: global variable (inline) management - C part                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -70,7 +70,7 @@ typedef struct gundo *PGUndo;
 
 typedef struct			/* Global variable element        */
 {				/* ------------------------------ */
-  int size;			/* <0:-array dim, 0:link, >0:copy */
+  PlLong size;			/* <0:-array dim, 0:link, >0:copy */
   WamWord val;			/* ptr to GVarElt or term or adr  */
   PGUndo undo;			/* ptr to 1st undo for this elem  */
 }
@@ -82,7 +82,7 @@ GVarElt;
 typedef struct gundo		/* Undo record                    */
 {				/* ------------------------------ */
   GVarElt *g_elem;		/* elem to restore (NULL=invalid) */
-  int save_size;		/* size to restore                */
+  PlLong save_size;		/* size to restore                */
   WamWord save_val;		/* value to restore               */
   PGUndo next;			/* chain to next undo entry       */
   PGUndo prev;			/* chain to previous undo entry   */
@@ -127,7 +127,7 @@ static void G_Assign_Arg(GVarElt *g_elem, WamWord *g_arg, WamWord word);
 static void G_Assign_Array(GVarElt *g_elem, WamWord *stc_adr, int array_op,
 			   Bool backtrack, Bool copy);
 
-static GVarElt *G_Alloc_Array(GVarElt *g_elem, int new_size, Bool backtrack);
+static GVarElt *G_Alloc_Array(GVarElt *g_elem, PlLong new_size, Bool backtrack);
 
 static GTarget *Get_Target_From_Gvar(WamWord gvar_word);
 
@@ -145,8 +145,7 @@ static void G_Free_Element(GVarElt *g_elem, Bool reinit_undo);
 
 static void G_Copy_Element(GVarElt *dst_g_elem, GVarElt *src_g_elem);
 
-static void G_Trail_For_Backtrack(GVarElt *g_elem, int save_size,
-				  WamWord save_val);
+static void G_Trail_For_Backtrack(GVarElt *g_elem, PlLong save_size, WamWord save_val);
 
 static void G_Untrail(int n, WamWord *arg_frame);
 
@@ -190,10 +189,10 @@ G_Var_Initializer(void)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void FC
-Pl_Blt_G_Assign(WamWord x, WamWord y)
+Pl_Blt_G_Assign(WamWord x_word, WamWord y_word)
 {
   Pl_Set_C_Bip_Name("g_assign", 2);
-  G_Assign(x, y, FALSE, TRUE);
+  G_Assign(x_word, y_word, FALSE, TRUE);
   Pl_Unset_C_Bip_Name();
 }
 
@@ -205,10 +204,10 @@ Pl_Blt_G_Assign(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void FC
-Pl_Blt_G_Assignb(WamWord x, WamWord y)
+Pl_Blt_G_Assignb(WamWord x_word, WamWord y_word)
 {
   Pl_Set_C_Bip_Name("g_assignb", 2);
-  G_Assign(x, y, TRUE, TRUE);
+  G_Assign(x_word, y_word, TRUE, TRUE);
   Pl_Unset_C_Bip_Name();
 }
 
@@ -220,10 +219,10 @@ Pl_Blt_G_Assignb(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void FC
-Pl_Blt_G_Link(WamWord x, WamWord y)
+Pl_Blt_G_Link(WamWord x_word, WamWord y_word)
 {
   Pl_Set_C_Bip_Name("g_link", 2);
-  G_Assign(x, y, TRUE, FALSE);
+  G_Assign(x_word, y_word, TRUE, FALSE);
   Pl_Unset_C_Bip_Name();
 }
 
@@ -235,12 +234,12 @@ Pl_Blt_G_Link(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Read(WamWord x, WamWord y)
+Pl_Blt_G_Read(WamWord x_word, WamWord y_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_read", 2);
-  res = G_Read(x, y);
+  res = G_Read(x_word, y_word);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -253,12 +252,12 @@ Pl_Blt_G_Read(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Array_Size(WamWord x, WamWord y)
+Pl_Blt_G_Array_Size(WamWord x_word, WamWord y_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_array_size", 2);
-  res = G_Array_Size(x, y);
+  res = G_Array_Size(x_word, y_word);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -271,10 +270,10 @@ Pl_Blt_G_Array_Size(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void FC
-Pl_Blt_G_Inc(WamWord x)
+Pl_Blt_G_Inc(WamWord x_word)
 {
   Pl_Set_C_Bip_Name("g_inc", 1);
-  G_Inc_Dec(x, 1, NOT_A_WAM_WORD, NOT_A_WAM_WORD);
+  G_Inc_Dec(x_word, 1, NOT_A_WAM_WORD, NOT_A_WAM_WORD);
   Pl_Unset_C_Bip_Name();
 }
 
@@ -286,12 +285,12 @@ Pl_Blt_G_Inc(WamWord x)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Inco(WamWord x, WamWord y)
+Pl_Blt_G_Inco(WamWord x_word, WamWord y_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_inco", 2);
-  res = G_Inc_Dec(x, 1, y, NOT_A_WAM_WORD);
+  res = G_Inc_Dec(x_word, 1, y_word, NOT_A_WAM_WORD);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -304,12 +303,12 @@ Pl_Blt_G_Inco(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Inc_2(WamWord x, WamWord y)
+Pl_Blt_G_Inc_2(WamWord x_word, WamWord y_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_inc", 2);
-  res = G_Inc_Dec(x, 1, NOT_A_WAM_WORD, y);
+  res = G_Inc_Dec(x_word, 1, NOT_A_WAM_WORD, y_word);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -322,12 +321,12 @@ Pl_Blt_G_Inc_2(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Inc_3(WamWord x, WamWord y, WamWord z)
+Pl_Blt_G_Inc_3(WamWord x_word, WamWord y_word, WamWord z_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_inc", 3);
-  res = G_Inc_Dec(x, 1, y, z);
+  res = G_Inc_Dec(x_word, 1, y_word, z_word);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -340,10 +339,10 @@ Pl_Blt_G_Inc_3(WamWord x, WamWord y, WamWord z)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void FC
-Pl_Blt_G_Dec(WamWord x)
+Pl_Blt_G_Dec(WamWord x_word)
 {
   Pl_Set_C_Bip_Name("g_dec", 1);
-  G_Inc_Dec(x, -1, NOT_A_WAM_WORD, NOT_A_WAM_WORD);
+  G_Inc_Dec(x_word, -1, NOT_A_WAM_WORD, NOT_A_WAM_WORD);
   Pl_Unset_C_Bip_Name();
 }
 
@@ -355,12 +354,12 @@ Pl_Blt_G_Dec(WamWord x)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Deco(WamWord x, WamWord y)
+Pl_Blt_G_Deco(WamWord x_word, WamWord y_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_deco", 2);
-  res = G_Inc_Dec(x, -1, y, NOT_A_WAM_WORD);
+  res = G_Inc_Dec(x_word, -1, y_word, NOT_A_WAM_WORD);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -373,12 +372,12 @@ Pl_Blt_G_Deco(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Dec_2(WamWord x, WamWord y)
+Pl_Blt_G_Dec_2(WamWord x_word, WamWord y_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_dec", 2);
-  res = G_Inc_Dec(x, -1, NOT_A_WAM_WORD, y);
+  res = G_Inc_Dec(x_word, -1, NOT_A_WAM_WORD, y_word);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -391,12 +390,12 @@ Pl_Blt_G_Dec_2(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Dec_3(WamWord x, WamWord y, WamWord z)
+Pl_Blt_G_Dec_3(WamWord x_word, WamWord y_word, WamWord z_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_dec", 3);
-  res = G_Inc_Dec(x, -1, y, z);
+  res = G_Inc_Dec(x_word, -1, y_word, z_word);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -409,10 +408,10 @@ Pl_Blt_G_Dec_3(WamWord x, WamWord y, WamWord z)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void FC
-Pl_Blt_G_Set_Bit(WamWord x, WamWord y)
+Pl_Blt_G_Set_Bit(WamWord x_word, WamWord y_word)
 {
   Pl_Set_C_Bip_Name("g_set_bit", 2);
-  G_Set_Bit(x, y);
+  G_Set_Bit(x_word, y_word);
   Pl_Unset_C_Bip_Name();
 }
 
@@ -424,10 +423,10 @@ Pl_Blt_G_Set_Bit(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void FC
-Pl_Blt_G_Reset_Bit(WamWord x, WamWord y)
+Pl_Blt_G_Reset_Bit(WamWord x_word, WamWord y_word)
 {
   Pl_Set_C_Bip_Name("g_reset_bit", 2);
-  G_Reset_Bit(x, y);
+  G_Reset_Bit(x_word, y_word);
   Pl_Unset_C_Bip_Name();
 }
 
@@ -439,12 +438,12 @@ Pl_Blt_G_Reset_Bit(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Test_Set_Bit(WamWord x, WamWord y)
+Pl_Blt_G_Test_Set_Bit(WamWord x_word, WamWord y_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_test_set_bit", 2);
-  res = G_Test_Set_Bit(x, y);
+  res = G_Test_Set_Bit(x_word, y_word);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -457,12 +456,12 @@ Pl_Blt_G_Test_Set_Bit(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_G_Test_Reset_Bit(WamWord x, WamWord y)
+Pl_Blt_G_Test_Reset_Bit(WamWord x_word, WamWord y_word)
 {
   Bool res;
 
   Pl_Set_C_Bip_Name("g_test_reset_bit", 2);
-  res = G_Test_Reset_Bit(x, y);
+  res = G_Test_Reset_Bit(x_word, y_word);
   Pl_Unset_C_Bip_Name();
   return res;
 }
@@ -547,14 +546,13 @@ G_Assign(WamWord gvar_word, WamWord gval_word, Bool backtrack, Bool copy)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-G_Assign_Element(GVarElt *g_elem, WamWord gval_word, Bool backtrack,
-		 Bool copy)
+G_Assign_Element(GVarElt *g_elem, WamWord gval_word, Bool backtrack, Bool copy)
 {
   WamWord word, tag_mask;
   WamWord *adr;
-  int size;
+  PlLong size;
   int atom;
-  int save_size;
+  PlLong save_size;
   WamWord save_val;
   int array_op;
 
@@ -625,7 +623,7 @@ static void
 G_Assign_Arg(GVarElt *g_elem, WamWord *g_arg, WamWord word)
 {
   WamWord *adr;
-  int size;
+  PlLong size;
   GUndo *u;
 
   if (Pl_Term_Size(*g_arg) == 1 && Pl_Term_Size(word) == 1)
@@ -668,19 +666,17 @@ G_Assign_Array(GVarElt *g_elem, WamWord *stc_adr, int array_op,
 	       Bool backtrack, Bool copy)
 {
   WamWord word, tag_mask;
-  PlLong arity;
+  int arity;
   Bool same_init_value;
   WamWord init_word = 0;	/* init for the compiler */
   WamWord lst_word = 0;		/* init for the compiler */
-  PlLong new_size, size;
+  PlLong new_size, size, i;
   GVarElt *p;
-  int i;
 
   arity = Arity(stc_adr);
 
   DEREF(Arg(stc_adr, 0), word, tag_mask);
-  new_size = (tag_mask == TAG_LST_MASK) ? Pl_List_Length(word) :
-    UnTag_INT(word);
+  new_size = (tag_mask == TAG_LST_MASK) ? Pl_List_Length(word) : UnTag_INT(word);
 
   if (!(new_size > 0 && ((tag_mask == TAG_INT_MASK && arity <= 2) ||
 			 (tag_mask == TAG_LST_MASK && arity == 1))))
@@ -771,11 +767,11 @@ G_Assign_Array(GVarElt *g_elem, WamWord *stc_adr, int array_op,
  * point to new cells (realloc can return a different starting address).   *
  *-------------------------------------------------------------------------*/
 static GVarElt *
-G_Alloc_Array(GVarElt *g_elem, int new_size, Bool backtrack)
+G_Alloc_Array(GVarElt *g_elem, PlLong new_size, Bool backtrack)
 {
   GVarElt *p, *p_new_end, *src, *dst;
   GUndo *u;
-  int old_size, i;
+  PlLong old_size, i;
 
   old_size = -g_elem->size;
   src = (GVarElt *) g_elem->val;
@@ -843,7 +839,7 @@ Get_Target_From_Gvar(WamWord gvar_word)
   WamWord *arg_adr;
   GVarElt *g_elem, *g_end;
   GVarElt *p;
-  int i, j, size;
+  PlLong i, j, size;
   int new_size;
   PlLong index;
   GTarget *gt = &g_target;
@@ -1066,10 +1062,9 @@ Get_Int_From_Word(WamWord start_word)
 static void
 G_Free_Element(GVarElt *g_elem, Bool reinit_undo)
 {
-  int size;
+  PlLong size, i;
   GVarElt *p;
   GUndo *u;
-  int i;
 
   if (reinit_undo)
     {
@@ -1111,8 +1106,7 @@ G_Copy_Element(GVarElt *dst_g_elem, GVarElt *src_g_elem)
 {
   WamWord *adr;
   GVarElt *p;
-  int size;
-  int i;
+  PlLong size, i;
 
   size = dst_g_elem->size = src_g_elem->size;
   dst_g_elem->undo = NULL;
@@ -1162,7 +1156,7 @@ G_Copy_Element(GVarElt *dst_g_elem, GVarElt *src_g_elem)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-G_Trail_For_Backtrack(GVarElt *g_elem, int save_size, WamWord save_val)
+G_Trail_For_Backtrack(GVarElt *g_elem, PlLong save_size, WamWord save_val)
 {
   WamWord arg_frame[1];
   GUndo *u = (GUndo *) Malloc(sizeof(GUndo));
@@ -1251,9 +1245,8 @@ static Bool
 G_Read_Element(GVarElt *g_elem, WamWord gval_word)
 {
   WamWord word;
-  int size = g_elem->size;
+  PlLong size = g_elem->size, i;
   GVarElt *p;
-  int i;
 
 
   if (size == 0)		/* a link: unify */
@@ -1304,7 +1297,7 @@ G_Array_Size(WamWord gvar_word, WamWord size_word)
   GTarget *gt = Get_Target_From_Gvar(gvar_word);
   GVarElt *g_elem = gt->g_elem;
   WamWord *g_arg = gt->g_arg;
-  int size;
+  PlLong size;
 
   Pl_Check_For_Un_Integer(size_word);
 
@@ -1364,7 +1357,7 @@ G_Set_Bit(WamWord gvar_word, WamWord bit_word)
   PlULong mask;
 
   adr = Get_Int_Addr_From_Gvar(gvar_word);
-  mask = 1 << (bit + TAG_SIZE_LOW);
+  mask = (PlULong) 1 << (bit + TAG_SIZE_LOW);
   *adr |= mask;
 }
 
@@ -1383,7 +1376,7 @@ G_Reset_Bit(WamWord gvar_word, WamWord bit_word)
   PlULong mask;
 
   adr = Get_Int_Addr_From_Gvar(gvar_word);
-  mask = 1 << (bit + TAG_SIZE_LOW);
+  mask = (PlULong) 1 << (bit + TAG_SIZE_LOW);
   *adr &= ~mask;
 }
 
@@ -1401,7 +1394,7 @@ G_Test_Set_Bit(WamWord gvar_word, WamWord bit_word)
   PlULong val, mask;
 
   val = Get_Int_From_Gvar(gvar_word);
-  mask = 1 << (bit + TAG_SIZE_LOW);
+  mask = (PlULong) 1 << (bit + TAG_SIZE_LOW);
   return (val & mask) != 0;
 }
 
@@ -1419,7 +1412,7 @@ G_Test_Reset_Bit(WamWord gvar_word, WamWord bit_word)
   PlULong mask, val;
 
   val = Get_Int_From_Gvar(gvar_word);
-  mask = 1 << (bit + TAG_SIZE_LOW);
+  mask = (PlULong) 1 << (bit + TAG_SIZE_LOW);
   return (val & mask) == 0;
 }
 
