@@ -6,7 +6,7 @@
  * Descr.: parser support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -76,7 +76,8 @@
 static StmInf *pstm_i;
 
 static Bool tok_present;
-static TokInf unget_tok;
+static char unget_tok_buffer[SCAN_BIG_BUFFER];
+static TokInf unget_tok = { .name = unget_tok_buffer };
 
 
 static sigjmp_buf jumper;
@@ -117,14 +118,20 @@ static int Lookup_In_Dico_Var(char *name);
 static void Parse_Error(char *err_msg);
 
 
-/* we simply save line/col (for error report) to avoid to 
- * duplicate the entire token (with the big buffer for names).
- * After Unget_Token it is possible to restore pl_token.line/col */
+/* ifdef MINUS_SIGN_CAN_BE_FOLLOWED_BY_SPACES we can have -   XXX. If XXX is not
+ * a number this forms 2 tokens. We have to Unget the last one (XXX) and thus we
+ * need 2 buffers.
+ * Except this case, a boolean tok_present + save pl_token.line/col (for error 
+ * report) to restore then after Unget_Token.
+ * NB: Unget_Token is called very ofter and MUST be fast. Do not copy too large
+ * blocks of memory */
 
 #define Unget_Token				\
 do {						\
-  tok_present = TRUE;				\
+  char *swap_buffer = unget_tok.name;		\
   unget_tok = pl_token;				\
+  pl_token.name = swap_buffer;			\
+  tok_present = TRUE;				\
 } while(0)
 
 
@@ -168,8 +175,11 @@ Read_Next_Token(Bool comma_is_punct)
 
   if (tok_present)
     {
-      tok_present = FALSE;
+      char *swap_buffer = pl_token.name;
       pl_token = unget_tok;
+      unget_tok.name = swap_buffer; /* swap name buffers to be fast */
+      
+      tok_present = FALSE;
       if (comma_is_punct && pl_token.type == TOKEN_NAME &&  !pl_token.quoted &&
 	  pl_token.name[0] == ',' && pl_token.name[1] == '\0')
 	{
@@ -300,7 +310,7 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
   Bool infix_op;
   int cur_left = 0;
   int i;
-  int flag_value;
+  PlLong flag_value;
   WamWord term, term1;
   WamWord w[2];
   Bool left_is_op = FALSE;
@@ -340,19 +350,18 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
       flag_value = (pl_token.type == TOKEN_STRING) ?
 	Flag_Value(double_quotes) : Flag_Value(back_quotes);
 
-      flag_value &= PF_QUOT_AS_PART_MASK;
-      if (flag_value == PF_QUOT_AS_ATOM)
+      if (flag_value == PF_QUOT_AS_ATOM || flag_value == PF_QUOT_AS_ATOM_NO_ESCAPE)
 	{
 	  atom = Pl_Create_Allocate_Atom(pl_token.name);
 	  goto a_name;
 	}
 
-      i = strlen(pl_token.name);
+      i = (int) strlen(pl_token.name);
       term = NIL_WORD;		/* faster than Pl_Put_Nil() */
       while (i--)
 	{
 	  term1 = Pl_Put_List();
-	  if (flag_value == PF_QUOT_AS_CODES)
+	  if (flag_value == PF_QUOT_AS_CODES || flag_value == PF_QUOT_AS_CODES_NO_ESCAPE)
 	    Pl_Unify_Integer(pl_token.name[i]);
 	  else
 	    Pl_Unify_Atom(ATOM_CHAR(pl_token.name[i]));
@@ -397,8 +406,8 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
 #endif
 	  )
 	{
-	  int save_line = pl_token.line;
-	  int save_col = pl_token.col;
+	  PlLong save_line = pl_token.line;
+	  PlLong save_col = pl_token.col;
 
 	  Read_Next_Token(COMMA_ANY);
 	  if (pl_token.type == TOKEN_INTEGER)
@@ -416,7 +425,7 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
 	    }
 
 	  /* '-' not followed by a number, pushback this token */
-	  /* (cannot occur ifndef MINUS_SIGN_CAN_BE_FOLLOWED_BY_SPACES) */
+	  /* (cannot occur ifdef MINUS_SIGN_CAN_BE_FOLLOWED_BY_SPACES) */
 
 	  Unget_Token;
 
@@ -566,8 +575,7 @@ Parse_Args_Of_Functor(int atom)
 static WamWord
 Parse_Bracketed_Term(void)
 {
-  WamWord term;
-
+  WamWord term = 0;		/* init for the compiler */
 
   switch (pl_token.punct)
     {
@@ -612,7 +620,7 @@ static WamWord
 Parse_List(Bool can_be_empty)
 {
   WamWord term;
-  WamWord car_word, cdr_word;
+  WamWord car_word, cdr_word = 0; /* init for the compiler */
 
 
   car_word = Parse_Term(MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, TRUE);
@@ -770,7 +778,7 @@ Pl_Read_Atom(StmInf *pstm)
   if ((err_msg = Pl_Scan_Next_Atom(pstm)) != NULL)
     {
       Pl_Set_Last_Syntax_Error(pl_atom_tbl[pstm->atom_file_name].name,
-			    pl_token.line, pl_token.col, err_msg);
+			       pl_token.line, pl_token.col, err_msg);
       return NOT_A_WAM_WORD;
     }
 
@@ -842,8 +850,8 @@ Pl_Read_Number(StmInf *pstm)
 WamWord
 Pl_Read_Token(StmInf *pstm)
 {
-  WamWord term, arg;
-  int func, atom;
+  WamWord term, arg = 0;	/* init for the compiler */
+  int func = 0, atom;		/* init for the compiler */
   char *err_msg;
 
   if ((err_msg = Pl_Scan_Token(pstm, FALSE)) != NULL)
