@@ -6,7 +6,7 @@
  * Descr.: pretty print clause management - C part                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -34,7 +34,7 @@
  * the GNU Lesser General Public License along with this program.  If      *
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
-
+#include <string.h>
 
 #define OBJ_INIT Pretty_Initializer
 
@@ -83,9 +83,11 @@ static WamWord dollar_varname_1;
 static WamWord equal_2;
 
 static PlLong *singl_var_ptr;
-static int nb_singl_var;
+static PlLong nb_singl_var;
 
-static int nb_to_try;
+static PlLong nb_excl_var_tabled;
+
+static PlLong nb_to_try;
 
 static WamWord *above_H;
 
@@ -98,8 +100,7 @@ static WamWord *above_H;
 
 static void Portray_Clause(StmInf *pstm, WamWord term_word);
 
-static Bool Check_Structure(WamWord term_word, int func, int arity,
-			    WamWord arg_word[]);
+static Bool Check_Structure(WamWord term_word, int func, int arity, WamWord arg_word[]);
 
 static Bool Is_Cut(WamWord body_word);
 
@@ -107,11 +108,11 @@ static void Show_Body(StmInf *pstm, int level, int context, WamWord body_word);
 
 static void Start_Line(StmInf *pstm, int level, char c_before);
 
-static Bool Collect_Singleton(WamWord *adr);
+static Bool Collect_Singleton(WamWord *adr, WamWord var_word);
 
 static int Var_Name_To_Var_Number(int atom);
 
-static void Exclude_A_Var_Number(int n);
+static void Exclude_A_Var_Number(PlLong n);
 
 static void Collect_Excluded_Rec(WamWord start_word);
 
@@ -156,7 +157,7 @@ Pl_Portray_Clause_3(WamWord sora_word, WamWord term_word, WamWord above_word)
   
 
   stm = (sora_word == NOT_A_WAM_WORD)
-    ? pl_stm_output : Pl_Get_Stream_Or_Alias(sora_word, STREAM_CHECK_OUTPUT);
+    ? pl_stm_current_output : Pl_Get_Stream_Or_Alias(sora_word, STREAM_CHECK_OUTPUT);
   pstm = pl_stm_tbl[stm];
 
   pl_last_output_sora = sora_word;
@@ -201,7 +202,7 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
 
   if (Check_Structure(term_word, atom_clause, 2, arg_word))
     {
-      Pl_Write_Term(pstm, -1, 1200 - 1, WRITE_MASK, above_H, arg_word[0]);
+      Pl_Write_Term(pstm, 0, 1200 - 1, WRITE_MASK, above_H, arg_word[0]);
       DEREF(arg_word[1], word, tag_mask);
       atom = UnTag_ATM(word);
       if (tag_mask != TAG_ATM_MASK || atom != pl_atom_true)
@@ -216,7 +217,7 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
 
   if (Check_Structure(term_word, atom_dcg, 2, arg_word))
     {
-      Pl_Write_Term(pstm, -1, 1200 - 1, WRITE_MASK, above_H, arg_word[0]);
+      Pl_Write_Term(pstm, 0, 1200 - 1, WRITE_MASK, above_H, arg_word[0]);
       DEREF(arg_word[1], word, tag_mask);
       atom = UnTag_ATM(word);
       if (tag_mask != TAG_ATM_MASK || atom != pl_atom_true)
@@ -237,7 +238,7 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
       return;
     }
 
-  Pl_Write_Term(pstm, -1, MAX_PREC, WRITE_MASK, above_H, term_word);
+  Pl_Write_Term(pstm, 0, MAX_PREC, WRITE_MASK, above_H, term_word);
   Pl_Write_A_Full_Stop(pstm);
 }
 
@@ -364,7 +365,7 @@ Show_Body(StmInf *pstm, int level, int context, WamWord body_word)
       return;
     }
 
-  Pl_Write_Term(pstm, -1, prec[context], WRITE_MASK, above_H, body_word);
+  Pl_Write_Term(pstm, 0, prec[context], WRITE_MASK, above_H, body_word);
 }
 
 
@@ -445,7 +446,7 @@ Pl_Name_Singleton_Vars_1(WamWord start_word)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static Bool
-Collect_Singleton(WamWord *adr)
+Collect_Singleton(WamWord *adr, WamWord var_word)
 {
   PlLong *p;
 
@@ -547,10 +548,12 @@ Pl_Bind_Variables_4(WamWord term_word, WamWord exclude_list_word,
   WamWord word, tag_mask;
   WamWord save_list_word;
   WamWord *lst_adr, *stc_adr;
-  int i;
 
-  for (i = 0; i < MAX_VAR_IN_TERM; i++)
-    pl_glob_dico_var[i] = 0;	/* pl_glob_dico_var: excluded var ? (0/1) */
+  /* pl_glob_dico_var: excluded var ? (0/1) 
+   * mark var nb I already used to avoid to bind with '$VAR'(I) 
+   * it is very long to initialize to 0 the whole pl_glob_dico_var 
+   * since MAX_VAR_IN_TERM can be very large, so we do it incrementally when needed */
+  nb_excl_var_tabled = 0;
 
   nb_to_try = Pl_Rd_Positive_Check(from_word);
 
@@ -626,10 +629,48 @@ Var_Name_To_Var_Number(int atom)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-Exclude_A_Var_Number(int n)
+Exclude_A_Var_Number(PlLong n)
 {
-  if (n >= 0 && n < MAX_VAR_IN_TERM)
-    pl_glob_dico_var[n] = 1;
+  PlLong k;
+  
+  if (n < 0 || n >= MAX_VAR_IN_TERM)
+    return;
+#ifdef DEBUG
+  printf("--- mark: %ld\n", n);
+  for(k = 0; k < nb_excl_var_tabled; k++)
+    printf("%3ld", k);
+  puts("");
+  for(k = 0; k < nb_excl_var_tabled; k++)
+    printf("%3ld", pl_glob_dico_var[k]);
+  puts("");
+#endif
+
+  /* if needed init (bzero) portion of pl_glob_dico_var in [nb_excl_var_tabled..n-1] */
+  k = n - nb_excl_var_tabled;
+  if (k > 0)
+    {
+#ifdef DEBUG
+      printf("enlarge, init [%ld..%ld] = %ld entries\n", nb_excl_var_tabled, n - 1, k);
+#endif
+      memset(pl_glob_dico_var + nb_excl_var_tabled, 0, k * sizeof(pl_glob_dico_var[0]));
+    }
+
+  if (nb_excl_var_tabled <= n)
+    nb_excl_var_tabled = n + 1;      /* +1 since n will be marked below thus it is tabled */
+
+
+  /* mark as excluded */
+  pl_glob_dico_var[n] = 1;
+
+#ifdef DEBUG
+  printf("===>\n");
+  for(k = 0; k < nb_excl_var_tabled; k++)
+    printf("%3ld", k);
+  puts("");
+  for(k = 0; k < nb_excl_var_tabled; k++)
+    printf("%3ld", pl_glob_dico_var[k]);
+  puts("");
+#endif
 }
 
 
@@ -703,10 +744,11 @@ Collect_Excluded_Rec(WamWord start_word)
 static Bool
 Bind_Variable(WamWord *adr, WamWord word)
 {
-  int i, j;
-  char buff[16];
+  int i;
+  PlLong j;
+  char buff[32];
 
-  while (pl_glob_dico_var[nb_to_try] && nb_to_try < MAX_VAR_IN_TERM)
+  while (nb_to_try < nb_excl_var_tabled && pl_glob_dico_var[nb_to_try])
     nb_to_try++;
 
   if (BIND_WITH_NUMBERVAR)
@@ -716,14 +758,14 @@ Bind_Variable(WamWord *adr, WamWord word)
       return TRUE;
     }
 
-  i = nb_to_try % 26;
+  i = (int) (nb_to_try % 26);
   j = nb_to_try / 26;
   nb_to_try++;
 
   buff[0] = 'A' + i;
 
   if (j)
-    sprintf(buff + 1, "%d", j);
+    sprintf(buff + 1, "%" PL_FMT_d, j);
   else
     buff[1] = '\0';
 
