@@ -6,7 +6,7 @@
  * Descr.: read/1 and friends - C part                                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -65,7 +65,7 @@ Prolog_Prototype(CURRENT_CHAR_CONVERSION_ALT, 0);
 
 #define CHECK_STREAM_AND_GET_STM(sora_word, stm)		\
   stm = (sora_word == NOT_A_WAM_WORD)				\
-         ? pl_stm_input :					\
+         ? pl_stm_current_input :				\
          Pl_Get_Stream_Or_Alias(sora_word, STREAM_CHECK_INPUT);	\
 								\
   pl_last_input_sora = sora_word;				\
@@ -78,8 +78,8 @@ Prolog_Prototype(CURRENT_CHAR_CONVERSION_ALT, 0);
   if (returned_word == NOT_A_WAM_WORD)				\
     {								\
       Pl_Syntax_Error((SYS_VAR_SYNTAX_ERROR_ACTON < 0)		\
-		   ? Flag_Value(syntax_error)		\
-		   : SYS_VAR_SYNTAX_ERROR_ACTON);		\
+		      ? (int) Flag_Value(syntax_error)		\
+		      : (int) SYS_VAR_SYNTAX_ERROR_ACTON);	\
       return FALSE;						\
     }								\
 								\
@@ -136,7 +136,7 @@ Pl_Read_Term_5(WamWord sora_word, WamWord term_word,
 	  pl_glob_dico_var[i] = Pl_Create_Allocate_Atom(pl_parse_dico_var[i].name);
 
 	  word = Pl_Put_Structure(ATOM_CHAR('='), 2);
-	  Pl_Unify_Atom(pl_glob_dico_var[i]);
+	  Pl_Unify_Atom((int) pl_glob_dico_var[i]);
 	  Pl_Unify_Value(pl_parse_dico_var[i].word);
 
 	  if (!Pl_Get_List(var_names_word) || !Pl_Unify_Value(word))
@@ -162,7 +162,7 @@ Pl_Read_Term_5(WamWord sora_word, WamWord term_word,
 	    pl_glob_dico_var[i] = Pl_Create_Allocate_Atom(pl_parse_dico_var[i].name);
 
 	  word = Pl_Put_Structure(ATOM_CHAR('='), 2);
-	  Pl_Unify_Atom(pl_glob_dico_var[i]);
+	  Pl_Unify_Atom((int) pl_glob_dico_var[i]);
 	  Pl_Unify_Value(pl_parse_dico_var[i].word);
 
 	  if (!Pl_Get_List(sing_names_word) || !Pl_Unify_Value(word))
@@ -464,8 +464,8 @@ Pl_Current_Char_Conversion_Alt_0(void)
 
   in_char_word = AB(B, 0);
   out_char_word = AB(B, 1);
-  c_in = AB(B, 2);
-  c_out = AB(B, 3);
+  c_in = (int) AB(B, 2);
+  c_out = (int) AB(B, 3);
 
   c_in1 = c_in;
   Find_Next_Char_Conversion(c_in1, c_out1);
