@@ -6,7 +6,7 @@
  * Descr.: scanner support - header file                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -40,7 +40,7 @@
  * Constants                       *
  *---------------------------------*/
 
-#define SCAN_BIG_BUFFER            10240
+#define SCAN_BIG_BUFFER            (1024 * 1024)
 
 
 
@@ -71,13 +71,13 @@ TypTok;
 typedef struct
 {
   TypTok type;
-  char name[SCAN_BIG_BUFFER];	/* for VARIABLE NAME STRING BACK_QUOTED */
+  char *name;			/* for VARIABLE NAME STRING BACK_QUOTED */
   int quoted;                   /* for NAME: was it quoted ?            */
   int punct;			/* for PUNCTUATION                      */
   PlLong int_num;		/* for INTEGER                          */
   double float_num;		/* for FLOAT                            */
-  int line;			/* source line of the pl_token          */
-  int col;			/* source column of the pl_token        */
+  PlLong line;			/* source line of the pl_token          */
+  PlLong col;			/* source column of the pl_token        */
 }
 TokInf;
 
@@ -90,7 +90,9 @@ TokInf;
 
 #ifdef SCAN_SUPP_FILE
 
-TokInf pl_token;
+static char scan_tok_buffer[SCAN_BIG_BUFFER];
+
+TokInf pl_token = { .name = scan_tok_buffer, };
 
 #else
 
