@@ -6,7 +6,7 @@
  * Descr.: sort management - C part                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -68,7 +68,7 @@ static WamWord minus_2;
 static PlLong Keysort_Cmp(WamWord u_word, WamWord v_word);
 
 static int Merge_Sort(WamWord *base, WamWord *aux, int n,
-		      Bool keep_dup, PlLong (*cmp) ());
+		      Bool keep_dup, PlLong (*cmp) (WamWord u_word, WamWord v_word));
 
 
 
@@ -138,7 +138,7 @@ Pl_Sort_List_2(WamWord list1_word, WamWord list2_word)
   int n;
   int sort_type;
 
-  sort_type = SYS_VAR_OPTION_MASK;	/* 0=sort/2, 1=msort/2, 2=keysort/2 */
+  sort_type = (int) SYS_VAR_OPTION_MASK; /* 0=sort/2, 1=msort/2, 2=keysort/2 */
 
   arg = H;			/* array in the heap */
 
@@ -159,8 +159,7 @@ Pl_Sort_List_2(WamWord list1_word, WamWord list2_word)
   if (n == 1)
     return Pl_Unify(list1_word, list2_word);
 
-  n = Merge_Sort(arg, arg + n, n, sort_type,
-		 (sort_type != 2) ? Pl_Term_Compare : Keysort_Cmp);
+  n = Merge_Sort(arg, arg + n, n, sort_type, (sort_type != 2) ? Pl_Term_Compare : Keysort_Cmp);
 
   /* n can have changed here (if dup removed) */
 
@@ -182,7 +181,7 @@ Pl_Sort_List_1(WamWord list_word)
   int n;
   int sort_type;
 
-  sort_type = SYS_VAR_OPTION_MASK;	/* 0=sort/1, 1=msort/1, 2=keysort/1 */
+  sort_type = (int) SYS_VAR_OPTION_MASK; /* 0=sort/1, 1=msort/1, 2=keysort/1 */
 
   arg = H;
 
@@ -198,8 +197,7 @@ Pl_Sort_List_1(WamWord list_word)
   if (n <= 1)
     return;
 
-  n = Merge_Sort(arg, arg + n, n, sort_type,
-		 (sort_type != 2) ? Pl_Term_Compare : Keysort_Cmp);
+  n = Merge_Sort(arg, arg + n, n, sort_type, (sort_type != 2) ? Pl_Term_Compare : Keysort_Cmp);
   /* n can have changed here (if dup removed) */
   /* update in-place the list */
   do
@@ -243,7 +241,7 @@ Keysort_Cmp(WamWord u_word, WamWord v_word)
  * elements of the array (2 WamWords) and classically returns <0, 0, >0.   *
  *-------------------------------------------------------------------------*/
 static int
-Merge_Sort(WamWord *base, WamWord *aux, int n, Bool keep_dup, PlLong (*cmp) ())
+Merge_Sort(WamWord *base, WamWord *aux, int n, Bool keep_dup, PlLong (*cmp) (WamWord u_word, WamWord v_word))
 {
   WamWord *l1, *l2;
   int n1, n2;
