@@ -6,7 +6,7 @@
  * Descr.: stream support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -80,7 +80,7 @@ extern void clearerr(FILE *stream);
 
 #define STR_STREAM_WRITE_BLOCK     1024
 
-#define TTY_BUFFER_SIZE            1024
+#define TTY_BUFFER_SIZE            10240
 
 #define BIG_BUFFER                 65535
 
@@ -102,8 +102,8 @@ extern void clearerr(FILE *stream);
  * Global Variables                *
  *---------------------------------*/
 
-static void Init_Stream_Supp();
-void (*pl_init_stream_supp)() = Init_Stream_Supp; /* overwrite var of engine.c */
+static void Init_Stream_Supp(void);
+void (*pl_init_stream_supp)(void) = Init_Stream_Supp; /* overwrite var of engine.c */
 
 static int atom_constant_term_stream;
 
@@ -190,6 +190,9 @@ Init_Stream_Supp(void)
   pl_atom_user_output = Pl_Create_Atom("user_output");
   pl_atom_user_error = Pl_Create_Atom("user_error");
 
+  pl_atom_current_input = Pl_Create_Atom("current_input");
+  pl_atom_current_output = Pl_Create_Atom("current_output");
+
   pl_atom_top_level_input = Pl_Create_Atom("top_level_input");
   pl_atom_top_level_output = Pl_Create_Atom("top_level_output");
 
@@ -223,12 +226,14 @@ Init_Stream_Supp(void)
   pl_atom_current = Pl_Create_Atom("current");
   pl_atom_eof = Pl_Create_Atom("eof");
 
+  pl_atom_null = Pl_Create_Atom("null");
+
   pl_le_prompt = "";
   pl_use_le_prompt = TRUE;
 
-  pl_stm_stdin = Pl_Add_Stream_For_Stdio_Desc(stdin, pl_atom_user_input,
-					      STREAM_MODE_READ, TRUE, TRUE);
 
+  
+  pl_stm_stdin = Pl_Add_Stream_For_Stdio_Desc(stdin, pl_atom_user_input, STREAM_MODE_READ, TRUE, TRUE);
 #ifndef NO_USE_LINEDIT
   if (pl_le_mode == LE_MODE_HOOK || (pl_le_mode == LE_MODE_TTY && isatty(0)))
     {
@@ -243,16 +248,12 @@ Init_Stream_Supp(void)
       pl_stream_use_linedit = TRUE;
     }
 #endif
-  Pl_Add_Alias_To_Stream(pl_atom_user_input, pl_stm_stdin);
-  pl_stm_input = pl_stm_stdin;
+  
 
 
-
-  pl_stm_stdout = Pl_Add_Stream_For_Stdio_Desc(stdout, pl_atom_user_output,
-					       STREAM_MODE_WRITE, TRUE, TRUE);
-
+  pl_stm_stdout = Pl_Add_Stream_For_Stdio_Desc(stdout, pl_atom_user_output, STREAM_MODE_WRITE, TRUE, TRUE);
 #if !defined(NO_USE_LINEDIT) && defined(_WIN32)
-		/* ok for both GUI and console EOM<->ANSI conversion */
+  /* ok for both GUI and console EOM<->ANSI conversion */
   pstm = pl_stm_tbl[pl_stm_stdout];
   pstm->prop.buffering = STREAM_BUFFERING_LINE;
   if (pl_le_hook_put_char && isatty(1))
@@ -261,16 +262,12 @@ Init_Stream_Supp(void)
   if (pl_le_hook_flush && isatty(1))
     pstm->fct_flush = (StmFct) pl_le_hook_flush;
 #endif
-  Pl_Add_Alias_To_Stream(pl_atom_user_output, pl_stm_stdout);
-  pl_stm_output = pl_stm_stdout;
 
 
 
-  pl_stm_stderr = Pl_Add_Stream_For_Stdio_Desc(stderr, pl_atom_user_error,
-					       STREAM_MODE_WRITE, TRUE, TRUE);
-
+  pl_stm_stderr = Pl_Add_Stream_For_Stdio_Desc(stderr, pl_atom_user_error, STREAM_MODE_WRITE, TRUE, TRUE);
 #if !defined(NO_USE_LINEDIT) && defined(_WIN32)
-		/* ok for both GUI and console EOM<->ANSI conversion */
+  /* ok for both GUI and console EOM<->ANSI conversion */
   pstm = pl_stm_tbl[pl_stm_stderr];
   pstm->prop.buffering = STREAM_BUFFERING_LINE;
   if (pl_le_hook_put_char && isatty(2))
@@ -279,17 +276,21 @@ Init_Stream_Supp(void)
   if (pl_le_hook_flush && isatty(2))
     pstm->fct_flush = (StmFct) pl_le_hook_flush;
 #endif
-  Pl_Add_Alias_To_Stream(pl_atom_user_error, pl_stm_stderr);
-  pl_stm_error = pl_stm_stderr;
 
-  pl_stm_top_level_input = pl_stm_debugger_input = pl_stm_input;
-  pl_stm_top_level_output = pl_stm_debugger_output = pl_stm_output;
+  /* these aliases are initially set to stdin/out/err and can be changed by the user */
+  
+  pl_alias_user_input = Pl_Set_Alias_To_Stream(pl_atom_user_input, pl_stm_stdin, TRUE);
+  pl_alias_user_output = Pl_Set_Alias_To_Stream(pl_atom_user_output, pl_stm_stdout, TRUE);
+  pl_alias_user_error = Pl_Set_Alias_To_Stream(pl_atom_user_error, pl_stm_stderr, TRUE);
 
-  Pl_Add_Alias_To_Stream(pl_atom_top_level_input, pl_stm_top_level_input);
-  Pl_Add_Alias_To_Stream(pl_atom_top_level_output, pl_stm_top_level_output);
+  pl_alias_current_input = Pl_Set_Alias_To_Stream(pl_atom_current_input, pl_stm_stdin, TRUE); 
+  pl_alias_current_output = Pl_Set_Alias_To_Stream(pl_atom_current_output, pl_stm_stdout, TRUE);
 
-  Pl_Add_Alias_To_Stream(pl_atom_debugger_input, pl_stm_debugger_input);
-  Pl_Add_Alias_To_Stream(pl_atom_debugger_output, pl_stm_debugger_output);
+  pl_alias_top_level_input = Pl_Set_Alias_To_Stream(pl_atom_top_level_input, pl_stm_stdin, TRUE);
+  pl_alias_top_level_output = Pl_Set_Alias_To_Stream(pl_atom_top_level_output, pl_stm_stdout, TRUE);
+
+  pl_alias_debugger_input = Pl_Set_Alias_To_Stream(pl_atom_debugger_input, pl_stm_stdin, TRUE);
+  pl_alias_debugger_output = Pl_Set_Alias_To_Stream(pl_atom_debugger_output, pl_stm_stdout, TRUE);
 }
 
 
@@ -363,8 +364,7 @@ Pl_Add_Stream_For_Stdio_Desc(FILE *f, int atom_path, int mode, Bool text,
   if (force_eof_reset || isatty(fileno(f)))
     prop.eof_action = STREAM_EOF_ACTION_RESET;
 
-  return Pl_Add_Stream(atom_path, (PlLong) f, prop,
-		       NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+  return Pl_Add_Stream(atom_path, f, fileno(f), prop, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 }
 
 
@@ -401,7 +401,7 @@ Pl_Add_Stream_For_Stdio_File(char *path, int mode, Bool text)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-Init_Stream_Struct(int atom_file_name, PlLong file, StmProp prop,
+Init_Stream_Struct(int atom_file_name, void *file, int fileno, StmProp prop,
 		   StmFct fct_getc, StmFct fct_putc,
 		   StmFct fct_flush, StmFct fct_close,
 		   StmFct fct_tell, StmFct fct_seek, StmFct fct_clearerr,
@@ -409,6 +409,7 @@ Init_Stream_Struct(int atom_file_name, PlLong file, StmProp prop,
 {
   pstm->atom_file_name = atom_file_name;
   pstm->file = file;
+  pstm->fileno = fileno;
   pstm->prop = prop;
   pstm->mirror = NULL;
   pstm->mirror_of = NULL;
@@ -443,7 +444,7 @@ Init_Stream_Struct(int atom_file_name, PlLong file, StmProp prop,
  *                                                                         *
  *-------------------------------------------------------------------------*/
 int
-Pl_Add_Stream(int atom_file_name, PlLong file, StmProp prop,
+Pl_Add_Stream(int atom_file_name, void *file, int fileno, StmProp prop,
 	      StmFct fct_getc, StmFct fct_putc,
 	      StmFct fct_flush, StmFct fct_close,
 	      StmFct fct_tell, StmFct fct_seek, StmFct fct_clearerr)
@@ -453,13 +454,12 @@ Pl_Add_Stream(int atom_file_name, PlLong file, StmProp prop,
 
   stm = Find_Free_Stream();
 
-  if (prop.reposition && (fct_tell == STREAM_FCT_UNDEFINED ||
-			  fct_seek == STREAM_FCT_UNDEFINED))
+  if (prop.reposition && (fct_tell == STREAM_FCT_UNDEFINED || fct_seek == STREAM_FCT_UNDEFINED))
     Pl_Fatal_Error(ERR_TELL_OR_SEEK_UNDEFINED);
 
 
   pstm = pl_stm_tbl[stm];
-  Init_Stream_Struct(atom_file_name, file, prop, fct_getc, fct_putc,
+  Init_Stream_Struct(atom_file_name, file, fileno, prop, fct_getc, fct_putc,
 		     fct_flush, fct_close, fct_tell, fct_seek, fct_clearerr,
 		     pstm);
 
@@ -470,14 +470,17 @@ Pl_Add_Stream(int atom_file_name, PlLong file, StmProp prop,
 
 
 /*-------------------------------------------------------------------------*
- * REMOVE_STREAM                                                           *
+ * PL_DELETE_STREAM                                                        *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Delete_Stream(int stm)
+Pl_Delete_Stream(int stm, Bool keep_stream)
 {
   Del_Aliases_Of_Stream(stm);
 
+  if (keep_stream)
+    return;
+
   Update_Mirrors_To_Del_Stream(stm);
 
   Free(pl_stm_tbl[stm]);
@@ -535,18 +538,29 @@ Pl_Find_Stream_By_Alias(int atom_alias)
 
 
 /*-------------------------------------------------------------------------*
- * PL_ADD_ALIAS_TO_STREAM                                                  *
+ * PL_SET_ALIAS_TO_STREAM                                                  *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-Bool
-Pl_Add_Alias_To_Stream(int atom_alias, int stm)
+AliasInf *
+Pl_Set_Alias_To_Stream(int atom_alias, int stm, Bool reassign)
 {
   AliasInf *alias;
   AliasInf alias_info;
 
   alias = (AliasInf *) Pl_Hash_Find(pl_alias_tbl, atom_alias);
   if (alias != NULL)
-    return alias->stm == stm;	/* fail if assigned to another stream */
+    {
+      if (!reassign) /* return NULL if the alias is assigned to another stream */
+	return (alias->stm == stm) ? alias : NULL; 
+
+      alias->stm = stm;		/* reassign it */
+      if (alias->atom == pl_atom_current_input)
+	pl_stm_current_input = stm;
+      else if (alias->atom == pl_atom_current_output)
+	pl_stm_current_output = stm;
+
+      return alias;
+    }      
 
   Pl_Extend_Table_If_Needed(&pl_alias_tbl);
 
@@ -555,25 +569,7 @@ Pl_Add_Alias_To_Stream(int atom_alias, int stm)
 
   alias = (AliasInf *) Pl_Hash_Insert(pl_alias_tbl, (char *) &alias_info, FALSE);
 
-  return TRUE;
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * PL_REASSIGN_ALIAS                                                       *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-void
-Pl_Reassign_Alias(int atom_alias, int stm)
-{
-  AliasInf *alias;
-
-  alias = (AliasInf *) Pl_Hash_Find(pl_alias_tbl, atom_alias);
-
-  if (alias != NULL)
-    alias->stm = stm;
+  return alias;
 }
 
 
@@ -583,7 +579,7 @@ Pl_Reassign_Alias(int atom_alias, int stm)
  * DEL_ALIASES_OF_STREAM                                                   *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
+static void
 Del_Aliases_Of_Stream(int stm)
 {
   HashScan scan;
@@ -591,8 +587,36 @@ Del_Aliases_Of_Stream(int stm)
 
   for (alias = (AliasInf *) Pl_Hash_First(pl_alias_tbl, &scan); alias;
        alias = (AliasInf *) Pl_Hash_Next(&scan))
-    if (alias->stm == stm)
+    {
+      if (alias->stm != stm)
+	continue;
+
+      if (alias->atom == pl_atom_user_input ||
+	  alias->atom == pl_atom_current_input ||
+	  alias->atom == pl_atom_top_level_input ||
+	  alias->atom == pl_atom_debugger_input)
+	{
+	  alias->stm = pl_stm_stdin;
+	  continue;
+	}
+      
+      if (alias->atom == pl_atom_user_output ||
+	  alias->atom == pl_atom_current_output ||
+	  alias->atom == pl_atom_top_level_output ||
+	  alias->atom == pl_atom_debugger_output)
+	{
+	  alias->stm = pl_stm_stdout;
+	  continue;
+	}
+      
+      if (alias->atom == pl_atom_user_error)
+	{
+	  alias->stm = pl_stm_stderr;
+	  continue;
+	}
+      
       Pl_Hash_Delete(pl_alias_tbl, alias->atom);
+    }
 }
 
 
@@ -772,8 +796,7 @@ Pl_Set_Stream_Buffering(int stm)
     }
 
 #ifndef NO_USE_LINEDIT		/* if GUI: inform it about buffering */
-  if ((pstm->file == (PlLong) stdout || pstm->file == (PlLong) stderr) && 
-      pl_le_hook_set_line_buffering)
+  if ((pstm->file == stdout || pstm->file == stderr) && pl_le_hook_set_line_buffering)
     (*pl_le_hook_set_line_buffering)(pstm->prop.buffering != STREAM_BUFFERING_NONE);
   else
 #endif
@@ -788,7 +811,10 @@ Pl_Set_Stream_Buffering(int stm)
 /*-------------------------------------------------------------------------*
  * PL_GET_STREAM_OR_ALIAS                                                  *
  *                                                                         *
- * return the associated stm or -1 if not exist (test==STREAM_CHECK_VALID) *
+ * return the associated stm if exists or                                  *
+ *        -1 if does not exist and test == STREAM_CHECK_VALID              *
+ *        -2 if uninstantiated and STREAM_CHECK_ACCEPT_VAR is set          *
+ *        -3 if atom null and STREAM_CHECK_ACCEPT_NULL                     *
  *-------------------------------------------------------------------------*/
 int
 Pl_Get_Stream_Or_Alias(WamWord sora_word, int test)
@@ -798,12 +824,21 @@ Pl_Get_Stream_Or_Alias(WamWord sora_word, int test)
   WamWord *stc_adr;
   PlLong stm = 0;		/* only for the compiler (NB: defined as PlLong to check validity) */
   int perm_oper;
+  Bool accept_var = (test & STREAM_CHECK_ACCEPT_VAR) != 0;
+  Bool accept_null = (test & STREAM_CHECK_ACCEPT_NULL) != 0;
+  Bool has_fileno = (test & STREAM_CHECK_HAS_FILENO) != 0;
+
+  test &= 255;
 
 
   DEREF(sora_word, word, tag_mask);
   if (tag_mask == TAG_ATM_MASK)	/* alias ? */
     {
       atom = UnTag_ATM(word);
+
+      if (atom == pl_atom_null && accept_null)
+	return -3;
+      
       stm = Pl_Find_Stream_By_Alias(atom);
       goto next_test;
     }
@@ -814,13 +849,17 @@ Pl_Get_Stream_Or_Alias(WamWord sora_word, int test)
       DEREF(Arg(stc_adr, 0), word, tag_mask1);
       stm = UnTag_INT(word);
 
-      if (Functor_And_Arity(stc_adr) == stream_1 &&
-	  tag_mask1 == TAG_INT_MASK)
+      if (Functor_And_Arity(stc_adr) == stream_1 && tag_mask1 == TAG_INT_MASK)
 	goto next_test;
     }
 
   if (tag_mask == TAG_REF_MASK)
-    Pl_Err_Instantiation();
+    {
+      if (accept_var)
+	return -2;
+      
+      Pl_Err_Instantiation();
+    }
 
   Pl_Err_Domain(pl_domain_stream_or_alias, sora_word);
 
@@ -834,7 +873,10 @@ Pl_Get_Stream_Or_Alias(WamWord sora_word, int test)
       Pl_Err_Existence(pl_existence_stream, sora_word);
     }
 
-  if (test == STREAM_CHECK_VALID || test == STREAM_CHECK_EXIST)
+ if (has_fileno && pl_stm_tbl[stm]->fileno < 0)
+    Pl_Err_Domain(pl_domain_file_stream, sora_word);
+    
+ if (test == STREAM_CHECK_VALID || test == STREAM_CHECK_EXIST)
     goto ok;
 
   if (test == STREAM_CHECK_INPUT)
@@ -861,6 +903,19 @@ Pl_Get_Stream_Or_Alias(WamWord sora_word, int test)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_GET_STREAM                                                           *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Get_Stream(int stm, WamWord start_word)
+{		/* similar to WAM Pl_Get_XXX instructions */
+  return Pl_Get_Structure(pl_atom_stream, 1, start_word) && Pl_Unify_Integer(stm);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CHECK_STREAM_TYPE                                                    *
  *                                                                         *
@@ -990,8 +1045,8 @@ Pl_Stdio_Desc_Of_Stream(int stm)
   if (stm == pl_stm_stdin)		/* works also for stdin with linedit */
     return stdin;
 
-  if (pstm->fct_getc == (StmFct) fgetc)
-    return (FILE *) (pstm->file);
+  if (pstm->fct_getc == (StmFct) fgetc) /* could also test pstm->fileno >= 0 */
+    return pstm->file;
 
   return NULL;
 }
@@ -1028,11 +1083,11 @@ Pl_Io_Fileno_Of_Stream(int stm)
 
 #ifndef NO_USE_LINEDIT
 
-#define SAVE_FOR_REENTRANCY				\
-{							\
-  int save_sys_var_option_mask = SYS_VAR_OPTION_MASK;	\
-  int save_last_read_line = pl_last_read_line;		\
-  int save_last_read_col = pl_last_read_col;
+#define SAVE_FOR_REENTRANCY					\
+{								\
+  int save_sys_var_option_mask = (int) SYS_VAR_OPTION_MASK;	\
+  PlLong save_last_read_line = pl_last_read_line;		\
+  PlLong save_last_read_col = pl_last_read_col;
 
 
 #define RESTORE_FOR_REENTRANCY				\
@@ -1161,11 +1216,11 @@ TTY_Clearerr(void)
 
 #ifdef FOR_EXTERNAL_USE
 
-#define Before_Reading(pstm, file)
+#define Before_Reading(pstm)
 
 #else
 
-#define Before_Reading(pstm, file)					\
+#define Before_Reading(pstm)						\
 {									\
   if (pstm->eof_reached)						\
     {									\
@@ -1184,7 +1239,7 @@ TTY_Clearerr(void)
       if (pstm->prop.reposition)					\
         Pl_Stream_Set_Position(pstm, SEEK_SET, 0, 0, 0, 0);		\
       if (pstm->fct_clearerr != STREAM_FCT_UNDEFINED)			\
-        (*pstm->fct_clearerr) (file);					\
+        CALL_CLEARERR(pstm);						\
     }									\
 }
 
@@ -1215,14 +1270,14 @@ Basic_Call_Fct_Getc(StmInf *pstm)
   int c;
   StmLst *m;
 #ifndef NO_USE_PIPED_STDIN_FOR_CONSULT
-  if (SYS_VAR_SAY_GETC && pstm->file == (PlLong) stdin)
+  if (SYS_VAR_SAY_GETC && pstm->file == stdin)
     /* could also test pstm->fct_getc == fgetc */
     {
-      putchar(CHAR_TO_EMIT_WHEN_CHAR);
+      putchar(CHAR_TO_EMIT_ON_PIPED_GETC);
       fflush(stdout);
     }
 #endif
-  c = (*pstm->fct_getc) (pstm->file);
+  c = CALL_GETC(pstm);
 
   if (c != EOF)
     for (m = pstm->mirror; m ; m = m->next)
@@ -1243,7 +1298,7 @@ Basic_Call_Fct_Putc(int c, StmInf *pstm)
 {
   StmLst *m;
 
-  (*pstm->fct_putc) (c, pstm->file);
+  CALL_PUTC(c, pstm);
 
   for (m = pstm->mirror; m ; m = m->next)
     Pl_Stream_Putc(c, pl_stm_tbl[m->stm]);
@@ -1272,8 +1327,7 @@ Pl_PB_Empty_Buffer(StmInf *pstm)
 int
 Pl_Stream_Get_Key(StmInf *pstm, int echo, int catch_ctrl_c)
 {
-  int c;
-  PlLong file = pstm->file;
+  PlLong c;
   Bool simulate;
 
 #ifndef NO_USE_LINEDIT
@@ -1283,7 +1337,7 @@ Pl_Stream_Get_Key(StmInf *pstm, int echo, int catch_ctrl_c)
 #endif
     simulate = TRUE;
 
-  Before_Reading(pstm, file);
+  Before_Reading(pstm);
 
   if (!PB_Is_Empty(pstm->pb_char))
     {
@@ -1318,7 +1372,7 @@ Pl_Stream_Get_Key(StmInf *pstm, int echo, int catch_ctrl_c)
 
   Update_Counters(pstm, c);
 
-  return c;
+  return (int) c;
 }
 
 
@@ -1331,10 +1385,9 @@ Pl_Stream_Get_Key(StmInf *pstm, int echo, int catch_ctrl_c)
 int
 Pl_Stream_Getc(StmInf *pstm)
 {
-  int c;
-  PlLong file = pstm->file;
+  PlLong c;
 
-  Before_Reading(pstm, file);
+  Before_Reading(pstm);
 
   if (!PB_Is_Empty(pstm->pb_char))
     {
@@ -1354,7 +1407,7 @@ Pl_Stream_Getc(StmInf *pstm)
 
   Update_Counters(pstm, c);
 
-  return c;
+  return (int) c;
 }
 
 
@@ -1399,11 +1452,9 @@ Pl_Stream_Ungetc(int c, StmInf *pstm)
 int
 Pl_Stream_Peekc(StmInf *pstm)
 {
-  int c;
-  PlLong file = pstm->file;
+  PlLong c;
 
-
-  Before_Reading(pstm, file);
+  Before_Reading(pstm);
 
   if (!PB_Is_Empty(pstm->pb_char))
     PB_Top(pstm->pb_char, c);
@@ -1413,7 +1464,7 @@ Pl_Stream_Peekc(StmInf *pstm)
       PB_Push(pstm->pb_char, c);
     }
 
-  return c;
+  return (int) c;
 }
 
 
@@ -1518,7 +1569,7 @@ Pl_Stream_Puts(char *str, StmInf *pstm)
       Update_Counters(pstm, c);
     }
 
-  return p - str;
+  return (int) (p - str);
 }
 
 
@@ -1548,7 +1599,7 @@ Pl_Stream_Printf(StmInf *pstm, char *format, ...)
       Update_Counters(pstm, c);
     }
 
-  return p - str;
+  return (int) (p - str);
 }
 
 
@@ -1561,10 +1612,8 @@ Pl_Stream_Printf(StmInf *pstm, char *format, ...)
 void
 Pl_Stream_Flush(StmInf *pstm)
 {
-  PlLong file = pstm->file;
-
   if (pstm->prop.output && pstm->fct_flush != STREAM_FCT_UNDEFINED)
-    (*pstm->fct_flush) (file);
+    CALL_FLUSH(pstm);
 }
 
 
@@ -1577,11 +1626,10 @@ Pl_Stream_Flush(StmInf *pstm)
 int
 Pl_Stream_Close(StmInf *pstm)
 {
-  PlLong file = pstm->file;
   int ret = 0;
 
   if (pstm->fct_close != STREAM_FCT_UNDEFINED)
-    ret = (*pstm->fct_close) (file);
+    ret = CALL_CLOSE(pstm);
 
   return ret;
 }
@@ -1622,12 +1670,10 @@ void
 Pl_Stream_Get_Position(StmInf *pstm, PlLong *offset, PlLong *char_count,
 		       PlLong *line_count, PlLong *line_pos)
 {
-  PlLong file = pstm->file;
-
   *offset = 0;
   if (pstm->prop.reposition && pstm->fct_tell != STREAM_FCT_UNDEFINED)
     {
-      if ((*offset = (*pstm->fct_tell) (file)) < 0)
+      if ((*offset = CALL_TELL(pstm)) < 0)
 	*offset = 0;
       else
 	{
@@ -1662,10 +1708,7 @@ int
 Pl_Stream_Set_Position(StmInf *pstm, int whence, PlLong offset, PlLong char_count,
 		       PlLong line_count, PlLong line_pos)
 {
-  PlLong file = pstm->file;
-  int x;
-
-  x = (*pstm->fct_seek) (file, (PlLong) offset, whence);
+  int x = CALL_SEEK(pstm, (long) offset, whence);
   if (x != 0)
     return x;
 
@@ -1681,7 +1724,7 @@ Pl_Stream_Set_Position(StmInf *pstm, int whence, PlLong offset, PlLong char_coun
     {
       pstm->eof_reached = FALSE;
       if (pstm->fct_clearerr != STREAM_FCT_UNDEFINED)
-	(*pstm->fct_clearerr) (file);
+	CALL_CLEARERR(pstm);
     }
 
   PB_Init(pstm->pb_char);
@@ -1701,21 +1744,20 @@ Pl_Stream_Set_Position(StmInf *pstm, int whence, PlLong offset, PlLong char_coun
 int
 Pl_Stream_Set_Position_LC(StmInf *pstm, PlLong line_count, PlLong line_pos)
 {
-  PlLong file = pstm->file;
   int x;
   PlLong *p;
   int c;
-  int offset;
+  PlLong offset;
   Bool save_eof_reached;
-  int save_char_count, save_line_count, save_line_pos;
+  PlLong save_char_count, save_line_count, save_line_pos;
   int save_char_nb_elems;
 
 
-  offset = (*pstm->fct_tell) (file);
+  offset = CALL_TELL(pstm);
   if (offset < 0)
-    return offset;
+    return (int) offset;
 
-  x = (*pstm->fct_seek) (file, (PlLong) 0, SEEK_SET);
+  x = CALL_SEEK(pstm, 0, SEEK_SET);
   if (x != 0)
     return x;
 
@@ -1736,7 +1778,7 @@ Pl_Stream_Set_Position_LC(StmInf *pstm, PlLong line_count, PlLong line_pos)
     {
       pstm->eof_reached = FALSE;
       if (pstm->fct_clearerr != STREAM_FCT_UNDEFINED)
-	(*pstm->fct_clearerr) (file);
+	CALL_CLEARERR(pstm);
     }
 
   p = &(pstm->line_count);
@@ -1768,7 +1810,7 @@ err:
   pstm->line_pos = save_line_pos;
   pstm->pb_char.nb_elems = save_char_nb_elems;
 
-  x = (*pstm->fct_seek) (file, (PlLong) offset, SEEK_SET);
+  x = CALL_SEEK(pstm, (long) offset, SEEK_SET);
   if (x != 0)
     return x;
 
@@ -1842,7 +1884,7 @@ Pl_Add_Str_Stream(char *buff, int prop_other)
   stm = Find_Free_Stream();
   pstm = pl_stm_tbl[stm];
 
-  Init_Stream_Struct(atom_constant_term_stream, (PlLong) str_stream, prop,
+  Init_Stream_Struct(atom_constant_term_stream, str_stream, -1, prop,
 		     (StmFct) Str_Stream_Getc, (StmFct) Str_Stream_Putc,
 		     STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED,
 		     STREAM_FCT_UNDEFINED, STREAM_FCT_UNDEFINED,
@@ -1861,7 +1903,7 @@ Pl_Add_Str_Stream(char *buff, int prop_other)
 void
 Pl_Delete_Str_Stream(int stm)
 {
-  StrSInf *str_stream = (StrSInf *) (pl_stm_tbl[stm]->file);
+  StrSInf *str_stream = pl_stm_tbl[stm]->file;
 
   if (str_stream == &static_str_stream_rd ||
       str_stream == &static_str_stream_wr)
@@ -1875,7 +1917,7 @@ Pl_Delete_Str_Stream(int stm)
       Free(str_stream);
     }
 
-  Pl_Delete_Stream(stm);
+  Pl_Delete_Stream(stm, FALSE);
 }
 
 
@@ -1889,9 +1931,7 @@ Pl_Delete_Str_Stream(int stm)
 char *
 Pl_Term_Write_Str_Stream(int stm)
 {
-  StrSInf *str_stream;
-
-  str_stream = (StrSInf *) (pl_stm_tbl[stm]->file);
+  StrSInf *str_stream = pl_stm_tbl[stm]->file;
   *(str_stream->ptr) = '\0';
 
   return str_stream->buff;
@@ -1928,7 +1968,7 @@ Str_Stream_Getc(StrSInf *str_stream)
 static void
 Str_Stream_Putc(int c, StrSInf *str_stream)
 {
-  int size = str_stream->ptr - str_stream->buff;
+  int size = (int) (str_stream->ptr - str_stream->buff);
   int new_size;
 
 
