@@ -6,7 +6,7 @@
  * Descr.: term (inline) management - defs for meta-call                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -91,6 +91,11 @@ copy_term(T1, T2) :-
 
 
 
+nb_setarg(ArgNo, Term, NewValue) :-
+	set_bip_name(nb_setarg, 3),
+	'$call_c_test'('Pl_Setarg_4'(ArgNo, Term, NewValue, false)).
+
+
 setarg(ArgNo, Term, NewValue) :-
 	set_bip_name(setarg, 3),
 	'$call_c_test'('Pl_Setarg_4'(ArgNo, Term, NewValue, true)).
