@@ -6,7 +6,7 @@
  * Descr.: term (inline) management - C part                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -61,7 +61,7 @@ static PlLong *base_var_ptr;
  * Function Prototypes             *
  *---------------------------------*/
 
-static Bool Collect_Variable(WamWord *adr);
+static Bool Collect_Variable(WamWord *adr, WamWord var_word);
 
 static Bool Check_Variable(WamWord *adr, WamWord var_word);
 
@@ -78,9 +78,9 @@ static Bool Check_Variable(WamWord *adr, WamWord var_word);
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_Term_Eq(WamWord x, WamWord y)
+Pl_Blt_Term_Eq(WamWord x_word, WamWord y_word)
 {
-  return Pl_Term_Compare(x, y) == 0;
+  return Pl_Term_Compare(x_word, y_word) == 0;
 }
 
 
@@ -91,9 +91,9 @@ Pl_Blt_Term_Eq(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_Term_Neq(WamWord x, WamWord y)
+Pl_Blt_Term_Neq(WamWord x_word, WamWord y_word)
 {
-  return Pl_Term_Compare(x, y) != 0;
+  return Pl_Term_Compare(x_word, y_word) != 0;
 }
 
 
@@ -104,9 +104,9 @@ Pl_Blt_Term_Neq(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_Term_Lt(WamWord x, WamWord y)
+Pl_Blt_Term_Lt(WamWord x_word, WamWord y_word)
 {
-  return Pl_Term_Compare(x, y) < 0;
+  return Pl_Term_Compare(x_word, y_word) < 0;
 }
 
 
@@ -117,9 +117,9 @@ Pl_Blt_Term_Lt(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_Term_Lte(WamWord x, WamWord y)
+Pl_Blt_Term_Lte(WamWord x_word, WamWord y_word)
 {
-  return Pl_Term_Compare(x, y) <= 0;
+  return Pl_Term_Compare(x_word, y_word) <= 0;
 }
 
 
@@ -130,9 +130,9 @@ Pl_Blt_Term_Lte(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_Term_Gt(WamWord x, WamWord y)
+Pl_Blt_Term_Gt(WamWord x_word, WamWord y_word)
 {
-  return Pl_Term_Compare(x, y) > 0;
+  return Pl_Term_Compare(x_word, y_word) > 0;
 }
 
 
@@ -143,9 +143,9 @@ Pl_Blt_Term_Gt(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_Term_Gte(WamWord x, WamWord y)
+Pl_Blt_Term_Gte(WamWord x_word, WamWord y_word)
 {
-  return Pl_Term_Compare(x, y) >= 0;
+  return Pl_Term_Compare(x_word, y_word) >= 0;
 }
 
 
@@ -156,15 +156,15 @@ Pl_Blt_Term_Gte(WamWord x, WamWord y)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool FC
-Pl_Blt_Compare(WamWord cmp_word, WamWord x, WamWord y)
+Pl_Blt_Compare(WamWord cmp_word, WamWord x_word, WamWord y_word)
 {
-  int cmp;
+  PlLong cmp;
   char c;
   Bool res;
 
   Pl_Set_C_Bip_Name("compare", 3);
 
-  cmp = Pl_Term_Compare(x, y);
+  cmp = Pl_Term_Compare(x_word, y_word);
   c = (cmp < 0) ? '<' : (cmp == 0) ? '=' : '>';
 
   res = Pl_Un_Atom_Check(ATOM_CHAR(c), cmp_word);
@@ -274,8 +274,8 @@ Pl_Blt_Functor(WamWord term_word, WamWord functor_word, WamWord arity_word)
 
   if (tag_functor == TAG_ATM_MASK && arity > 0)
     {
-      res = (Pl_Get_Structure(UnTag_ATM(functor_word), arity, term_word)) ?
-	Pl_Unify_Void(arity), TRUE : FALSE;
+      res = (Pl_Get_Structure(UnTag_ATM(functor_word), (int )arity, term_word)) ?
+	Pl_Unify_Void((int) arity), TRUE : FALSE;
       goto finish;
     }
 
@@ -487,13 +487,13 @@ Pl_Copy_Term_2(WamWord u_word, WamWord v_word)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Setarg_4(WamWord arg_no_word, WamWord term_word, WamWord new_value_word,
-	 WamWord undo_word)
+	    WamWord undo_word)
 {
   WamWord word, tag_mask;
   int func, arity;
   int undo;
   WamWord *arg_adr;
-  int arg_no;
+  PlULong arg_no;
 
   arg_adr = Pl_Rd_Compound_Check(term_word, &func, &arity);
   arg_no = Pl_Rd_Positive_Check(arg_no_word) - 1;
@@ -503,7 +503,7 @@ Pl_Setarg_4(WamWord arg_no_word, WamWord term_word, WamWord new_value_word,
   if (!undo && tag_mask != TAG_ATM_MASK && tag_mask != TAG_INT_MASK)
     Pl_Err_Type(pl_type_atomic, word);	/* pl_type_atomic but float not allowed */
 
-  if ((unsigned) arg_no >= (unsigned) arity)
+  if (arg_no >= arity)		/* include arg_no < 0 since PlULong */
     return FALSE;
 
   if (undo)
@@ -613,7 +613,7 @@ Pl_Term_Variables_2(WamWord start_word, WamWord list_word)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static Bool
-Collect_Variable(WamWord *adr)
+Collect_Variable(WamWord *adr, WamWord var_word)
 {
   PlLong *p;
 
