@@ -6,7 +6,7 @@
  * Descr.: term support                                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -95,7 +95,8 @@ PlLong Pl_Term_Compare(WamWord start_u_word, WamWord start_v_word)
   WamWord *u_arg_adr;
   int v_func, v_arity;
   WamWord *v_arg_adr;
-  int i, x;
+  int i;
+  PlLong x;
   double d1, d2;
 
   DEREF(start_u_word, u_word, u_tag_mask);
@@ -185,7 +186,7 @@ PlLong Pl_Term_Compare(WamWord start_u_word, WamWord start_v_word)
  * Call fct for each variable found in a term.                             *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Treat_Vars_Of_Term(WamWord start_word, Bool generic_var, Bool (*fct) ())
+Pl_Treat_Vars_Of_Term(WamWord start_word, Bool generic_var, Bool (*fct) (WamWord *adr, WamWord word))
 {
   WamWord word, tag_mask;
   WamWord *adr;
@@ -481,7 +482,7 @@ terminal_rec:
  *-------------------------------------------------------------------------*/
 void
 Pl_Copy_Contiguous_Term(WamWord *dst_adr, WamWord *src_adr)
-#define Old_Adr_To_New_Adr(adr)  ((dst_adr)+((adr)-(src_adr)))
+#define Old_Adr_To_New_Adr(adr)  ((dst_adr) + ((adr) - (src_adr)))
 {
   WamWord word, *adr;
   WamWord *q;
@@ -625,7 +626,7 @@ Pl_Acyclic_Term_1(WamWord start_word)
 /*-------------------------------------------------------------------------*
  * PL_ACYCLIC_TERM_1                                                       *
  *                                                                         *
- * This is implemented in linear time markink each sub-term (LST or STC)   *
+ * This is implemented in linear time marking each sub-term (LST or STC)   *
  * with a special mark (NB we cannot use NOT_A_WAM_WORD, <REF,0> since it  * 
  * would be dereferenced. So we use <LST,0>                                *
  *                                                                         *
@@ -878,8 +879,7 @@ terminal_rec:
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Term_Hash_4(WamWord start_word, WamWord depth_word, WamWord range_word, 
-	       WamWord hash_word)
+Pl_Term_Hash_4(WamWord start_word, WamWord depth_word, WamWord range_word, WamWord hash_word)
 {
   PlLong depth = Pl_Rd_Integer_Check(depth_word);
   PlLong range = Pl_Rd_Positive_Check(range_word);
@@ -961,7 +961,7 @@ Pl_Get_Pred_Indicator(WamWord pred_indic_word, Bool must_be_ground, int *arity)
       if (arity1 > MAX_ARITY)
 	Pl_Err_Representation(pl_representation_max_arity);
 
-      *arity = arity1;
+      *arity = (int) arity1;
     }
 
   return func;
