@@ -6,7 +6,7 @@
  * Descr.: top Level                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -38,21 +38,30 @@
 
 :-	built_in.
 
-:-	ensure_linked([consult / 1, load / 1]).
+:-	ensure_linked([consult/1, load/1]).
 
 
 top_level :-
+	(   current_prolog_flag(show_banner, on) ->
+	    '$banner'(top_level_output),
+	    nl(top_level_output)
+	;   true
+	),
+	break.
+
+
+
+
+'$banner'(Stream) :-
 	current_prolog_flag(prolog_name, Name),
 	current_prolog_flag(prolog_version, Version),
 	current_prolog_flag(prolog_copyright, Copyright),
 	current_prolog_flag(address_bits, Bits),
 	current_prolog_flag(compiled_at, Date),
 	current_prolog_flag(c_cc, CC),
-	format(top_level_output, '~N~a ~a (~d bits)~n', [Name, Version, Bits]),
-	format(top_level_output, 'Compiled ~a with ~a~n', [Date, CC]),
-	format(top_level_output, '~a~n', [Copyright]),
-	nl,
-	break.
+	format(Stream, '~N~a ~a (~d bits)~n', [Name, Version, Bits]),
+	format(Stream, 'Compiled ~a with ~a~n', [Date, CC]),
+	format(Stream, '~a~n', [Copyright]).
 
 
 
@@ -129,11 +138,13 @@ break :-
 '$reinit_after_exception' :-
 % g_read('$char_conv', CharConv),
 % set_prolog_flag(char_conversion, CharConv),
-	(   '$sys_var_read'(12, 1) ->
-	    g_read('$user_prompt', UserPrompt),
-	    '$set_linedit_prompt'(UserPrompt)
-	;   true
-	).
+	'$sys_var_read'(12, 1),
+	g_read('$user_prompt', UserPrompt),
+	atom(UserPrompt),
+	'$set_linedit_prompt'(UserPrompt),
+	fail.
+
+'$reinit_after_exception'.
 
 
 
@@ -156,6 +167,7 @@ break :-
 	;   write(top_level_output, Prompt)
 	),
 	flush_output(top_level_output),
+	'$call_c'('Pl_Save_Regs_For_Signal'),  % save some registers in case of CTRL+C
 	'$read_query'(X, QueryVars),
 	(   '$sys_var_read'(12, 1) ->
 	    '$set_linedit_prompt'(UserPrompt)
@@ -222,12 +234,16 @@ break :-
 	Prompt = '| ?- ',
 	(   '$sys_var_read'(12, 1) ->
 	    write(top_level_output, Prompt)
-	;   true),
+	;   true
+	),
 	format(top_level_output, '~a.~n', [Goal]),
 	read_term_from_atom(Goal, X, [end_of_term(eof), variable_names(QueryVars)]).
 
 '$read_query'(X, QueryVars) :-
-	read_term(top_level_input, X, [variable_names(QueryVars)]),
+	read_term(top_level_input, X, [variable_names(QueryVars)]), !,
+	'$PB_empty_buffer'(top_level_input).
+
+'$read_query'(fail, []) :-   % read_term can fail if flag syntax_error = fail
 	'$PB_empty_buffer'(top_level_input).
 
 
@@ -248,8 +264,7 @@ break :-
 	(   fail,                             % do not activate 'alt if vars'
 	    ToDispVars = [] ->
 	    true                              % no alt if only anonymous vars
-	;
-	    '$write_solution'(ToDispVars, B1, B),
+	;   '$write_solution'(ToDispVars, B1, B),
 	    (   B1 > B ->
 	        g_read('$all_solutions', f),          % fail for previous 'a'
 	        write(top_level_output, ' ? '),
@@ -293,7 +308,7 @@ break :-
 	).
 
 '$write_solution'(ToDispVars, _, _) :-
-	( current_op(Prior, xfx, =) ; Prior = 700), !,
+	( current_op(Prior, xfx, =) ; Prior = 700 ), !,
 	Prior1 is Prior - 1,
 	'$write_solution1'(ToDispVars, Prior1).
 
@@ -303,7 +318,9 @@ break :-
 '$write_solution1'([Name = Value|ToDispVars], Prior) :-
 	(   acyclic_term(Value) ->
 	    format(top_level_output, '~n~a = ', [Name]),
-	    write_term(top_level_output, Value, [quoted(true), numbervars(false), namevars(true), priority(Prior)])
+	    write_term(top_level_output, Value,
+		       [quoted(true), numbervars(false), namevars(true), priority(Prior), space_args(true)]),
+	    flush_output(top_level_output)
 	;
 	    format(top_level_output, '~ncannot display cyclic term for ~a', [Name])
 	),
@@ -346,9 +363,10 @@ break :-
 /* interface with command-line option consulting files */
 
 '$exec_cmd_line_consult_files'([File|_LFile]) :-
-	'$catch_internal'('$consult2'(File), error(Err, _), true, false),
-	nonvar(Err),
-	format('~Nwarning: command-line consulting file ~q failed due to ~q~n', [File, Err]),
+%	'$catch_internal'(consult(File), error(Err, _), true, false),
+%	nonvar(Err),
+%	format(top_level_output, '~Nwarning: command-line consulting file ~q failed due to ~q~n', [File, Err]),
+	'$exec_cmd_consult_file'(File),
 	fail.
 	
 '$exec_cmd_line_consult_files'([_|LFile]) :-
@@ -358,29 +376,61 @@ break :-
 '$exec_cmd_line_consult_files'(_).		% can be another term than []
 
 
+'$exec_cmd_consult_file'(File) :-
+	% BEWARE: do not remove this '$sys_var_write'(11, B),
+	% e.g. important for a commannd-line goal calling abort
+	'$get_current_B'(B),	% the current choice-point
+	'$sys_var_write'(11, B),
+	'$catch_internal'(consult(File), error(Err, _), true, false),
+	nonvar(Err),
+	format(top_level_output, '~Nwarning: command-line consulting file ~q failed due to ~q~n', [File, Err]),
+	fail.
+
+'$exec_cmd_line_consult_file'(_).	% BEWARE: to create a choice-point
+
+
 /* interface with command-line options executing goals */
 
 '$exec_cmd_line_entry_goals'([Goal|LGoal]):-
-	!,
-	'$exec_cmd_line_goal'(Goal),
+	% (
+	% '$get_current_B'(B),   % the current choice-point
+	% '$sys_var_write'(11, B),
+				%   '$exec_cmd_line_goal'(Goal) ; true ), !,
+	'$exec_cmd_line_entry_goal'(Goal),
 	'$exec_cmd_line_entry_goals'(LGoal).
 
 '$exec_cmd_line_entry_goals'(_).		% can be another term than []
 
 
+'$exec_cmd_line_entry_goal'(Goal) :-
+	% BEWARE: do not remove this '$sys_var_write'(11, B),
+	% e.g. important for a commannd-line goal calling abort
+	'$get_current_B'(B),	% the current choice-point
+	'$sys_var_write'(11, B),
+	'$exec_cmd_line_goal'(Goal), !.
+
+'$exec_cmd_line_entry_goal'(_).	% BEWARE: create a choice-point (nothing actually fails)
 
 
-'$exec_cmd_line_goal'(Goal) :-		% called by top_level.c
+
+
+'$exec_cmd_line_goal'(Goal) :-		% also called by top_level_main.c for --init-goal
 	(   '$catch'('$exec_cmd1'(Goal), Err, '$exec_cmd_err'(Goal, Err), 'command-line', -1, false) ->
 	    true
-	;   format('~Nwarning: command-line goal ~q failed~n', [Goal])).
+	;   format(top_level_output, '~Nwarning: command-line goal ~q failed~n', [Goal])
+	).
+
+
 
 
 '$exec_cmd1'(Goal) :-
 	read_term_from_atom(Goal, TermGoal, [end_of_term(eof)]),
+	'$call_c'('Pl_Save_Regs_For_Signal'),  % save some registers in case of CTRL+C (preventive)
 	'$call'(TermGoal, 'command-line', -1, false).
 
 
+
+
 '$exec_cmd_err'(Goal, Err) :-
-	format('~Nwarning: command-line goal ~q caused exception: ~q~n', [Goal, Err]).
+	format(top_level_output, '~Nwarning: command-line goal ~q caused exception: ~q~n', [Goal, Err]).
 
