@@ -6,7 +6,7 @@
  * Descr.: write term support                                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -59,35 +59,35 @@
 
 
 
- /* The output of the term -(T) using operator notation requires some attention if 
-  * the notation representation of T starts with a number. It is important to disinguish
+ /* The output of the term -(T) using operator notation requires some attention if the
+  * notation representation of T starts with a number. It is important to disinguish
   * between the compound term -(1) and the integer -1.
   *
-  * If MINUS_SIGN_CAN_BE_FOLLOWED_BY_SPACES is not defined, we can simply use a space
+  * If MINUS_SIGN_CAN_BE_FOLLOWED_BY_SPACES is not defined, we can simply use a space.
   *    -(1) can be output as - 1
   *
   * If MINUS_SIGN_CAN_BE_FOLLOWED_BY_SPACES is defined we need brackets. 
-  *    -(1) can be output as - (1)    NB: -(1) is also OK but does not show the op notation
+  *    -(1) can be output as - (1) NB: -(1) is also OK but does not show the op notation
   * 
-  * The following macros control how brackets are handled around T (or part of T). For this
-  * we consider 2 cases (pointed out by Ulrich Neumerkel).
+  * The following macros control how brackets are handled around T (or part of T). 
+  * For this we consider 2 cases (pointed out by Ulrich Neumerkel).
   *
   *    - (1^2) which is -(^(1,2)) and can produce - (1^2) or - (1)^2
   *    - (a^2) which is -(^(a,2)) and can produce - (a^2) or -a^2 
   *
-  * OP_MINUS_BRACKETS_SIMPLE: to homegenize the output and to simplify the implementation, 
-  * brackets are used around T if T is a positive number or an {infix,postifx} op term
+  * OP_MINUS_BRACKETS_SIMPLE: to homegenize the output and to simplify the implementation,
+  * brackets are used around T if T is a positive number or an {infix,postifx} op term.
   *
   *    writeq(- (1^2)) produces - (1^2)
   *    writeq(- (a^2)) produces - (a^2)
   *
-  * OP_MINUS_BRACKETS_SHORTEST: avoids useless brackets else the bracketed is as short as 
-  * possible. opening ( is before T, and closing ) can be inside T
+  * OP_MINUS_BRACKETS_SHORTEST: avoid useless brackets else the bracketed is as short as
+  * possible. opening ( is before T, and closing ) can be inside T.
   *
   *    writeq(- (1^2)) produces - (1)^2
   *    writeq(- (a^2)) produces -a^2
   *
-  * OP_MINUS_BRACKETS_MIXED: avoids useless brackets else the whole T is bracketed.
+  * OP_MINUS_BRACKETS_MIXED: avoid useless brackets else the whole T is bracketed.
   *
   *    writeq(- (1^2)) produces - (1^2)   as in OP_MINUS_BRACKETS_SIMPLE
   *    writeq(- (a^2)) produces -a^2      as in OP_MINUS_BRACKETS_SHORTEST
@@ -102,6 +102,26 @@
 #endif
 
 
+/* max_depth discussion: https://github.com/didoudiaz/gprolog/issues/56 */
+
+		/* ... (ellipsis) stands for a non-variable term */
+#if 1
+#define NO_ELLIPSIS_FOR_VAR
+#endif
+
+		/* first element of list with same depth as the list */
+#if 1
+#define FIRST_ELEM_OF_LIST_WITH_SAME_DEPTH
+#endif
+
+		/* allow f(...) for f(a,b,c) almost all systems do that (except SWI) */
+#if 1
+#define ELLIPSIS_MORE_THAN_ONE_TERM_IN_STC
+#endif
+
+
+
+
 /*---------------------------------*
  * Constants                       *
  *---------------------------------*/
@@ -165,7 +185,7 @@ static Bool *p_bracket_op_minus;
  * Function Prototypes             *
  *---------------------------------*/
 
-static void Need_Space(int c);
+static void Emit_Space_If_Needed(int c);
 
 static void Out_Space(void);
 
@@ -175,6 +195,12 @@ static void Out_String(char *str);
 
 static void Show_Term(int depth, int prec, int context, WamWord term_word);
 
+static int Get_Var_Name_From_Stc(WamWord f_n, WamWord *stc_adr);
+
+static int Get_Var_Number_From_Stc(WamWord f_n, WamWord *stc_adr);
+
+static Bool Is_Rendered_As_Var(WamWord word);
+
 static void Show_Global_Var(WamWord *adr);
 
 #ifndef NO_USE_FD_SOLVER
@@ -189,7 +215,7 @@ static void Show_Float(double x);
 
 static void Show_Number_Str(char *str);
 
-static void Show_List_Arg(int depth, WamWord *lst_adr);
+static void Show_List_Elements(int depth, WamWord *lst_adr);
 
 static void Show_Structure(int depth, int prec, int context, WamWord *stc_adr);
 
@@ -197,6 +223,15 @@ static Bool Try_Portray(WamWord word);
 
 
 
+#ifdef SPACE_ARGS_FOR_LIST_PIPE
+#define SHOW_LIST_PIPE do { if (space_args) Out_String(" | "); else Out_Char('|'); } while(0)
+#else
+#define SHOW_LIST_PIPE Out_Char('|')
+#endif
+
+
+#define SHOW_COMMA   do { Out_Char(','); if (space_args) Out_Space(); } while(0)
+
 
 /*-------------------------------------------------------------------------*
  * WRITE_SUPP_INITIALIZER                                                  *
@@ -237,6 +272,9 @@ Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord *above_H,
   last_is_space = FALSE;
   last_prefix_op = W_NO_PREFIX_OP;
   pl_last_writing = W_NOTHING;
+  
+  if (depth == 0)		/* no limit */
+    depth = -1;
 
   Show_Term(depth, prec, (prec >= 1200) ? GENERAL_TERM : INSIDE_ANY_OP, term_word);
 }
@@ -246,20 +284,39 @@ Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord *above_H,
 
 /*-------------------------------------------------------------------------*
  * PL_WRITE                                                                *
- *                                                                         *
+ * useful for debugging                                                    *
  *-------------------------------------------------------------------------*/
 void
 Pl_Write(WamWord term_word)
 {
-  StmInf *pstm = pl_stm_tbl[pl_stm_output];
+  StmInf *pstm = pl_stm_tbl[pl_stm_current_output];
 
-  Pl_Write_Term(pstm, -1, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS, NULL, term_word);
+  Pl_Write_Term(pstm, 0, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS,
+		NULL, term_word);
   /* like write/1 */
 }
 
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_WRITELN                                                              *
+ * useful for debugging                                                    *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Writeln(WamWord term_word)
+{
+  StmInf *pstm = pl_stm_tbl[pl_stm_current_output];
+
+  Pl_Write_Term(pstm, 0, MAX_PREC, WRITE_NUMBER_VARS | WRITE_NAME_VARS,
+		NULL, term_word);
+  Pl_Nl_0();
+  /* like write/1+nl/0 */
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * OUT_SPACE                                                               *
  *                                                                         *
@@ -285,9 +342,9 @@ Out_Space(void)
 static void
 Out_Char(int c)
 {
-  Need_Space(c);
+  Emit_Space_If_Needed(c);
   Pl_Stream_Putc(c, pstm_o);
-#if 0		     /* actually, we do not use Out_Char to display spaces */
+#if 0		     /* currently, we do not use Out_Char to display spaces */
   last_is_space = (c == ' ');  /* use isspace ? */
 #else
   last_is_space = FALSE;
@@ -304,7 +361,7 @@ Out_Char(int c)
 static void
 Out_String(char *str)
 {
-  Need_Space(*str);
+  Emit_Space_If_Needed(*str);
   Pl_Stream_Puts(str, pstm_o);
 
  /* Do not take into account space in strings , e.g.
@@ -323,14 +380,14 @@ Out_String(char *str)
 
 
 /*-------------------------------------------------------------------------*
- * NEED_SPACE                                                              *
+ * EMIT_SPACE_IF_NEEDED                                                    *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-Need_Space(int c)
+Emit_Space_If_Needed(int c)
 {
   int c_type = pl_char_type[c];
-  int space;
+  Bool space;
 
   switch (pl_last_writing)
     {
@@ -478,7 +535,11 @@ Show_Term(int depth, int prec, int context, WamWord term_word)
   WamWord word, tag_mask;
   WamWord *adr;
 
-  if (depth == 0)
+  if (depth == 0
+#ifdef NO_ELLIPSIS_FOR_VAR
+      && !Is_Rendered_As_Var(term_word)
+#endif
+      )
     {
       Show_Atom(GENERAL_TERM, atom_dots);
       return;
@@ -524,14 +585,14 @@ Show_Term(int depth, int prec, int context, WamWord term_word)
 	{
 	  Out_String("'.'(");
 	  Show_Term(depth - 1, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, Car(adr));
-	  Out_Char(',');
+	  SHOW_COMMA;
 	  Show_Term(depth - 1, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, Cdr(adr));
 	  Out_Char(')');
 	}
       else
 	{
 	  Out_Char('[');
-	  Show_List_Arg(depth, adr);
+	  Show_List_Elements(depth, adr);
 	  Out_Char(']');
 	}
       break;
@@ -546,6 +607,126 @@ Show_Term(int depth, int prec, int context, WamWord term_word)
 
 
 
+/*-------------------------------------------------------------------------*
+ * GET_VAR_NAME_FROM_STC                                                   *
+ *                                                                         *
+ * return the atom of '$VARNAME'(ATOM) (or -1 if none)                     *
+ *-------------------------------------------------------------------------*/
+static int
+Get_Var_Name_From_Stc(WamWord f_n, WamWord *stc_adr)
+{
+  WamWord word, tag_mask;
+
+  if (name_vars && f_n == dollar_varname_1 && stc_adr >= name_number_above_H)
+    {
+      DEREF(Arg(stc_adr, 0), word, tag_mask);
+      if (tag_mask == TAG_ATM_MASK)
+	{
+	  int atom = UnTag_ATM(word);
+#if 0				/* check the validity of the atom */
+	  if (Is_Valid_Var_Name(pl_atom_tbl[atom].name))
+	    return atom;
+#else  				/* accept any atom - call Show_Atom to set pl_last_writing */
+	  return atom;
+#endif
+	}
+    }
+  return -1;			/* not an atom */
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * GET_VAR_NUMBER_FROM_STC                                                 *
+ *                                                                         *
+ * return the number N from '$VAR'(N) (or -1 if none)                      *
+ *-------------------------------------------------------------------------*/
+static int
+Get_Var_Number_From_Stc(WamWord f_n, WamWord *stc_adr)
+{
+  WamWord word, tag_mask;
+  int n;
+
+  if (number_vars && f_n == dollar_var_1 && stc_adr >= name_number_above_H)
+    {
+      DEREF(Arg(stc_adr, 0), word, tag_mask);
+      if (tag_mask == TAG_INT_MASK && (n = (int) UnTag_INT(word)) >= 0)
+	return n;
+    }
+  return -1;
+}
+
+
+
+
+#ifdef FIRST_ELEM_OF_LIST_WITH_SAME_DEPTH
+
+/*-------------------------------------------------------------------------*
+ * IS_RENDERED_AS_COMPOUND                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static Bool
+Is_Rendered_As_Compound(WamWord term_word)
+{
+  WamWord word, tag_mask;
+  
+  DEREF(term_word, word, tag_mask);
+
+  if (tag_mask == TAG_LST_MASK)
+    return TRUE;
+
+  if (tag_mask != TAG_STC_MASK)
+    return FALSE;
+ 
+  WamWord *stc_adr = UnTag_STC(word);
+  WamWord f_n = Functor_And_Arity(stc_adr);
+
+  if (Get_Var_Name_From_Stc(f_n, stc_adr) >= 0)
+    return FALSE;
+
+  if (Get_Var_Number_From_Stc(f_n, stc_adr) >= 0)
+    return FALSE;
+  
+  return TRUE;
+}
+
+#endif
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * IS_RENDERED_AS_VAR                                                      *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static Bool
+Is_Rendered_As_Var(WamWord term_word)
+{
+  WamWord word, tag_mask;
+  
+  DEREF(term_word, word, tag_mask);
+  if (tag_mask == TAG_REF_MASK)
+    return TRUE;
+
+  if (tag_mask != TAG_STC_MASK)
+    return FALSE;
+  
+  WamWord *stc_adr = UnTag_STC(word);
+  WamWord f_n = Functor_And_Arity(stc_adr);
+
+  if (Get_Var_Name_From_Stc(f_n, stc_adr) >= 0)
+    return TRUE;
+
+  if (Get_Var_Number_From_Stc(f_n, stc_adr) >= 0)
+    return TRUE;
+  
+  return FALSE;
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * SHOW_GLOBAL_VAR                                                         *
  *                                                                         *
@@ -752,32 +933,52 @@ Show_Number_Str(char *str)
 
 
 /*-------------------------------------------------------------------------*
- * SHOW_LIST_ARG                                                           *
+ * SHOW_LIST_ELEMENTS                                                      *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-
-#ifdef SPACE_ARGS_FOR_LIST_PIPE
-#define SHOW_LIST_PIPE if (space_args) Out_String(" | "); else Out_Char('|')
-#else
-#define SHOW_LIST_PIPE Out_Char('|')
-#endif
-
 static void
-Show_List_Arg(int depth, WamWord *lst_adr)
+Show_List_Elements(int depth, WamWord *lst_adr)
 {
+  Bool first_elem = TRUE;
   WamWord word, tag_mask;
+  WamWord car_word;
+  Bool emit_car;
 
+  /* at function entry depth > 0 */
  terminal_rec:
-  depth--;
+  car_word = Car(lst_adr);
 
-  Show_Term(depth, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, Car(lst_adr));
-
-  if (depth == 0)		/* dots already written by Show_Term */
-    return;
+#ifdef FIRST_ELEM_OF_LIST_WITH_SAME_DEPTH
+  if (depth > 0 && (!first_elem || Is_Rendered_As_Compound(car_word)
+#ifdef NO_ELLIPSIS_FOR_VAR
+		    || Is_Rendered_As_Var(car_word)
+#endif
+		    ))
+#endif
+    depth--;
+  
+  emit_car = (depth != 0 || first_elem || Is_Rendered_As_Var(car_word));
+  if (emit_car)
+    {
+      if (!first_elem)
+	SHOW_COMMA;
 
+      Show_Term(depth, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, car_word);
+    }
 
   DEREF(Cdr(lst_adr), word, tag_mask);
+  if (depth == 0)
+    {
+      if (!emit_car || word != NIL_WORD)
+	{
+	  SHOW_LIST_PIPE;
+	  Show_Atom(GENERAL_TERM, atom_dots);
+	}
+      return;
+    }
 
+  first_elem = FALSE;
+  
   switch (Tag_From_Tag_Mask(tag_mask))
     {
     case REF:
@@ -823,9 +1024,6 @@ Show_List_Arg(int depth, WamWord *lst_adr)
       break;
 
     case LST:
-      Out_Char(',');
-      if (space_args)
-	Out_Space();
       lst_adr = UnTag_LST(word);
       goto terminal_rec;
       break;
@@ -834,9 +1032,7 @@ Show_List_Arg(int depth, WamWord *lst_adr)
       SHOW_LIST_PIPE;
       if (Try_Portray(word))
 	return;
-
-      Show_Structure(depth, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM,
-		     UnTag_STC(word));
+      Show_Structure(depth, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, UnTag_STC(word));
       break;
     }
 }
@@ -893,63 +1089,47 @@ Pl_Is_Valid_Var_Name_1(WamWord name_word)
 static void
 Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 {
-  WamWord word, tag_mask;
   WamWord *adr;
   WamWord f_n = Functor_And_Arity(stc_adr);
   int functor = Functor(stc_adr);
   int arity = Arity(stc_adr);
+  int atom_of_varname;
+  int n, i, j;
   OperInf *oper;
   int nb_args_to_disp;
-  int i, j, n;
   char str[32];
   Bool bracket;
   Bool surround_space;
 
-
+			/* at function entry depth > 0 */
   depth--;
 
-  if (name_vars && f_n == dollar_varname_1 && stc_adr >= name_number_above_H)
-    {
-      DEREF(Arg(stc_adr, 0), word, tag_mask);
-      if (tag_mask == TAG_ATM_MASK)
-	{
-#if 0				/* check the validity of the atom */
-	  char *p = pl_atom_tbl[UnTag_ATM(word)].name;
-	  if (Is_Valid_Var_Name(p))
-	    {
-	      Out_String(p);
-	      pl_last_writing = W_IDENTIFIER;
-	      return;
-	    }
-#else  				/* accept any atom - call Show_Atom to set pl_last_writing */
-	  int save_quoted = quoted;
-	  quoted = FALSE;
-	  Show_Atom(GENERAL_TERM, UnTag_ATM(word)); /* could pass context instead of GENERAL_TERM */
-	  quoted = save_quoted;	      
-	  return;
-#endif
-	}
+  atom_of_varname = Get_Var_Name_From_Stc(f_n, stc_adr);
+  if (atom_of_varname >= 0)	/* '$VARNAME'(ATOM) */
+    {				/* could be simplified with Out_String() if we know ATOM is a valid var name */
+      int save_quoted = quoted;
+      quoted = FALSE;
+      Show_Atom(GENERAL_TERM, atom_of_varname); /* could pass context instead of GENERAL_TERM */
+      quoted = save_quoted;	      
+      return;
     }
 
-  if (number_vars && f_n == dollar_var_1 && stc_adr >= name_number_above_H)
+  n = Get_Var_Number_From_Stc(f_n, stc_adr);
+  if (n >= 0)			/* '$VAR'(N) */
     {
-      DEREF(Arg(stc_adr, 0), word, tag_mask);
-      if (tag_mask == TAG_INT_MASK && (n = UnTag_INT(word)) >= 0)
+      i = n % 26;
+      j = n / 26;
+
+      Out_Char('A' + i);
+
+      if (j)
 	{
-	  i = n % 26;
-	  j = n / 26;
-
-	  Out_Char('A' + i);
-
-	  if (j)
-	    {
-	      sprintf(str, "%d", j);
-	      Out_String(str);
-	    }
-
-	  pl_last_writing = W_IDENTIFIER;
-	  return;
+	  sprintf(str, "%d", j);
+	  Out_String(str);
 	}
+
+      pl_last_writing = W_IDENTIFIER;
+      return;
     }
 
   if (ignore_op || arity > 2)
@@ -972,9 +1152,10 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
   if (arity == 1 && (oper = Pl_Lookup_Oper(functor, PREFIX)))
     {
 #if 1
-      /* Koen de Bosschere says "in case of ambiguity :          */
-      /* select the associative operator over the nonassociative */
-      /* select prefix over postfix".                            */
+      /* Koen de Bosschere: "in case of ambiguity :
+       * select the associative operator over the nonassociative,
+       * select prefix over postfix".                            
+       */
 
       OperInf *oper1;
 
@@ -1021,8 +1202,8 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
        * The term ((-(1)) & b must be displayed as: (- (1)) & b
        * Concerning the sub-term - (1), the first ( is emitted  10 lines above
        * because the precedence of - (200) is > precedence of & (100).
-       * The second ( is emitted by Need_Space() because the argument of - begins 
-       * by a digit. At the return we have to close 2 ).
+       * The second ( is emitted by Emit_Space_If_Needed() because the 
+       * argument of - begins by a digit. At the return we have to close 2 ).
        */
 
       while (bracket--)	
@@ -1045,8 +1226,7 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 	  bracket = TRUE;
 	}
 
-      context =
-	(oper->left == oper->prec) ? INSIDE_LEFT_ASSOC_OP : INSIDE_ANY_OP;
+      context = (oper->left == oper->prec) ? INSIDE_LEFT_ASSOC_OP : INSIDE_ANY_OP;
 
       Show_Term(depth, oper->left, context, Arg(stc_adr, 0));
 
@@ -1080,13 +1260,12 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 	  bracket = TRUE;
 	}
 
-      context =
-	(oper->left == oper->prec) ? INSIDE_LEFT_ASSOC_OP : INSIDE_ANY_OP;
+      context =	(oper->left == oper->prec) ? INSIDE_LEFT_ASSOC_OP : INSIDE_ANY_OP;
 
       Show_Term(depth, oper->left, context, Arg(stc_adr, 0));
 
-#if 1 /* to show | unquoted if it is an infix operator with prec > 1000 */
-      if (functor == ATOM_CHAR('|') && oper->prec > 1000)
+#if 1 /* to show | unquoted if it is an infix operator (thus prec > 1000) */
+      if (functor == ATOM_CHAR('|'))
 	{
 	  if (space_args)
 	    Out_Space();
@@ -1098,9 +1277,7 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 #endif
 	if (functor == ATOM_CHAR(','))
 	  {
-	    Out_Char(',');
-	    if (space_args)
-	      Out_Space();
+	    SHOW_COMMA;
 	  }
 	else
 	  {
@@ -1139,16 +1316,19 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
   Show_Atom(GENERAL_TERM, functor);
   Out_Char('(');
 
-  nb_args_to_disp = i = (arity < depth + 1 || depth < 0) ? arity : depth + 1;
+#ifdef ELLIPSIS_MORE_THAN_ONE_TERM_IN_STC /* limit #ags f(1, 2, 3) can be displayed f(...) or f(1, ...)  */
+  nb_args_to_disp = (depth < 0 || arity < depth + 1) ? arity : depth + 1;
+#else
+  nb_args_to_disp = arity;	/* do no limit #args f(1, 2, 3) can be f(..., ..., ...) */
+#endif
+  i = nb_args_to_disp;
   adr = &Arg(stc_adr, 0);
 
   goto start_display;
 
   do
     {
-      Out_Char(',');
-      if (space_args)
-	Out_Space();
+      SHOW_COMMA;
     start_display:
       Show_Term(depth, MAX_ARG_OF_FUNCTOR_PREC, GENERAL_TERM, *adr++);
     }
@@ -1156,9 +1336,7 @@ Show_Structure(int depth, int prec, int context, WamWord *stc_adr)
 
   if (arity != nb_args_to_disp)
     {
-      Out_Char(',');
-      if (space_args)
-	Out_Space();
+      SHOW_COMMA;
       Show_Atom(GENERAL_TERM, atom_dots);
     }
 
@@ -1237,7 +1415,7 @@ Pl_Get_Print_Stm_1(WamWord stm_word)
   int stm = Pl_Find_Stream_From_PStm(pstm_o);
 
   if (stm < 0)
-    stm = pl_stm_output;
+    stm = pl_stm_current_output;
 
   return Pl_Get_Integer(stm, stm_word);
 }
