@@ -6,7 +6,7 @@
  * Descr.: FD instruction implementation                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -174,8 +174,8 @@ static PlULong optim2_date_always = DATE_ALWAYS; /* must be considered as == all
 
 
 
-void (*pl_fd_init_solver) () = Pl_Fd_Init_Solver0;	/* overwrite var of if_no_fd.c */
-void (*pl_fd_reset_solver) () = Pl_Fd_Reset_Solver0;	/* overwrite var of if_no_fd.c */
+void (*pl_fd_init_solver) (void) = Pl_Fd_Init_Solver0;	/* overwrite var of if_no_fd.c */
+void (*pl_fd_reset_solver) (void) = Pl_Fd_Reset_Solver0;	/* overwrite var of if_no_fd.c */
 
 
 
@@ -244,7 +244,6 @@ static void Clear_Queue(void);
 
 
 
-
 #define Trail_Chains_If_Necessary(fdv_adr)           		\
   do								\
     {								\
@@ -462,7 +461,7 @@ Pl_Fd_Prolog_To_Fd_Var(WamWord arg_word, Bool pl_var_ok)
     }
 
   if (tag_mask == TAG_INT_MASK)
-    return Pl_Fd_New_Int_Variable(UnTag_INT(word));
+    return Pl_Fd_New_Int_Variable((int) UnTag_INT(word));
 
   if (tag_mask != TAG_FDV_MASK)
     Pl_Err_Type(pl_type_fd_variable, word);
@@ -766,8 +765,7 @@ Pl_Fd_Prolog_To_Array_Fdv(WamWord list_word, Bool pl_var_ok)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 WamWord *
-Pl_Fd_Create_C_Frame(PlLong (*cstr_fct) (), WamWord *AF, WamWord *fdv_adr,
-		     Bool optim2)
+Pl_Fd_Create_C_Frame(CstrFct cstr_fct , WamWord *AF, WamWord *fdv_adr, Bool optim2)
 {
   WamWord *CF = CS;
 
@@ -801,7 +799,7 @@ Pl_Fd_Add_Dependency(WamWord *fdv_adr, int chain_nb, WamWord *CF)
 
   Trail_Chains_If_Necessary(fdv_adr);
 
-  Chains_Mask(fdv_adr) |= (1 << chain_nb);
+  Chains_Mask(fdv_adr) |= ((WamWord) 1 << chain_nb);
 
   chain_adr = (&Chain_Min(fdv_adr) + chain_nb);
 
@@ -823,7 +821,7 @@ Pl_Fd_Add_Dependency(WamWord *fdv_adr, int chain_nb, WamWord *CF)
 void
 Pl_Fd_Add_List_Dependency(WamWord *array, int chain_nb, WamWord *CF)
 {
-  int n = *array++;
+  PlLong n = *array++;
 
   while (n--)
     Pl_Fd_Add_Dependency((WamWord *) (*array++), chain_nb, CF);
@@ -1353,7 +1351,7 @@ Pl_Fd_After_Add_Cstr(Bool result_of_tell)
   PlULong date = DATE;		/* local copy for efficiency */
   PlULong *pdate;
   WamWord *AF;
-  PlLong (*fct) ();
+  CstrFct fct;
 
   if (!result_of_tell)
     {
@@ -1398,16 +1396,16 @@ Pl_Fd_After_Add_Cstr(Bool result_of_tell)
 		fct = Cstr_Address(CF);
 		AF = AF_Pointer(CF);
 
-		fct = (PlLong (*)()) (*fct) (AF);
+		fct = (CstrFct) (*fct) (AF);
 
-		if (fct == (PlLong (*)()) FALSE)
+		if (fct == (CstrFct) FALSE)
 		  {
 		  failure:
 		    Queue_Next_Fdv_Adr(dummy_fd_var) = BP; /* update begin of remaining queue */
 		    goto clear_queue;
 		  }
-#if 1				/* FD switch */
-		if (fct != (PlLong (*)()) TRUE)	/* FD switch case triggered */
+#if 1						/* FD switch */
+		if (fct != (CstrFct) TRUE)	/* FD switch case triggered */
 		  {
 		    if ((*fct) (AF) == FALSE)
 		      goto failure;
@@ -1690,7 +1688,7 @@ Pl_Fd_Copy_Variable0(WamWord *dst_adr, WamWord *fdv_adr)
 
   Pl_Range_Copy(Range(dst_adr), Range(fdv_adr));
 
-  size = CS - dst_adr;
+  size = (int) (CS - dst_adr);
   CS = save_CS;
 
   return size;
