@@ -6,7 +6,7 @@
  * Descr.: FD instruction implementation - header file                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -139,13 +139,15 @@
 
 	  /* Constraint Frame */
 
+typedef PlLong (*CstrFct) (WamWord *af);
+
 #define CONSTRAINT_FRAME_SIZE      3
 
 #define OFFSET_OF_OPTIM_POINTER    1	/* this offset must correspond to >>> */
 
 #define AF_Pointer(cf)             (*(WamWord **)    &(cf[0]))
 #define Optim_Pointer(cf)          (*(PlULong **)    &(cf[1]))	/* >>> this cell */
-#define Cstr_Address(cf)           (*(PlLong (**)()) &(cf[2]))
+#define Cstr_Address(cf)           (*(CstrFct *) &(cf[2]))
 
 
 
@@ -154,6 +156,7 @@
 
 #define ENV_VAR_VECTOR_MAX         "VECTORMAX"
 #define DEFAULT_VECTOR_MAX         127
+#define VECTOR_MAX_LIMIT           1000000
 
 
 #define Fd_Variable_Is_Ground(fdv_adr) (Tag_Of(FD_Tag_Value(fdv_adr)) == INT)
@@ -177,13 +180,13 @@
 
 #ifdef FD_INST_FILE
 
-WamWord pl_vec_size;
-WamWord pl_vec_max_integer;
+int pl_vec_size;
+int pl_vec_max_integer;
 
 #else
 
-extern WamWord pl_vec_size;
-extern WamWord pl_vec_max_integer;
+extern int pl_vec_size;
+extern int pl_vec_max_integer;
 
 #endif
 
@@ -216,8 +219,7 @@ WamWord *Pl_Fd_New_Variable_Range(Range *r);
 
 WamWord *Pl_Fd_New_Int_Variable(int n);
 
-WamWord *Pl_Fd_Create_C_Frame(PlLong (*cstr_fct) (), WamWord *AF,
-			      WamWord *fdv_adr, Bool optim2);
+WamWord *Pl_Fd_Create_C_Frame(CstrFct cstr_fct, WamWord *AF, WamWord *fdv_adr, Bool optim2);
 
 void Pl_Fd_Add_Dependency(WamWord *fdv_adr, int chain_nb, WamWord *CF);
 
