@@ -6,7 +6,7 @@
  * Descr.: architecture dependent features - Header file                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -39,6 +39,8 @@
 #ifndef _ARCH_DEP_H
 #define _ARCH_DEP_H
 
+#include <math.h>		/* for isnan testing */
+
 /* for signals (see try_sigaction.c) */
 
 #if defined(M_ix86_sco)
@@ -49,11 +51,15 @@
 
 
 
-#define CPP_STR1(s) #s
-#define CPP_STR(s) CPP_STR1(s)
+/* C Preprocessor utilities */
+
+#define PP_STR1(s) #s
+#define PP_STR(s) PP_STR1(s)
+
+#define PP_CAT1(x, y)   x ## y
+#define PP_CAT(x, y)    PP_CAT1(x, y)
+
 
-#define CPP_CAT1(x, y)   x ## y
-#define CPP_CAT(x, y)    CPP_CAT1(x, y)
 
 
 /* C compiler version (for more general handling see http://sourceforge.net/projects/predef) */
@@ -105,10 +111,12 @@
 
 #if defined(__GNUC__) && !defined(M_win64) /* gcc on mingw64 warns about PRIdPTR in x86_64_any.c ??? */
 #define ATTR_PRINTF(x) __attribute__((format(printf, x, x + 1)))
+#pragma GCC diagnostic ignored "-Wformat-zero-length"
 #else
 #define ATTR_PRINTF(x)
 #endif
 
+#define AVOID_UNUSED_WARNING(x) ((void)(x))
 
 
 
@@ -127,6 +135,7 @@
 
 #ifdef _MSC_VER
 #pragma warning(disable : 4996)
+/*#pragma warning(disable : 4244)*/ /* possible loss of data */
 #endif
 
 #define MAXPATHLEN                 1024
@@ -149,8 +158,10 @@
 #define strcasecmp                 stricmp
 #define strncasecmp                strnicmp
 #define spawnvp                    _spawnvp
+#ifndef isnan
 #define isnan                      _isnan
 #endif
+#endif
 
 #ifndef F_OK
 #define F_OK                       00
@@ -404,5 +415,48 @@ typedef struct _excp_lst
 
 #endif /* defined(USE_SEH) */
 
-#endif /* !_ARCH_DEP_H */
+#ifdef _WIN32
 
+/* Provided by arch_dep.c */
+
+int Pl_Win_Error_To_Errno(void);
+
+/* see unix <dirent.h> */
+
+struct dirent	     	
+{
+  long d_ino;
+  unsigned short d_reclen;
+  unsigned char d_type;
+  char d_name[MAXPATHLEN];
+};
+  
+  
+#define DT_UNKNOWN      0 /* d_type: file types.  */
+#define DT_FIFO         1
+#define DT_CHR          2
+#define DT_DIR          4
+#define DT_BLK          6
+#define DT_REG          8
+#define DT_LNK          10
+#define DT_SOCK         12
+#define DT_WHT          14
+
+struct DIR
+{
+  char *dirname;
+  struct dirent ret;          /* Used to return to caller */
+  void *handle;		      /* equivalent of windows HANDLE (avoid to include windows.h) */
+};
+
+typedef struct DIR DIR;
+  
+/* Protoypes */
+
+DIR *opendir(char *dir_path);
+struct dirent *readdir(DIR *dir);
+int closedir(DIR *dir);
+
+#endif
+
+#endif /* !_ARCH_DEP_H */
