@@ -6,7 +6,7 @@
  * Descr.: atom table management - header file                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -112,7 +112,7 @@ typedef struct			/* Atom information               */
   char *name;			/* key is <name> (the string)     */
   unsigned hash;		/* the hash code of string (name) */
   AtomProp prop;		/* associated properties          */
-  void *info;			/* an user info (used by g_var)   */
+  void *info;			/* a user info (used by g_var)    */
 }
 AtomInf;
 
