@@ -6,7 +6,7 @@
  * Descr.: general engine                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -121,7 +121,8 @@ void Pl_Call_Compiled(CodePtr codep);   /* defined in engine1.c */
 int
 Pl_Start_Prolog(int argc, char *argv[])
 {
-  int i, x;
+  int i;
+  PlLong x;
   char *p;
   void (*copy_of_pl_init_stream_supp)() = Pl_Dummy_Ptr(pl_init_stream_supp);
 #if defined(_WIN32) || defined(__CYGWIN__)
@@ -156,7 +157,7 @@ Pl_Start_Prolog(int argc, char *argv[])
           p = (char *) getenv(pl_stk_tbl[i].env_var_name);
           if (p && *p)
             {
-              sscanf(p, "%d", &x);
+              x = strtol(p, &p, 0);
               pl_stk_tbl[i].size = KBytes_To_Wam_Words(x);
             }
 #if defined(_WIN32) || defined(__CYGWIN__)
@@ -176,7 +177,7 @@ Pl_Start_Prolog(int argc, char *argv[])
       p = (char *) getenv(ENV_VAR_MAX_ATOM);
       if (p && *p)
 	{
-	  sscanf(p, "%d", &x);
+	  x = strtol(p, &p, 0);
 	  pl_max_atom = x;
 	}
 #if defined(_WIN32) || defined(__CYGWIN__)
@@ -197,10 +198,6 @@ Pl_Start_Prolog(int argc, char *argv[])
   /* must be changed to store global info (see the debugger) */
   heap_actual_start = Global_Stack;
 
-  Pl_Init_Atom();
-  Pl_Init_Pred();
-  Pl_Init_Oper();
-
   pl_le_mode = 0;	/* not compiled with linedit or deactivated (using env var) */
 
 #ifndef NO_USE_LINEDIT
@@ -208,6 +205,10 @@ Pl_Start_Prolog(int argc, char *argv[])
     pl_le_mode = (*pl_le_initialize)();
 #endif
 
+  Pl_Init_Atom();
+  Pl_Init_Pred();
+  Pl_Init_Oper();
+
   if (copy_of_pl_init_stream_supp)
     (*copy_of_pl_init_stream_supp)();
 
