@@ -6,7 +6,7 @@
  * Descr.: general header file                                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -38,7 +38,7 @@
 
 #include "gp_config.h"
 #include "pl_params.h"
-#include "hash_fct.h"
+#include "../Tools/hash_fct.h"
 #include "hash.h"
 #include "bool.h"
 #include "wam_regs.h"
