@@ -1,15 +1,15 @@
 
 #define __GNU_PROLOG__        	1
 #define __GPROLOG__           	1
-#define __GPROLOG_MINOR__     	4
-#define __GPROLOG_PATCHLEVEL__	5
-#define __GPROLOG_VERSION__   	10405
+#define __GPROLOG_MINOR__     	6
+#define __GPROLOG_PATCHLEVEL__	0
+#define __GPROLOG_VERSION__   	10600
 
 #define PROLOG_DIALECT  	"gprolog"
 #define PROLOG_NAME     	"GNU Prolog"
-#define PROLOG_VERSION  	"1.4.5"
-#define PROLOG_DATE     	"Dec  4 2016"
-#define PROLOG_COPYRIGHT	"Copyright (C) 1999-2015 Daniel Diaz"
+#define PROLOG_VERSION  	"1.6.0"
+#define PROLOG_DATE     	"Sep 19 2025"
+#define PROLOG_COPYRIGHT	"Copyright (C) 1999-2025 Daniel Diaz"
 
 #define PL_MIN_INTEGER		-1152921504606846976
 #define PL_MAX_INTEGER		1152921504606846975
