@@ -6,7 +6,7 @@
  * Descr.: C Compiler options and WAM Configuration                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -61,7 +61,7 @@
 
 #define FILE_WAM_STACKS_H          "wam_stacks.h"
 
-#define FILE_GPROLOG_CST_H         "gprolog_cst.h"
+#define FILE_GPROLOG_CST_H         PROLOG_DIALECT "_cst.h"
 
 
 
@@ -146,7 +146,7 @@ char *Read_Identifier(char *s, int fail_if_error, char **end);
 
 char *Read_String(char *s, char **end);
 
-int Pl_Read_Integer(char *s, char **end);
+int Read_Integer(char *s, char **end);
 
 void Generate_Regs(FILE *f, FILE *g);
 
@@ -154,7 +154,7 @@ void Generate_Tags(FILE *f, FILE *g);
 
 void Generate_Stacks(FILE *f, FILE *g);
 
-void Pl_Fatal_Error(char *format, ...);
+void Fatal_Error(char *format, ...);
 
 
 
@@ -177,26 +177,25 @@ main(void)
 
   if (WORD_SIZE != (sizeof(PlLong) * 8))
     {
-      printf("Bad Value of WORD_SIZE - should be %d\n",
-             (int) (sizeof(PlLong) * 8));
+      printf("Bad Value of WORD_SIZE - should be %d\n", (int) (sizeof(PlLong) * 8));
 
       return 1;
     }
 
   if ((fw_r = fopen(FILE_WAM_REGS_H, "wb")) == NULL)
-    Pl_Fatal_Error("cannot open %s", FILE_WAM_REGS_H);
+    Fatal_Error("cannot open %s", FILE_WAM_REGS_H);
 
   fprintf(fw_r, "/* this file is automatically generated by pl_config.c */\n");
   fprintf(fw_r, "\n#include \"gp_config.h\"\n\n");
 
 
   if ((fw_s = fopen(FILE_WAM_STACKS_H, "wb")) == NULL)
-    Pl_Fatal_Error("cannot open %s", FILE_WAM_STACKS_H);
+    Fatal_Error("cannot open %s", FILE_WAM_STACKS_H);
 
   fprintf(fw_s, "/* this file is automatically generated by pl_config.c */\n");
 
   if ((fg_c = fopen(FILE_GPROLOG_CST_H, "wb")) == NULL)
-    Pl_Fatal_Error("cannot open %s", FILE_GPROLOG_CST_H);
+    Fatal_Error("cannot open %s", FILE_GPROLOG_CST_H);
 #if 0
   fprintf(fg_c, "/* this file is automatically generated by pl_config.c */\n");
 #endif
@@ -376,6 +375,11 @@ Write_C_Compiler_Info(void)
   fprintf(fw_r, "#define CFLAGS_REGS\t\t\"");
   for (i = 0; i < nb_of_used_mach_regs; i++)
     {
+
+#ifdef _MSC_VER
+#pragma warning(disable : 4474)	/* too many args for fprintf */
+#endif
+
       fprintf(fw_r, CFLAGS_PREFIX_REG, used_mach_reg[i].mach_reg_name);
       fputc(' ', fw_r);
     }
@@ -398,10 +402,10 @@ Generate_Archi(void)
   char *p1, *p2;
 
   if ((f = fopen(FILE_WAM_ARCHI_DEF, "rt")) == NULL)
-    Pl_Fatal_Error("cannot open %s", FILE_WAM_ARCHI_DEF);
+    Fatal_Error("cannot open %s", FILE_WAM_ARCHI_DEF);
 
   if ((g = fopen(FILE_WAM_ARCHI_H, "wb")) == NULL)
-    Pl_Fatal_Error("cannot open %s", FILE_WAM_ARCHI_H);
+    Fatal_Error("cannot open %s", FILE_WAM_ARCHI_H);
 
   while (!feof(f) && fgets(str, sizeof(str), f))
     {
@@ -415,8 +419,7 @@ Generate_Archi(void)
       p1 = Read_Identifier(str + 1, 1, &p2);
 
       if (strcmp(p1, "begin") != 0)
-        Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s",
-                    save_str);
+        Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
 
       p1 = Read_Identifier(p2 + 1, 1, &p2);
 
@@ -439,7 +442,7 @@ Generate_Archi(void)
         }
 
 
-      Pl_Fatal_Error("Syntax error: unknown section in: %s", save_str);
+      Fatal_Error("Syntax error: unknown section in: %s", save_str);
     }
 
   fclose(f);
@@ -463,7 +466,7 @@ Read_Identifier(char *s, int fail_if_error, char **end)
   if (!isalpha(**end))
     {
       if (fail_if_error)
-        Pl_Fatal_Error("Syntax error: identifier expected in: %s", save_str);
+        Fatal_Error("Syntax error: identifier expected in: %s", save_str);
       else
         return NULL;
     }
@@ -473,8 +476,7 @@ Read_Identifier(char *s, int fail_if_error, char **end)
   while (isalnum(**end) || **end == '_');
 
   if (!isspace(**end))
-    Pl_Fatal_Error("Syntax error: space expected after identifier in: %s",
-                save_str);
+    Fatal_Error("Syntax error: space expected after identifier in: %s", save_str);
 
   **end = '\0';
 
@@ -495,11 +497,11 @@ Read_String(char *s, char **end)
     s++;
 
   if (*s != '"')
-    Pl_Fatal_Error("Syntax error: string expected in: %s", save_str);
+    Fatal_Error("Syntax error: string expected in: %s", save_str);
 
   for(*end = s + 1; **end != '"'; (*end)++)
     if (**end == '\0')
-      Pl_Fatal_Error("Syntax error: string expected in: %s", save_str);
+      Fatal_Error("Syntax error: string expected in: %s", save_str);
 
   (*end)++;
   **end = '\0';
@@ -511,20 +513,29 @@ Read_String(char *s, char **end)
 
 
 /*-------------------------------------------------------------------------*
- * PL_READ_INTEGER                                                         *
+ * READ_INTEGER                                                            *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 int
-Pl_Read_Integer(char *s, char **end)
+Read_Integer(char *s, char **end)
 {
+#if 1
   int x = 0;
 
+  x = (int) strtol(s, end, 0);
+  if (*end == s)
+    Fatal_Error("Syntax error: integer expected in: %s", save_str);
+  if (!isspace(**end))
+    Fatal_Error("Syntax error: space expected after integer in: %s", save_str);
+  **end = '\0';
+  return x;
+#else
   while (isspace(*s))
     s++;
 
   *end = s;
   if (!isdigit(**end))
-    Pl_Fatal_Error("Syntax error: integer expected in: %s", save_str);
+    Fatal_Error("Syntax error: integer expected in: %s", save_str);
 
   do
     {
@@ -534,10 +545,10 @@ Pl_Read_Integer(char *s, char **end)
   while (isdigit(**end) || **end == '_');
 
   if (!isspace(**end))
-    Pl_Fatal_Error("Syntax error: space expected after identifier in: %s",
-                save_str);
+    Fatal_Error("Syntax error: space expected after integer in: %s", save_str);
   **end = '\0';
   return x;
+#endif
 }
 
 
@@ -560,7 +571,7 @@ Generate_Regs(FILE *f, FILE *g)
 {
   char *p1, *p2;
   static char str[STR_LENGTH];
-  char str_base[32] = "";
+  char str_base[32 + STR_LENGTH] = ""; /* 32 is enoungh but to avoid gcc sprintf warning */
   char *used_regs[] = M_USED_REGS;
   char **p = used_regs;
   RegInf reg[15][50];
@@ -578,7 +589,7 @@ Generate_Regs(FILE *f, FILE *g)
   for (;;)
     {
       if (feof(f) || !fgets(str, sizeof(str), f))
-        Pl_Fatal_Error("Syntax error: end expected for @begin reg");
+        Fatal_Error("Syntax error: end expected for @begin reg");
 
       if (*str != '@')
         {
@@ -598,7 +609,7 @@ Generate_Regs(FILE *f, FILE *g)
           p1 = Read_Identifier(p2 + 1, 0, &p2);
           if (!p1)
             {
-              i = Pl_Read_Integer(p2, &p2);
+              i = Read_Integer(p2, &p2);
               p1 = str;
               sprintf(p1, "%d", i);
             }
@@ -609,14 +620,14 @@ Generate_Regs(FILE *f, FILE *g)
 
       if (strcmp(p1, "reg") == 0)
         {
-          i = Pl_Read_Integer(p2 + 1, &p2);
+          i = Read_Integer(p2 + 1, &p2);
           strcpy(reg[i][nb_reg[i]].type, Read_Identifier(p2 + 1, 1, &p2));
           strcpy(reg[i][nb_reg[i]].name, Read_Identifier(p2 + 1, 1, &p2));
           nb_reg[i]++;
           continue;
         }
 
-      Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
+      Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
     }
 
 
@@ -862,7 +873,7 @@ Generate_Tags(FILE *f, FILE *g)
   for (;;)
     {
       if (feof(f) || !fgets(str, sizeof(str), f))
-        Pl_Fatal_Error("Syntax error: end expected for @begin tag");
+        Fatal_Error("Syntax error: end expected for @begin tag");
 
       if (*str != '@')
         {
@@ -889,16 +900,16 @@ Generate_Tags(FILE *f, FILE *g)
           else if (strcmp(p1, "address") == 0)
             tag[nb_tag].type = ADDRESS;
           else
-            Pl_Fatal_Error("Syntax error: wrong tag type in: %s", save_str);
+            Fatal_Error("Syntax error: wrong tag type in: %s", save_str);
 
-          tag[nb_tag].value = Pl_Read_Integer(p2 + 1, &p2);
+          tag[nb_tag].value = Read_Integer(p2 + 1, &p2);
           if (tag[nb_tag].value > max_value)
             max_value = tag[nb_tag].value;
           nb_tag++;
           continue;
         }
 
-      Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
+      Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
     }
 
 
@@ -911,7 +922,7 @@ Generate_Tags(FILE *f, FILE *g)
 #else
   max_value++;
   if (max_value < nb_tag)
-    Pl_Fatal_Error("There is an invalid tag value (repetition ?)\n");
+    Fatal_Error("There is an invalid tag value (repetition ?)\n");
 
   for (tag_size = 0; (1 << tag_size) < max_value; tag_size++)
     ;
@@ -983,13 +994,13 @@ Generate_Tags(FILE *f, FILE *g)
  * optimizes things like Tag_Address(2, H + 1) with only 1 instruction (+ 6)
  * instead of 2 (1 for + 4, 1 for | TAG_STC_MASK)
  */
-  fprintf(g, "#define Tag_Address(tm, v)  \t((PlLong) (v) + (tm))\n");
+  fprintf(g, "#define Tag_Address(tm, v)\t((PlLong) (v) + (tm))\n");
 
   fprintf(g, "\n");
-  fprintf(g, "#define UnTag_Long_Int(w)    \t((PlLong) ((w) << %d) >> %d)\n",
+  fprintf(g, "#define UnTag_Long_Int(w) \t((PlLong) ((w) << %d) >> %d)\n",
           tag_size_high, tag_size);
 
-  fprintf(g, "#define UnTag_Short_Uns(w)\tUnTag_Long_Int(w)\n");
+  fprintf(g, "#define UnTag_Short_Uns(w)\t((int) UnTag_Long_Int(w))\n");
 
   fprintf(g, "#define UnTag_Address(w)  \t((WamWord *) ((w) & VALUE_MASK))\n");
 
@@ -1039,7 +1050,7 @@ Generate_Tags(FILE *f, FILE *g)
 
         case SHORT_UNS:
           if (tag[i].value <= 3)
-            fprintf(g, "((PlULong) (w) >> %d)\n", tag_size_low);
+            fprintf(g, "((int) ((PlULong) (w) >> %d))\n", tag_size_low);
           else
             fprintf(g, "UnTag_Short_Uns(w)\n");
           break;
@@ -1118,7 +1129,7 @@ Generate_Stacks(FILE *f, FILE *g)
   for (;;)
     {
       if (feof(f) || !fgets(str, sizeof(str), f))
-        Pl_Fatal_Error("Syntax error: end expected for @begin stack");
+        Fatal_Error("Syntax error: end expected for @begin stack");
 
       if (*str != '@')
         {
@@ -1137,16 +1148,15 @@ Generate_Stacks(FILE *f, FILE *g)
         {
           strcpy(stack[nb_stack].name, Read_Identifier(p2 + 1, 1, &p2));
           strcpy(stack[nb_stack].desc, Read_String(p2 + 1, &p2));
-          i = Pl_Read_Integer(p2 + 1, &p2);
-          stack[nb_stack].def_size = i * 1024 / sizeof(PlLong);
+          i = Read_Integer(p2 + 1, &p2);
+          stack[nb_stack].def_size = (i * 1024 + sizeof(PlLong) - 1) / sizeof(PlLong);
 
-          strcpy(stack[nb_stack].top_macro,
-                 Read_Identifier(p2 + 1, 1, &p2));
+          strcpy(stack[nb_stack].top_macro, Read_Identifier(p2 + 1, 1, &p2));
           nb_stack++;
           continue;
         }
 
-      Pl_Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
+      Fatal_Error("Syntax error: incorrect @ declaration in: %s", save_str);
     }
 
 
@@ -1182,9 +1192,9 @@ Generate_Stacks(FILE *f, FILE *g)
   fprintf(fw_s, "  char *name;\n");
   fprintf(fw_s, "  char *desc;\n");
   fprintf(fw_s, "  char *env_var_name;\n");
-  fprintf(fw_s, "  PlLong *p_def_size;\t/* used for fixed_sizes */\n");
-  fprintf(fw_s, "  int default_size; \t/* in WamWords */\n");
-  fprintf(fw_s, "  int size;         \t/* in WamWords */\n");
+  fprintf(fw_s, "  PlLong *p_def_size;  \t/* used for fixed_sizes */\n");
+  fprintf(fw_s, "  PlLong default_size; \t/* in WamWords */\n");
+  fprintf(fw_s, "  PlLong size;         \t/* in WamWords */\n");
   fprintf(fw_s, "  WamWord *stack;\n");
   fprintf(fw_s, "}InfStack;\n\n\n");
 
@@ -1226,11 +1236,11 @@ Generate_Stacks(FILE *f, FILE *g)
 
 
 /*-------------------------------------------------------------------------*
- * PL_FATAL_ERROR                                                          *
+ * FATAL_ERROR                                                             *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Fatal_Error(char *format, ...)
+Fatal_Error(char *format, ...)
 {
   va_list arg_ptr;
 
