@@ -6,7 +6,7 @@
  * Descr.: predicate table management                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -131,8 +131,7 @@ Pl_Init_Pred(void)
  * byte-code support.                                                      *
  *-------------------------------------------------------------------------*/
 PredInf * FC
-Pl_Create_Pred(int func, int arity, int pl_file, int pl_line, int prop,
-	       PlLong *codep)
+Pl_Create_Pred(int func, int arity, int pl_file, int pl_line, int prop, PlLong *codep)
 {
   PredInf pred_info;
   PredInf *pred;
