@@ -6,7 +6,7 @@
  * Descr.: predicate table management - header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -66,7 +66,7 @@ typedef struct			/* Predicate information          */
   int pl_line;			/* pl file line of its definition */
   int prop;			/* predicate props (cf BipsPl)    */
   PlLong *codep;		/* compiled code                  */
-  PlLong *dyn;			/* dynamic info (cf BipsPl)       */
+  void *dyn;			/* dynamic info (cf BipsPl)       */
 }
 PredInf;
 
