@@ -6,7 +6,7 @@
  * Descr.: stack hardware overflow detection (SIGSEGV)                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2021 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -81,6 +81,11 @@
 #endif
 #endif
 
+#ifdef __GNUC__
+#if defined(__clang__) && __clang_major__ >= 14
+#pragma GCC diagnostic ignored "-Wdeprecated-non-prototype"
+#endif
+#endif
 
 
 #if 0
@@ -110,9 +115,9 @@
 #define ERR_CANNOT_FREE            "VirtualFree failed : %" PL_FMT_u
 #define ERR_CANNOT_PROTECT         "VirtualProtect failed : %" PL_FMT_u
 
-#define ERR_STACK_OVERFLOW_ENV     "%s stack overflow (size: %d Kb, reached: %d Kb, environment variable used: %s)"
+#define ERR_STACK_OVERFLOW_ENV     "%s stack overflow (size: %" PL_FMT_d " Kb, reached: %" PL_FMT_d " Kb, environment variable used: %s)"
 
-#define ERR_STACK_OVERFLOW_NO_ENV  "%s stack overflow (size: %d Kb, reached: %d Kb - fixed size)"
+#define ERR_STACK_OVERFLOW_NO_ENV  "%s stack overflow (size: %" PL_FMT_d " Kb, reached: %" PL_FMT_d " Kb - fixed size)"
 
 
 
@@ -177,12 +182,11 @@ getpagesize(void)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void *
-Virtual_Mem_Alloc(void *addr, int length)
+Virtual_Mem_Alloc(void *addr, size_t length)
 {
 #if defined(_WIN32) || defined(__MSYS__)
 
-  addr = (void *) VirtualAlloc(addr, length,
-			       MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
+  addr = (void *) VirtualAlloc(addr, length, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
 
 #elif defined(HAVE_MMAP)
 
@@ -193,7 +197,7 @@ Virtual_Mem_Alloc(void *addr, int length)
     fd = open("/dev/zero", 0);
 
   if (fd == -1)
-    Pl_Fatal_Error(ERR_CANNOT_OPEN_DEV0, Pl_M_Sys_Err_String(-1));
+    Pl_Fatal_Error(ERR_CANNOT_OPEN_DEV0, Pl_M_Sys_Err_String(errno));
 #endif /* !MAP_ANON */
 
   addr = (void *) mmap((void *) addr, length, PROT_READ | PROT_WRITE,
@@ -228,7 +232,7 @@ Virtual_Mem_Alloc(void *addr, int length)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-Virtual_Mem_Free(void *addr, int length)
+Virtual_Mem_Free(void *addr, size_t length)
 {
 #if defined(_WIN32) || defined(__MSYS__)
 
@@ -238,7 +242,7 @@ Virtual_Mem_Free(void *addr, int length)
 #elif defined(HAVE_MMAP)
 
   if (munmap((void *) addr, length) == -1)
-    Pl_Fatal_Error(ERR_CANNOT_UNMAP, Pl_M_Sys_Err_String(-1));
+    Pl_Fatal_Error(ERR_CANNOT_UNMAP, Pl_M_Sys_Err_String(errno));
 
 #else
 
@@ -255,7 +259,7 @@ Virtual_Mem_Free(void *addr, int length)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static void
-Virtual_Mem_Protect(void *addr, int length)
+Virtual_Mem_Protect(void *addr, size_t length)
 {
   WamWord *end = (WamWord *) addr;
 #if defined(_WIN32) || defined(__MSYS__)
@@ -270,7 +274,7 @@ Virtual_Mem_Protect(void *addr, int length)
   if (mprotect((void *) addr, length, PROT_NONE) == -1)
 #  endif
     if (munmap((void *) addr, length) == -1)
-      Pl_Fatal_Error(ERR_CANNOT_UNMAP, Pl_M_Sys_Err_String(-1));
+      Pl_Fatal_Error(ERR_CANNOT_UNMAP, Pl_M_Sys_Err_String(errno));
 
 #endif
 
@@ -292,7 +296,7 @@ Virtual_Mem_Protect(void *addr, int length)
 void
 Pl_Allocate_Stacks(void)
 {
-  unsigned length = 0, stk_sz;
+  size_t length = 0, stk_sz;
   WamWord *addr;
   int i;
   WamWord *addr_to_try[] = {
@@ -735,8 +739,8 @@ Stack_Overflow_Err_Msg(int stk_nb)
 {
   InfStack *s = pl_stk_tbl + stk_nb;
   char *var = s->env_var_name;
-  int size = s->size;
-  int usage = Stack_Top(stk_nb) - s->stack;
+  PlLong size = s->size;
+  PlLong usage = (Stack_Top(stk_nb) - s->stack);
   static char msg[256];
 
   if (s->stack == Global_Stack)
