@@ -6,7 +6,7 @@
  * Descr.: Wam architecture definition - description file                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2015 Daniel Diaz                                     *
+ * Copyright (C) 1999-2025 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -244,10 +244,10 @@ extern char *pl_reg_tbl[];
 
 #define Tag_Long_Int(tm, v)  	((((PlLong) ((v) << 3)) >> 0) | (tm))
 #define Tag_Short_Uns(tm, v)	(((PlLong) (v) << 3) + (tm))
-#define Tag_Address(tm, v)  	((PlLong) (v) + (tm))
+#define Tag_Address(tm, v)	((PlLong) (v) + (tm))
 
-#define UnTag_Long_Int(w)    	((PlLong) ((w) << 0) >> 3)
-#define UnTag_Short_Uns(w)	UnTag_Long_Int(w)
+#define UnTag_Long_Int(w) 	((PlLong) ((w) << 0) >> 3)
+#define UnTag_Short_Uns(w)	((int) UnTag_Long_Int(w))
 #define UnTag_Address(w)  	((WamWord *) ((w) & VALUE_MASK))
 
 
@@ -265,7 +265,7 @@ extern char *pl_reg_tbl[];
 #define UnTag_REF(w)  		((WamWord *) (w))
 #define UnTag_LST(w)  		UnTag_Address(w)
 #define UnTag_STC(w)  		UnTag_Address(w)
-#define UnTag_ATM(w)  		((PlULong) (w) >> 3)
+#define UnTag_ATM(w)  		((int) ((PlULong) (w) >> 3))
 #define UnTag_FLT(w)  		UnTag_Address(w)
 #define UnTag_FDV(w)  		UnTag_Address(w)
 #define UnTag_INT(w)  		UnTag_Long_Int(w)
